#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon ':all';
use MyPrintwrap ':all';


my $level = 2;
print "  % generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level);
}
sub scale {
  my ($x,$y) = @_;
  return ("{($x)/2}", "{($y)*sqrt(3)/2}");
}

{
  my ($x,$y) = $path->n_to_xy($n_hi);
  ($x,$y) = transform($x,$y);
  ($x,$y) = scale($x,$y);
  print "  \\node (start) at (0,0) [my dot];\n";
  print "  \\node (end) at ($x,$y) [my dot];\n";
}

$Printwrap = 2;
printwrap("\\draw (start)");
$Printwrap = 4;
foreach my $n ($n_lo+1 .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  ($x,$y) = transform($x,$y);

  my ($sdx,$sdy) = $path->n_to_dxdy($n-1);
  my ($tdx,$tdy) = $path->n_to_dxdy($n);
  ($sdx,$sdy) = transform($sdx,$sdy);
  ($tdx,$tdy) = transform($tdx,$tdy);

  my $sx = $x . chamfer(-$sdx);
  my $sy = $y . chamfer(-$sdy);
  my $tx = $x . chamfer($tdx);
  my $ty = $y . chamfer($tdy);
  ($sx,$sy) = scale($sx,$sy);
  ($tx,$ty) = scale($tx,$ty);
  printwrap("--($sx,$sy)");
  printwrap("--($tx,$ty)");
}
printwrap("--(end);\n");

print "  % end generated\n";
exit 0;
