#!/usr/bin/perl -w

# Copyright 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.010;
use strict;
use warnings;
use FindBin;
use Math::BaseCnv 'cnv';
use Math::PlanePath::TerdragonCurve 118; # v.118 for xyxy_to_n_either()

BEGIN { unshift @INC, $FindBin::Bin }
use MyLineSegments;
use MyPrintwrap ':all';


print "  % generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::TerdragonCurve->new;

$Printwrap = 2;
my $s = 3**4;
my $n_lo = 2*$s-1;
my $n_hi = 373;

foreach my $i (0 .. 4) {
  my ($x,$y) = $path->n_to_xy($i*$s);
  print "  \\node (p$i) at ($x,$y) [my dot];\n";
}
{
  my ($x,$y) = $path->n_to_xy($n_lo);
  print "  \\coordinate (lo) at ($x,$y);\n";
}
{
  my ($x,$y) = $path->n_to_xy($n_hi);
  print "  \\coordinate (hi) at ($x,$y);\n";
}

my $segments = MyLineSegments->new;
foreach my $n (0 .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  my ($x2,$y2) = ($x+$dx, $y+$dy);
  $segments->add($x,$y, $x2,$y2);
}

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");

print "% GP-Test-Last  EpredTwelve($n_hi) == 1\n";
print "  % end generated\n";
exit 0;
