#!/usr/bin/perl -w

# Copyright 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.010;
use strict;
use warnings;
use FindBin;
use Math::NumSeq::PlanePathTurn;
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon 'xy_to_node_name';
use MyPrintwrap ':all';


my $x_min = -50; my $x_max = 50;
my $y_min = -25; my $y_max = 25;
my $xy_mod = 2;

print "  % generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::TerdragonCurve->new (arms => 6);
my $seq = Math::NumSeq::PlanePathTurn->new(planepath_object => $path,
                                           turn_type => 'LSR');
my %xy_to_turn;
foreach (1 .. 10000) {
  my ($n,$turn) = $seq->next;
  my $arm = $n % 6;
  if ($arm % 2) { $turn = -$turn; }
  my ($x,$y) = $path->n_to_xy($n);
  my $str = ($turn == 1 ? 'L'
             : $turn == -1 ? 'R'
             : die);
  my $key = "$x,$y"; # ($x % $xy_mod).','.($y % $xy_mod);
  if (defined $xy_to_turn{$key}) {
    $xy_to_turn{$key} eq $str or die;
  }
  $xy_to_turn{$key} = $str;
}

$Printwrap = 2;
foreach my $y ($y_min .. $y_max) {
  foreach my $x ($x_min .. $x_max) {
    my $key = "$x,$y"; # ($x % $xy_mod).','.($y % $xy_mod);
    my $str = $xy_to_turn{$key} // next;
    my $name = xy_to_node_name($x,$y);
    if ($str eq 'R') {
      printwrap(" \\pic at ($x/2,{$y*sqrt(3)/2}) [transform shape] {my hexagon pic};");
      # printwrap(" \\node ($name) at ($x,$y) [my dot];");
    }
    # printwrap(" \\node ($name) at ($x,$y) [my box] {$str};");
  }
}
printwrap_newline();

print "  % end generated\n";
exit 0;
