#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Lingua::EN::Numbers;
use List::Util 'sum';
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 4;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";
my $path = Math::PlanePath::TerdragonCurve->new;

my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  $segments->add($path->n_to_xy($n),
                 $path->n_to_xy($n+1));
}

my @count = (0,0,0);
foreach my $d (0,1,2) {

  my $shorten = sub {
    my ($x1,$y1, $x2,$y2) = @_;
    if ($x1 == 0 && $y1 == 0) {
      return "origin-$d";
    }
    if ($x1 == $x_hi && $y1 == $y_hi) {
      return "end-$d";
    }
    return "$x1,$y1";
  };

  print "  \\begin{scope}[shift=(pos-$d)]\n";
  print "    \\node (origin-$d) at (0,0) [my dot];\n";
  print "    \\node (end-$d) at ($x_hi,$y_hi) [my dot];\n";
  $Printwrap = 4;
  printwrap("\\draw");
  $Printwrap += 2;
  foreach my $line ($segments->lines) {
    my ($x1,$y1, $x2,$y2) = @$line;

    if ($y1 == $y2) {
      next unless $d == 0;
    } elsif ($y1 > $y2) {
      next unless $d == 1;
    } else {
      next unless $d == 2;
    }
    $count[$d]++;

    my $xy1 = $shorten->($x1,$y1, $x2,$y2);
    my $xy2 = $shorten->($x2,$y2, $x1,$y1);
    printwrap(" ($xy1)--($xy2)");
  }
  printwrap(";\n");
  print "  \\end{scope}\n";
}

foreach my $d (0,1,2) {
  my $name = Lingua::EN::Numbers::num2en($d);
  print "  \\newcommand{\\MyCount\u$name}{$count[$d]}\n";
}

my $total = sum(@count);
print "  \\newcommand{\\MyTotal}{$total}\n";
print "% GP-Test-Last  Lines($level) == $total\n";
print "% GP-Test-Last  Lines_by_direction_0($level) == $count[0]\n";
print "% GP-Test-Last  Lines_by_direction_1($level) == $count[1]\n";
print "% GP-Test-Last  Lines_by_direction_2($level) == $count[2]\n";

print "  % end generated\n";
exit 0;
