#!/usr/bin/perl -w

# Copyright 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use Math::PlanePath::AlternateTerdragon;

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon 'alternate_hull_and_end';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = $ARGV[0] || 6;

print "  % generated by tools/$FindBin::Script $level\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::AlternateTerdragon->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  $segments->add($path->n_to_xy($n),
                 $path->n_to_xy($n+1));
}

my ($points,$ex,$ey) = alternate_hull_and_end($level);
$ex == $x_hi or die;
$ey == $y_hi or die;

# no curve segments where the hull sides drawing overlaps
my $removed = 0;
foreach my $i (0 .. $#$points) {
  eval {
    $segments->remove(@{$points->[$i-1]}, @{$points->[$i]});
    $removed++;
  };
}
$removed == 4 or die "removed $removed";  # triangular hull sides

$Printwrap = 2;
printwrap("\\draw[my grey] ");
$Printwrap = 4;
$segments->printwrap;
printwrap(";\n");

my $num = 0;
my $dash = '';
foreach my $p (@$points) {
  my ($x,$y) = @$p;
  print "  \\node (AltP$num$dash) at ($x,$y) [my dot];\n";
  $num++;
  if ($num == 3) {
    $num = 0;
    $dash = 'dash';
  }
}

$num = 0;
$dash = '';
foreach my $p (@$points) {
  my ($x,$y) = @$p;
  my $want = ($num == 0 && !$dash ? "0"
              : $num == 0 && $dash ? "AltEnd($level)"
              : "AltP$num$dash($level)");
  print "% GP-Test-Last  $want == $x/2 + $y*sqrt3i/2\n";
  $num++;
  if ($num == 3) {
    $num = 0;
    $dash = 'dash';
  }
}

print "  \\coordinate (start) at (0,0);\n";
print "  \\coordinate (end) at ($x_hi,$y_hi);\n";

print "  % end generated\n";
exit 0;
