#!/usr/bin/perl -w

# Copyright 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use FindBin;
use List::Util 'max','min','sum';
use POSIX 'floor','ceil';
use Math::Libm 'hypot';
use Math::Trig 'rad2deg';
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()
use Math::NumSeq::PlanePathDelta;
use Memoize 'memoize';
use Math::BaseCnv 'cnv';
use Math::PlanePath::Base::Digits
  'bit_split_lowtohigh',
  'digit_join_lowtohigh', 'digit_split_lowtohigh';
use Math::PlanePath;
*_divrem_mutate = \&Math::PlanePath::_divrem_mutate;

use Math::PlanePath::GcdRationals;
BEGIN { *gcd = \&Math::PlanePath::GcdRationals::_gcd; }


use FindBin;
use lib::abs "$FindBin::Bin/../tools";
use MyTerdragon ':all';
use MyFLAT ':all';
$|=1;

# uncomment this to run the ### lines
# use Smart::Comments;



{
  # Dekking morphism sigmaodd
  # no match, even remapped

  require Math::OEIS::Grep;
  my @sigmaodd = (3,2,4,3,2,2,3,2,4,3,2,4,3,2,2,3,2,2,3,2,4,3,2,2,3,2,4,3,2);
  my $count = 0;
  foreach my $a (-1..4) {
    foreach my $b (-1..4) {
      next if $b==$a;
      foreach my $c (-1..4) {
        next if $c==$a || $c==$b;
        my @map = (undef,undef,$a,$b,$c);
        my @values = map {$map[$_]} @sigmaodd;
        ### @values
        Math::OEIS::Grep->search (array => \@values,
                                  name => "mapped $a$b$c",
                                  try_abs => 0,
                                  max_matches => 3,
                                  verbose => 0,
                                 );
        $count++;
      }
    }
  }
  print "$count attempts\n";
  exit 0;
}
{
  # dNextL_flat()

  dNextL_flat(1)->MyFLAT::digits_increment(add => -1)
    #    ->MyFLAT::reverse
    ->MyFLAT::optional_leading_0s
    ->MyFLAT::set_name('dNextLplus1 = 1')
    ->MyFLAT::minimize(verbose=>1)
    ->MyFLAT::view;
  dNextL_flat(2)->MyFLAT::digits_increment(add => -1)
    #    ->MyFLAT::reverse
    ->MyFLAT::optional_leading_0s
    ->MyFLAT::set_name('dNextLplus1 = 2')
    ->MyFLAT::minimize(verbose=>1)
    ->MyFLAT::view;
  dNextL_flat(3)->MyFLAT::digits_increment(add => -1)
    #    ->MyFLAT::reverse
    ->MyFLAT::optional_leading_0s
    ->MyFLAT::set_name('dNextLplus1 = 3')
    ->MyFLAT::minimize(verbose=>1)
    ->MyFLAT::view;

  # my $d = $f2->intersect(TurnLpred_flat());
  # $d = $d->MyFLAT::digits_increment(add => -1);
  # $d = $d->MyFLAT::set_name('dTurnLeft')
  #   ->MyFLAT::minimize(verbose=>1);
  # $d->MyFLAT::view;
  exit 0;
}

{
  # @fAltBpred_state_table

  require MyFLAT;
  my $f = MyFLAT::aref_to_FLAT_DFA(\@fAltBpred_state_table,
                                   starting => 0,
                                   name => 'fAltBpred');
  $f->MyFLAT::view;
  exit 0;
}


{
  # fAltRpred

  &fAltBpred_hullstouch_init(verbose => 1);
  my $f = $fAltRpred_hullstouch_flat;
  $f = $f->MyFLAT::prefix;

  # my $sub = AltRpred9_flat_even();
  # print "subset ",$sub->is_subset_of($f) ? 1: 0, "\n";
  # print "subset ",$f->is_subset_of($sub) ? 1: 0, "\n";
  # $f = $f->difference($sub);

  $f = $f->MyFLAT::minimize(verbose=>1);
  # $f->MyFLAT::view;

  $f = MyFLAT::FLAT_transition_split
    ($f,
     new_accepting_from => 0,
     symbols_func => sub {
       my ($n) = @_;
       return int($n/3), $n%3;
     });

  $f = $f->MyFLAT::prefix;
  $f = $f->MyFLAT::minimize(verbose=>1);
   # $f->MyFLAT::FLAT_print_tikz;
   $f->MyFLAT::FLAT_print_perl_table;
  $f->MyFLAT::view;
  exit;
}
{
  # fAltNonRpred

  # print "non ",AltNonRpred3_flat_odd()->contains('44444') ? 1 : 0, "\n";

  &fAltBpred_hullstouch_init(verbose => 1);
  my $f = $fAltNonRpred_hullstouch_flat;
  $f = $f->MyFLAT::eventually_accepting;
  $f = $f->MyFLAT::minimize(verbose=>1);
  $f->MyFLAT::view;

  # my $sub = AltNonRpred9_flat_even();
  # print "subset ",$sub->is_subset_of($f) ? 1: 0, "\n";
  # print "subset ",$f->is_subset_of($sub) ? 1: 0, "\n";
  # $f = $sub->difference($f);

  $f = $f->MyFLAT::as_nfa;
  $f = MyFLAT::FLAT_transition_split
    ($f,
     new_accepting_to => 0,
     symbols_func => sub {
       my ($n) = @_;
       return int($n/3), $n%3;
     },
    );
  $f = $f->as_dfa;
  # $f = $f->MyFLAT::eventually_accepting;

  # $f = $f->MyFLAT::minimize(verbose=>1);
  $f->MyFLAT::view;
  exit;
}

{
  &RpointPred_init(verbose => 2);

  $RpointPred_flat->MyFLAT::view;
  $RpointPred_flat->MyFLAT::minimize(verbose=>1)->MyFLAT::view;

  # foreach my $state (0,2,8) {
  #   my $conf = $RpointPred_state_to_conf[$state];
  #   $conf->print;
  #   print "boundary: ",MyTerdragon::RpointPred_is_boundary($conf),
  #     " is enc ",$MyTerdragon::RpointPred_state_is_R_enclosed[$state]||0,
  #     "\n";
  # }
  exit 0;
}


{
  # dTurnLeft states
  #
  # k=4
  # 01112 <= m < 11111
  # 1/2(3^k+1) = 1112
  # m<02222 subtract 01112
  # m==02222 return 3
  # m>=10000 chop high

  # with 3^k+1 <= 2m   is 3^k < 2m <= 3*3^k+1
  # 2 if 2m==0
  # 1 if 2m==2
  # dTurnLeft(2m- 3^k - 1) if 2m<2*3^k-2
  # 3 if 2m==2*3^k-2
  # dTurnLeft(2m - 2*3^k)) if 2m>=2*3^k        drop high 2

  # with 3^k+1 <= 2m-2   is 3^k <= 2m-2 <= 3*3^k-1
  # 2 if 2m-2 == -2
  # 1 if 2m-2 == 0
  # dTurnLeft(2m- 3^k - 1) if 2m-2<2*3^k-4
  # 3 if 2m-2 == 2*3^k-4
  # dTurnLeft(2m - 2*3^k)) if 2m >= 2*3^k-2

  foreach my $m (0 .. 160) {
    my $k = dTurnLeft_k($m);
    my $d = dTurnLeft_by_recurrence($m, verbose=>1);
    $d==3 or next;
    printf "%5s k=%d  %d  2m=%s\n",
      Math::BaseCnv::cnv($m,10,3),
        $k, $d,
        Math::BaseCnv::cnv(2*$m,10,3);
  }
  exit 0;

  sub dTurnLeft_by_recurrence {
    my ($m, %options) = @_;
    if ($options{'verbose'}) {
      print " dTurnLeft_by_recurrence m=",Math::BaseCnv::cnv($m,10,3),"\n";
    }
    for (;;) {
      if ($options{'verbose'}) {
        print "  at m=",Math::BaseCnv::cnv($m,10,3),
          "  2m=",Math::BaseCnv::cnv(2*$m,10,3),"\n";
      }
      $m >= 0 or die;
      if ($m == 0) {
        if ($options{'verbose'}) { print "  final m==0 is 2\n"; }
        return 2;
      }
      if ($m == 1) {
        if ($options{'verbose'}) { print "  final m==1 is 1\n"; }
        return 1;
      }
      my $k = dTurnLeft_k($m);
      my $pow = 3**$k;
      $m >= ($pow+1)/2 or die;
      if ($m < $pow-1)     {
        if ($options{'verbose'}) {
          print "   sub half ",Math::BaseCnv::cnv(($pow+1)/2,10,3),"\n";
        }
        $m -= ($pow+1)/2;
      } elsif ($m == $pow-1) {
        if ($options{'verbose'}) { print "  final m==3^k-1 is 3\n"; }
        return 3;
      } else {
        if ($options{'verbose'}) {
          print "   sub pow ",Math::BaseCnv::cnv($pow,10,3),"\n";
        }
        $m -= $pow;
      }
    }
  }
  sub dTurnLeft_k {
    my ($m) = @_;
    my $k = 1;
    for (my $t=1; ; $t++) {
      if ($m >= 1/2*(3**$t+1)) { $k=$t; } else { return $k; }
    }
  }
}
{
  # Triangle of Terdragon vs Alternate Terdragon

  require Math::PlanePath::AlternateTerdragon;
  my $path;
  $path = Math::PlanePath::AlternateTerdragon->new;
  $path = Math::PlanePath::TerdragonCurve->new;
  require MyLineSegments;
  my $k = 6;
  my ($n_lo, $n_hi) = $path->level_to_n_range($k);
  my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

  my $segments = MyLineSegments->new;
  foreach my $n ($n_lo .. $n_hi - 1) {
    my ($x1,$y1) = $path->n_to_xy($n);
    my ($x2,$y2) = $path->n_to_xy($n+1);
    foreach (1..3) {
      $segments->add($x1,$y1, $x2,$y2);
      ($x1,$y1) = xy_rotate_plus120($x1,$y1);
      ($x2,$y2) = xy_rotate_plus120($x2,$y2);
      $x1 += $x_hi; $y1 += $y_hi;
      $x2 += $x_hi; $y2 += $y_hi;
    }
  }

  require MyPrintwrap;
  $MyPrintwrap::Printwrap = 2;
  MyPrintwrap::printwrap("\\draw ");
  $MyPrintwrap::Printwrap += 2;
  $segments->printwrap;
  MyPrintwrap::printwrap(";\n");

  exit 0;
}
{
  # Alternate Terdragon AltRpred Base-9

  my $f = AltRpred9_flat();
  $f->MyFLAT::view;
  $f = $f->MyFLAT::reverse->MyFLAT::minimize(verbose=>1);
  $f->MyFLAT::view;
  exit 0;

  # my ($m,$e,$r) = $f->add_states(3);
  # $f->set_accepting($m,$e,$r);
  # $f->add_transition($m,1, '0');
  # $f->add_transition($e,2, '1');
  # $f->add_transition($r,0, '2');

  $f = $f->MyFLAT::minimize(verbose=>1);
  # $f->MyFLAT::view;

  my $r = $f->MyFLAT::reverse->MyFLAT::minimize(verbose=>1);

  $r = $r->MyFLAT::FLAT_rename(symbols_func => sub {
                                 my ($n) = @_;
                                 reverse sprintf '%02d', Math::BaseCnv::cnv($n,10,3);
                               });
  split_transitions($r);
  $r = $r->MyFLAT::minimize(verbose=>1);
  $r->set_accepting(1..4);
  $r = $r->MyFLAT::minimize(verbose=>1);
  $r->MyFLAT::view;

  # 0,4,5 low 0,1,2
  # 6,7,8 low 0,1,2
  # 1,2,3 low 1,2,0

  exit 0;

  # FIXME: is this right?
  sub split_transitions {
    my ($fa, $n, %options) = @_;
    my @alphabet = $fa->alphabet;
    my $num_symbols = scalar(@alphabet);

    foreach my $from_state ($fa->get_states) {
      foreach my $to_state ($fa->successors($from_state)) {
        my @labels = MyFLAT::FLAT_get_transition_labels($fa,$from_state,$to_state);
        $fa->remove_transition($from_state,$to_state);
        foreach my $symbol (@labels) {
          my $new_from_state = $from_state;
          foreach my $pos (0 .. length($symbol)-2) {
            my ($new_to_state) = $fa->add_states(1);
            $fa->add_transition($new_from_state, $new_to_state,
                                substr($symbol,$pos,1));
            $new_from_state = $new_to_state;
          }
          $fa->add_transition($new_from_state, $to_state,
                              substr($symbol,-1,1));
        }
      }
    }
  }
}
{
  # Alternate Terdragon AltRpred3 vs path

  AltRpred3_flat()->MyFLAT::view;
  AltRpred3_flat()->MyFLAT::reverse->MyFLAT::view;

  require Math::PlanePath::AlternateTerdragon;
  my $path = Math::PlanePath::AlternateTerdragon->new;
  foreach my $k (0 .. 8) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my $AltRpred_by_path = sub {
      my ($n,$k) = @_;
      ### AltRpred_by_path(): "$n  k=$k"
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my ($rx,$ry) = xy_rotate_minus60($dx,$dy);
      $n = $path->xyxy_to_n_either($x,$y, $x+$rx,$y+$ry);
      ### $n
      if (! defined $n || $n >= $n_hi) { return 1; }
      $n = $path->xyxy_to_n_either($x+$dx,$y+$dy, $x+$rx,$y+$ry);
      if (! defined $n || $n >= $n_hi) { return 1; }
      return 0;
    };
    my $bad = 0;
    foreach my $n (0 .. 3**$k-1) {
      my $str = Math::BaseCnv::cnv($n,10,3);
      if ($k==0 && $n==0) { $str = ''; }
      $str = sprintf '%0*s', $k, $str;
      my $by_flat = AltRpred3_flat()->contains($str);
      my $by_path = $AltRpred_by_path->($n);
      if ($by_flat != $by_path) {
        my $n3 = cnv($n,10,3);
        print "k=$k n=$n [$n3]  by_flat=$by_flat  by_path=$by_path  str=$str\n";
        die if ++$bad > 10;
      }
    }
  }
  print "ok\n";
  exit 0;
}

{
  # Rpred low to high

  Rpred_flat()->MyFLAT::view;
  Rpred_flat()->MyFLAT::reverse->MyFLAT::view;
  exit 0;
}

{
  # Alternate Terdragon AltRpred9 vs path

  require Math::PlanePath::AlternateTerdragon;
  my $path = Math::PlanePath::AlternateTerdragon->new;
  foreach my $k (0,2,4,8) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my $AltRpred_by_path = sub {
      my ($n) = @_;
      ### AltRpred_by_path(): "$n  k=$k"
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my ($rx,$ry) = xy_rotate_minus60($dx,$dy);
      $n = $path->xyxy_to_n_either($x,$y, $x+$rx,$y+$ry);
      ### $n
      if (! defined $n || $n >= $n_hi) { return 1; }
      $n = $path->xyxy_to_n_either($x+$dx,$y+$dy, $x+$rx,$y+$ry);
      if (! defined $n || $n >= $n_hi) { return 1; }
      return 0;
    };
    my $bad = 0;
    foreach my $n (0 .. 3**$k-1) {
      my $str = Math::BaseCnv::cnv($n,10,9);
      if ($k==0 && $n==0) { $str = ''; }
      $str = sprintf '%0*s', $k/2, $str;
      my $by_flat = AltRpred9_flat()->contains($str);
      my $by_path = $AltRpred_by_path->($n);
      if ($by_flat != $by_path) {
        my $n3 = cnv($n,10,3);
        print "k=$k n=$n [$n3]  by_flat=$by_flat  by_path=$by_path  digits $str\n";
        die if ++$bad > 10;
      }
    }
  }
  print "ok\n";
  exit 0;
}

{
  # Alternate Terdragon graph diameter ends

  unshift @INC, "../gmaker/devel/lib";
  require Graph::Maker::PlanePath;
  foreach my $k (8, 0 .. 6) {
    my $graph = Graph::Maker->new('planepath',
                                  undirected=>1,
                                  level=>$k,
                                  planepath=>'AlternateTerdragon');
    my $num_vertices = $graph->vertices;
    my $num_edges = $graph->edges;
    my $apsp = $graph->APSP_Floyd_Warshall;
    my $diameter = $apsp->diameter;
    print "k=$k vertices $num_vertices edges $num_edges  diameter $diameter\n";

    my $count_total = 0;
    my $count_diams = 0;
    my $for = $apsp->for_shortest_paths
      (sub {
         my ($t, $u,$v, $n) = @_;
         my $length = $t->path_length($u,$v);
         if ($u le $v && $length == $diameter) {
           if ($k <= 2) {
             print "  at $u to $v\n";
           }
           $count_diams++;
         }
         $count_total++;
       });
    print "  total paths $count_total diams $count_diams\n";
  }
  exit 0;
}
{
  # Alternate Terdragon graph diameter
  # not in OEIS: 1, 3, 3, 7, 9, 19, 27,
  #
  # even levels 3^(k/2) straight line
  # odd levels cut corner to get to middle bridge ...

  unshift @INC, "../gmaker/devel/lib";
  require Graph::Maker::PlanePath;
  foreach my $k (0 .. 6) {
    my $graph = Graph::Maker->new('planepath',
                                  undirected=>1,
                                  level=>$k,
                                  planepath=>'AlternateTerdragon');
    my $apsp = $graph->APSP_Floyd_Warshall;
    my $diameter = $apsp->diameter;
    print "$diameter,\n";
  }
  exit 0;
}
{
  # Alternate Terdragon Contiguous Lines
  #

  require Math::PlanePath::AlternateTerdragon;
  my $path = Math::PlanePath::AlternateTerdragon->new;
  foreach my $k (0 .. 8) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my @counts = (0,0,0);
    foreach my $n (0 .. $n_hi-1) {
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my $n = $path->xyxy_to_n_either($x,$y, $x-$dx,$y-$dy);
      next if defined $n && $n <= $n_hi;
      my $dir3 = dxdy_to_dir3($dx,$dy);
      $counts[$dir3]++;
    }
    print join(',',@counts)," total ",sum(@counts),"\n";
  }
  exit 0;
}
{
  # alternate terdragon ends path count, by graph SPT
  #

  # not in OEIS: 1, 1, 1, 4, 1, 2116, 1, 11761470250000,
  # odds
  # not in OEIS: 1, 4, 2116, 11761470250000,

  require MyGraphs;
  unshift @INC, "../gmaker/devel/lib";
  require Graph::Maker::PlanePath;

  require Math::PlanePath::AlternateTerdragon;
  my $path = Math::PlanePath::AlternateTerdragon->new;
  require Math::BigInt;

  for (my $k = 1; $k <= 10; $k+=2) {
    my $graph = Graph::Maker->new('planepath',
                                  undirected => 1,
                                  level => $k,
                                  planepath_object => $path);
    my $root = '0,0';
    my $sptg = $graph->SPT_Dijkstra (first_root => $root);
    MyGraphs::Graph_SPT_counts($graph,$sptg,
                               one => Math::BigInt->new(1));

    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    my $end = "$x_hi,$y_hi";

    print $sptg->get_vertex_attribute($end,'count'),",\n";
  }
  exit 0;
}
{
  # alternate terdragon ends path count, by path

  # unsquared
  # not in OEIS: 1, 2, 46, 3429500, 15601119091200326339620, 21925707536782091342151987853272377132855313016376485388672569530311000
  # GP-Test  46^2 == 2116
  # GP-Test  3429500^2 == 11761470250000

  require Math::PlanePath::AlternateTerdragon;
  my $path = Math::PlanePath::AlternateTerdragon->new;
  my $verbose = 0;
  for (my $k = 1; $k <= 13; $k += 2) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my $has_edge = sub {
      my ($x1,$y1, $x2,$y2) = @_;
      my $n = $path->xyxy_to_n_either($x1,$y1,$x2,$y2);
      return defined $n && $n < $n_hi;
    };
    my ($x_end, $y_end) = $path->n_to_xy($n_hi);
    my $y_target = int($y_end/2);
    my $x_target = int($x_end/2) + 1;
    print "k=$k n_hi=$n_hi end $x_end,$y_end target $x_target,$y_target\n";

    my $limit = 3**int($k/2);
    my @row;
    $row[0][0] = to_bigint(1);
    foreach my $y (0 .. $y_target) {
      if ($y-2 >= 0) { undef $row[$y-2]; }
      my $x_limit = $x_target - $y_target + $y;
      for (my $x = $y; $x <= $x_limit; $x += 2) {
        if ($x||$y) { $row[$y][$x] = 0; }
        if ($x-2 >= 0 && defined $row[$y][$x-2] && $has_edge->($x-2,$y, $x,$y)) {
          $row[$y][$x] += $row[$y][$x-2];
        }
        if ($y-1 >= 0 && defined $row[$y-1][$x-1] && $has_edge->($x-1,$y-1, $x,$y)) {
          $row[$y][$x] += $row[$y-1][$x-1];
        }
      }
      if ($verbose) {
        print " y=$y row ";
        my $aref = $row[$y];
        for (my $x = $y; $x <= $#$aref; $x+=2) {
          print $aref->[$x],",";
        }
        print " x_limit=$x_limit\n";
      }
    }
    print " target ",$row[$y_target][$x_target],"\n";
  }
  exit 0;
}

{
  # alternate terdragon turns
  # d(3n+1) = 1
  # d(3n+2) = -1
  # d(3n) = - d(n)
  # not in OEIS: 1,-1,-1,1,-1,1,1,-1,1,1,-1,-1,1,-1,1,1,-1,-1,1,-1,-1,1,-1,1,1
  
  my $d;
  $d = sub {
    my ($n) = @_;
    if ($n < 1) { die; }
    return ($n % 3 == 1 ? 1
            : $n % 3 == 2 ? -1
            : - $d->($n/3));
  };
  foreach my $n (1 .. 27) {
    print $d->($n),",";
  }
  print "\n";

  my $a = 0;
  my $unfold = 60;
  print "\\draw (0,0)";
  foreach my $n (1 .. 3**7) {
    print "--++($a:1)";
    $a = ($a + $d->($n)*(180-$unfold)) % 360;
    if ($n % 6 == 0) { print "\n  "; }
  }
  print ";\n";
  exit 0;
}


{
  # Epred runs of 2 to 12
  my $f = Epred_flat();
  for (1 .. 11) {
    $f = Epred_flat()->intersect($f->MyFLAT::digits_increment(add => -1))
      ->MyFLAT::minimize(verbose=>1);

    my $r = $f->MyFLAT::reverse->MyFLAT::minimize(verbose=>1);
    $r->MyFLAT::view;
  }
  exit 0;
}

{
  # Epred pairs state machine

  my $orig = EpredL_flat();
  my $f = $orig->intersect($orig->MyFLAT::digits_increment(add => -1));

  # $f = $f->MyFLAT::digits_increment(add => 1);

  # $f = $f->MyFLAT::minimize(verbose=>1);
  # $f->MyFLAT::view;
  $f = $f->MyFLAT::reverse;
  $f = $f->MyFLAT::minimize(verbose=>1);
  $f->MyFLAT::view;
   MyFLAT::FLAT_show_breadth($f,4, 'lowtohigh');
  exit 0;
}

{
  # EpredR followed by EpredL
  my $f = EpredR_flat();
  $f = EpredL_flat()->intersect($f->MyFLAT::digits_increment(add => -1));

  $f = $f->MyFLAT::digits_increment(add => 1);

  $f = $f->MyFLAT::minimize(verbose=>1);
  $f->MyFLAT::view;
  $f = $f->MyFLAT::reverse;
  $f = $f->MyFLAT::minimize(verbose=>1);
  $f->MyFLAT::view;
  exit 0;
}

{
  # EpredR separate fsm

  require MyFSM;
  my $fsm = MyFSM->new
    (table => { rs0  => { 0 => 'rs0',  1 => 'rs0',  2 => 'rs1' },
                rs1  => { 0 => 'rs2',  1 => 'rs0',  2 => 'rs1' },
                rs2  => { 0 => 'rs2',  1 => 'rs1',  2 => 'rs2' },
              },
     accepting => { rs0 => 'one',
                    rs1 => 'two',
                    rs2 => 'three',
                  },
     initial   => 'rs0',
    );
  # $fsm->show;
  # $fsm->view;

  $fsm = $fsm->reverse;
  $fsm->show;
  $fsm->view;
  print $fsm->print_tikz;

  # $fsm->simplify;
  # $fsm->show;
  # $fsm->view;
  # 
  # print $fsm->as_graphviz;
  exit 0;
}

{
  # ER1sides, ER2sides

  # rs0  222...21   20[02]11
  # rs1
  # rs2  20 and below any 1s are 10

  my $f;
  $f = RsideNum1_flat();
  $f = $f->MyFLAT::reverse;
  $f = $f->MyFLAT::minimize(verbose=>1);
  $f->MyFLAT::view;

  $f = ER2sides_flat();
  $f = $f->MyFLAT::reverse;
  $f = $f->MyFLAT::minimize(verbose=>1);
  $f->MyFLAT::view;

  # my $sans = Rpred_flat();
  # my $any = FLAT::Regex->new('(0|1|2)*')->as_dfa->MyFLAT::minimize;
  # my $p12 = FLAT::Regex->new('12')      ->as_dfa->MyFLAT::minimize;
  # my $p20 = FLAT::Regex->new('20')      ->as_dfa->MyFLAT::minimize;
  # my $s1 = $any->MyFLAT::concat($p12,$sans);
  # MyFLAT::FLAT_check_is_equal($s1,RsideNum1_flat());
  # exit 0;
  # 
  # my $s2 = $any->MyFLAT::concat($p20,$sans);
  # MyFLAT::FLAT_check_is_equal($s2,EpredR_flat());

  exit 0;
}

{
  # Epred run of 12
  my $f = Epred_flat();
  for (1 .. 11) {
    $f = Epred_flat()->intersect($f->MyFLAT::digits_increment(add => -1))
      ->MyFLAT::minimize(verbose=>1);
  }
  # MyFLAT::FLAT_show_breadth($f, 8);
  my $max_k = 15;

  # $f = $f->MyFLAT::digits_increment(add => -22);

  $f->MyFLAT::view;
  $f = $f->MyFLAT::reverse->MyFLAT::minimize(verbose=>1);
  $f->MyFLAT::view;
  $f->MyFLAT::FLAT_print_tikz;
  exit 0;
}


{
  # Lines in direction and total

  # d=0
  # countH = 1,2,4,7,12,22,43
  # countH(k) = 1/3*(2^(k+1) + [ 1, 2, 4, 5, 4, 2][(k%6)+1])
  # vector(20,k,countH(k-1))
  # vector(11,k,my(k=k-1); (1/3)*(2^(k+1) + (-1)^k) + [0,1,1,2,1,1][(k%6)+1])
  # vector(11,k,my(k=k-1); (1/3)*(2^(k+1) + 2*(-1)^(k+1)) + [1,0,2,1,2,0][(k%6)+1])
  # vector(11,k,my(k=k-1); (1/3)*(2^(k+1) - 4*(-1)^(k+1)) + [-1,2,0,3,0,2][(k%6)+1])
  #
  # d=1
  # countB = 0,1,2,4, 9,20,42,85,170
  # countB(k) = 1/3*(2^(k+1) + [-2,-1,-2,-4,-5,-4][(k%6)+1])
  # vector(20,k,countB(k-1))
  # vector(11,k,my(k=k-1); (1/3)*(2^(k+1) - (-1)^(k+1)) - [1,0,1,1,2,1][(k%6)+1])
  #
  # d=2
  # countF = 0,0,1,4,10,21,42,84,169,340,682
  # A111927  x^3/((x-1)*(2*x-1)*(x^2-x+1)).
  #  x^3/((x-1)*(2*x-1)*(x^2-x+1)) == 1/3*(  -3/(1-x) + 1/(1-2*x) + (2 - x)/(1-x+x^2)  )
  #  sqrt3i = quadgen(4*-3)
  #  w6 = 1/2 + 1/2*sqrt3i
  #  x^3/((x-1)*(2*x-1)*(x^2-x+1)) == 1/3*(  -3/(1-x) + 1/(1-2*x) + w6/(w6-x) + conj(w6)/(conj(w6)-x)   )
  # countF(k) = 1/3*(2^(k+1) + [-2,-4,-5,-4,-2,-1][(k%6)+1])
  # vector(20,k,countF(k-1))
  # vector(11,k,my(k=k-1); (1/3)*(2^(k+1) - (-1)^(k+1)) - [1,1,2,1,1,0][(k%6)+1])
  #
  # [-2,-4,-5,-4,-2,-1] + [ 1, 2, 4, 5, 4, 2] + [-2,-1,-2,-4,-5,-4] == [-3, -3, -3, -3, -3, -3]
  #
  # [-2,-4,-5,-4,-2,-1]
  # [ 1, 2, 4, 5, 4, 2]
  # [-2,-1,-2,-4,-5,-4]
  #  -1 -2 -2 -2 -1 -1  floor
  #   0  0  1  1  1  0
  #  -1 -1 -1 -2 -2 -2
  #   1, 2, 3, 3, 3, 2  range
  #
  #             [    1, 2, 4, 5, 4, 2]
  #             [-2,-1,-2,-4,-5,-4]
  #             [      -2,-4,-5,-4,-2,-1]
  # [-2,-4,-5,-4,-2,-1]
  #
  # [-2,-4,-5,-4,-2,-1] + 3 = [1, -1, -2, -1, 1, 2]

  # 1,0,2  1-d = 0,1,-1
  #
  # total = 2^(k+1) - 1
  # T_formula(k) = 3^(k-1) - 2*2^(k-1) + 1
  # T(k) = if(k==0,0, T_formula(k))
  # D(k) = if(k==0,0, 2^k-2)
  # Lines(k) = 3^k - 3*T(k) - D(k)
  # vector(10,k,my(k=k-1); Lines(k)) == vector(10,k,my(k=k-1); 2^(k+1)-1)
  #
  # Lines(k,d) = Skd(k,d) - T(k) - D(k,d)
  # Lines(k,d) = D(k,d+1)+D(k,d+1) + S(k,d+1)+S(k,d+2)  + (d=0)
  # k=2 Lines(k,0) = D(k,1)+D(k,2) + S(k,1)+S(k,2)
  #                = 2     + 0     +     4     0  + 2  = 8 ends = 4 lines
  # S(2,0) =
  #
  # sqrt3 = quadgen(4*3)
  # w12 = sqrt3/2 + 1/2*I
  #
  require MyLineSegments;
  my $path = Math::PlanePath::TerdragonCurve->new;
  my @countH;
  my @countF;
  my @countB;
  my @total;
  my @ranges;
  foreach my $level (0 .. 10) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($level);
    my $segments = MyLineSegments->new;
    foreach my $n ($n_lo .. $n_hi - 1) {
      $segments->add($path->n_to_xy($n),
                     $path->n_to_xy($n+1));
    }

    my @lines = $segments->lines;
    my $countH = 0;
    my $countF = 0;
    my $countB = 0;
    foreach my $line (@lines) {
      my ($x1,$y1, $x2,$y2) = @$line;
      my $dx = $x2 - $x1;
      my $dy = $y2 - $y1;
      my $length;
      if ($dy == 0) {
        $countH++; # horizontal
        $length = $dx/2;
      } elsif ($dx == $dy) {
        $countF++; # 60deg = 240deg
        $length = $dx;
      } elsif ($dx == -$dy) {
        $countB++; # 120deg
        $length = $dx;
      } else {
        die "oops $dx $dy";
      }
    }
    my $total = scalar(@lines);
    my $min = min($countH, $countF, $countB);
    my $max = max($countH, $countF, $countB);
    my $range = $max - $min;
    print "k=$level  $total = $countH + $countF + $countB   range $range\n";

    push @total, $total;
    push @countH, $countH;
    push @countF, $countF;
    push @countB, $countB;
    push @ranges, $range;
  }

  require Math::OEIS::Grep;
  Math::OEIS::Grep->search (array => \@total, verbose => 1);
  Math::OEIS::Grep->search (array => \@countH, verbose => 1);
  Math::OEIS::Grep->search (array => \@countB, verbose => 1);
  Math::OEIS::Grep->search (array => \@countF, verbose => 1);
  Math::OEIS::Grep->search (array => \@ranges, verbose => 1);
  exit 0;
}
{
  # other N at same X,Y, digit relationship
  my $path = Math::PlanePath::TerdragonCurve->new;
  my $width = 7;
  my $count = 0;
  foreach my $n (3**5 .. 3**6) {
    next if $n % 3 == 0;
    next if $n % 3 == 2;
    my ($x,$y) = $path->n_to_xy($n);
    my @n_list = $path->xy_to_n_list($x,$y);
    next unless @n_list == 3;
    next unless @n_list >= 2;
    next unless $n_list[0] == $n;

    printf "%*s orig\n", $width, Math::BaseCnv::cnv($n,10,3);

    foreach my $n (@n_list) {
      my @digits = digit_split_lowtohigh($n,3);
      my $n_ones = grep {$_==1} @digits;
      foreach my $i (0 .. $#digits) {
        my $ones_below = 0;
        foreach my $j (0 .. $i-1) {
          if ($digits[$i] == 1) { $ones_below++; }
        }
        my $ones_above = 0;
        foreach my $j ($i .. $#digits) {
          if ($digits[$j] == 1) { $ones_above++; }
        }
        print "ones $ones_above $ones_below\n";
        $digits[$i] -= ($digits[0]==1 ? 1 : +1) * $ones_below;
        $digits[$i] %= 3;
      }
      my $str = join('',reverse @digits);
      my ($other_n) = other_n($n);
      printf "%*s -> %*s  [%d]    other %s\n",
        $width, Math::BaseCnv::cnv($n,10,3),
        $width, $str,
        $n_ones,
        Math::BaseCnv::cnv($other_n,10,3);
    }

    print "\n";
    last if $count++ > 5;
  }
  exit 0;
}
{
  fRpred_flat()->MyFLAT::view;
  fLpred_flat()->MyFLAT::view;
  my $f = fBpred_flat();
  $f = $f->MyFLAT::minimize;
  $f = $f->MyFLAT::separate_sinks;
  $f->MyFLAT::view;
  exit 0;
}
{
  # Triangle of Terdragon Graph Diameter
  # edges 3^(k+1)
  # vertices [3, 7, 16, 40, 106, 292]
  # diameter [1,2,4,7,13,22,40]
  # reps      6,24,36,72,72,144
  #
  # TP_formula(k) = 3*P(k) - 3*J(k) - 2;
  # TP_formula(k) = 3^k + 3/2*2^k + 1;
  # TP(k) = if(k==0,3, TP_formula(k));
  # vector(10,k,k--; TP(k))
  # vector(10,k,k--; TP_formula(k))

  require MyGraphs;
  unshift @INC, "../gmaker/devel/lib";
  require Graph::Maker::PlanePath;
  foreach my $k (0..8) {
    # triangle of terdragons graph
    my $graph = Graph::Maker->new('planepath', undirected => 1,
                                  level=>$k,
                                  planepath=>'TerdragonCurve',
                                  n_lo => 2 * 3**$k,
                                  n_hi => 5 * 3**$k,
                                 );
    $graph->set_graph_attribute('vertex_name_type','xy-triangular');
    if ($k <= 2) {
      # MyGraphs::Graph_view($graph);
      MyGraphs::Graph_xy_print($graph);
    }

    my $apsp = $graph->APSP_Floyd_Warshall;
    my $diameter = $apsp->diameter;
    print "  diameter $diameter\n";

    my $count_total = 0;
    my $count_diams = 0;
    my $for = $apsp->for_shortest_paths
      (sub {
         my ($t, $u,$v, $n) = @_;
         my $length = $t->path_length($u,$v);
         if ($length == $diameter) {
           if ($k <= 2) {
             print "  at $u to $v\n";
           }
           $count_diams++;
         }
         $count_total++;
       });

    my $num_vertices = $graph->vertices;
    my $num_edges = $graph->edges;
    print "  total paths $count_total diams $count_diams num vertices $num_vertices num edges $num_edges\n";
  }
  exit 0;
}

{
  # N on centreline, counts
  #
  # distinct points between start,end
  # 2,4,8,18,46,126,362,1054,3106
  # 2,11,22,200,1201,11200,111102,1110001,11021001
  #
  # distinct points all of line start to end
  # 2,4,8,20,52,142,402,1162,3406
  # 2,11,22,202,1221,12021,112220,1121001,11200011
  #
  # total N between start,end
  # 2,6,14,40,112,338,1004,3022,9048
  # 2,20,112,1111,11011,110112,1101012,11010221,110102010
  #
  # total N whole line start,end
  # 2,6,14,42,122,366,1094,3282,9842
  # 2,20,112,1120,11112,111120,1111112,11111120,111111112
  # A122983(n) = 3^n/4 + (-1)^n/4 + 1/2;
  # vector(10,k,k--; 2*A122983(k))
  # CV0(k) = 3/2*3^k + vector_modulo([1/2,3/2],k);
  # vector(10,k,k--; CV0(k))
  # in direction +120
  # 2,4,8,18,46,126,362,1054,3106

  my $distinct = 0;
  my $extend = 1;
  my $rotate = 1;

  my $path = Math::PlanePath::TerdragonCurve->new;
  my @values;
  for (my $k = 0; $k <= 20; $k+=2) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my ($ex,$ey) = $path->n_to_xy($n_hi);
    my $g = gcd($ex,$ey);
    my $dx = $ex/$g;
    my $dy = $ey/$g;
    if ($dy==0) { $dx *= 2; }
    my $x = 0;
    my $y = 0;
    if ($extend) { $x=-$ex; $y=-$ey; $ex *=2; $ey *= 2; }
    ### $k
    ### $x
    ### $y
    ### $ex
    ### $ey
    ### $dx
    ### $dy
    foreach (1 .. $rotate) {
      ($x,$y) = xy_rotate_plus120($x,$y); 
      ($ex,$ey) = xy_rotate_plus120($ex,$ey); 
      ($dx,$dy) = xy_rotate_plus120($dx,$dy); 
    }
    my @count = (0,0,0,0);
    for (;;) {
      my @n_list = $path->xy_to_n_list($x,$y);
      @n_list = grep {$_>=$n_lo && $_<=$n_hi} @n_list;
      if (@n_list) {
        if ($distinct) {
          $count[scalar(@n_list)]++;      # distinct points
        } else {
          $count[1] += scalar(@n_list);   # N visits
        }
      }
      last if $x == $ex && $y == $ey;
      $x += $dx; $y += $dy;
    }
    my $total = sum(@count);
    print "k=$k  $count[1],$count[2],$count[3] = $total\n";
    push @values,$total;
  }
  print join(',',@values),"\n";
  print join(',',map{cnv($_,10,3)} @values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}

{
  # N on centreline state machine, 2 directions
  #
  #               *       *
  #              / \20   / \
  #           12/   \   /   \22
  #            /  11 \ /21   \
  #   *       *-------*       *
  #    \     / \     /
  #  00 \   /   \   /
  #      \ /   02\ /10
  #       *       *
  #        01

  my $h = 0;
  my $s1 = 1;
  my $s2 = 2;
  my $e1 = 3;
  my $e2 = 4;
  my $no = 5;
  my @state_table = ([ $s2,$e1,$s2, $e1,$h, $s1, $e2,$s1,$e2 ],  # $h

                     [ $h, $s2,$no, $no,$no,$no, $no,$no,$no ],  # $s1
                     [ $s1,$no,$no, $no,$no,$no, $no,$no,$no ],  # $s2

                     [ $no,$no,$no, $no,$no,$no, $no,$e2,$h  ],  # $e1
                     [ $no,$no,$no, $no,$no,$no, $no,$no,$e1 ],  # $e2

                     [ $no,$no,$no, $no,$no,$no, $no,$no,$no ],  # $no
                    );
  require MyFLAT;
  my $f = MyFLAT::aref_to_FLAT_DFA(\@state_table, name => 'NcentrePred',
                                   accepting_list=>[$h, $s1,$s2]);
  $f->MyFLAT::view;
  $f->MyFLAT::FLAT_print_tikz;
  $f = $f->MyFLAT::minimize;
  $f->MyFLAT::view;
  # MyFLAT::FLAT_show_breadth($f,2);
  # MyFLAT::FLAT_show_transitions($f,'0000');

  for (my $k=0; $k <= 10; $k+=2) {
    my $path = Math::PlanePath::TerdragonCurve->new;
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my $nk_on_centreline_by_path = sub {
      my ($n) = @_;
      my ($x,$y) = $path->n_to_xy($n);
      ($x,$y) = transform_to_horizontal($x,$y, $k);
      ### path horiz: "$n  $x,$y"
      return $y == 0;
    };
    foreach my $n ($n_lo .. $n_hi-1) {
      my $len = $k/2;
      my $n9 = sprintf '%0*s', $len, cnv($n,10,9);
      my $by_flat = $f->contains($n9) ? 1 : 0;
      my $by_path = $nk_on_centreline_by_path->($n) ? 1 : 0;
      unless ($by_path == $by_flat) {
        print "k=$k n=$n [$n9] path $by_path flat $by_flat\n";
        print "  "; MyFLAT::FLAT_show_transitions($f,$n9);
        exit 0;
      }
    }
  }
  exit 0;
}

{
  # N on centreline state machine, ternary
  #

  my $h = 0;
  my $s1 = 1;
  my $s2 = 2;
  my $e1 = 3;
  my $e2 = 4;
  my $no = 5;
  my @state_table = ([ $s2,$e1,$s2, $e1,$h, $s1, $e2,$s1,$e2 ],  # $h

                     [ $h, $s2,$no, $no,$no,$no, $no,$no,$no ],  # $s1
                     [ $s1,$no,$no, $no,$no,$no, $no,$no,$no ],  # $s2

                     [ $no,$no,$no, $no,$no,$no, $no,$e2,$h  ],  # $e1
                     [ $no,$no,$no, $no,$no,$no, $no,$no,$e1 ],  # $e2

                     [ $no,$no,$no, $no,$no,$no, $no,$no,$no ],  # $no
                    );

  my @new_state_table;
  foreach my $state (0 .. $#state_table) {
    foreach my $hi (0,1,2) {
      $new_state_table[4*$state]->[$hi] = 4*$state+$hi+1;
      foreach my $lo (0,1,2) {
        $new_state_table[4*$state+$hi+1]->[$lo]
          = 4*$state_table[$state]->[3*$hi+$lo];
      }
    }
  }

  require MyFLAT;
  my $f9 = MyFLAT::aref_to_FLAT_DFA(\@state_table, name => 'NcentrePred',
                                    accepting_list=>[$h, $s1, $s2]);
  my $f = MyFLAT::aref_to_FLAT_DFA(\@new_state_table, name => 'NcentrePred',
                                   accepting_list=>[$h, 4*$s1, 4*$s2]);
  $f9->MyFLAT::view;
  $f->MyFLAT::view;
  $f = $f->MyFLAT::minimize;
  $f->MyFLAT::view;
  # MyFLAT::FLAT_show_breadth($f,2);
  # MyFLAT::FLAT_show_transitions($f,'0000');

  for (my $k = 2; $k <= 10; $k+=2) {
    my $path = Math::PlanePath::TerdragonCurve->new;
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my $nk_on_centreline_by_path = sub {
      my ($n) = @_;
      my ($x,$y) = $path->n_to_xy($n);
      ($x,$y) = transform_to_horizontal($x,$y, $k);
      ### path horiz: "$n  $x,$y"
      return $y == 0;
    };
    foreach my $n ($n_lo .. $n_hi-1) {
      my $n3 =  sprintf '%0*s', $k, cnv($n,10,3);
      my $by_flat = $f->contains($n3) ? 1 : 0;
      my $by_path = $nk_on_centreline_by_path->($n) ? 1 : 0;
      unless ($by_path == $by_flat) {
        print "k=$k n=$n [$n3] path $by_path flat $by_flat\n";
        print "  "; MyFLAT::FLAT_show_transitions($f,$n3);
        exit 0;
      }
    }
  }
  exit 0;
}

{
  # N on centreline state machine, 4 directions
  #
  #               *       *
  #              / \20   / \
  #           12/   \   /   \22
  #            /  11 \ /21   \
  #   *       *-------*       *
  #    \     / \     /
  #  00 \   /   \   /
  #      \ /   02\ /10
  #       *       *
  #        01

  my $h = 0;
  my $s1 = 1;
  my $s2 = 2;
  my $s4 = 3;
  my $s5 = 4;
  my $e1 = 5;
  my $e2 = 6;
  my $e4 = 7;
  my $e5 = 8;
  my $no = 9;
  my @state_table = ([ $s5,$e1,$s5, $e1,$h, $s1, $e5,$s1,$e5 ],  # $h

                     [ $h, $s2,$no, $no,$no,$no, $no,$no,$no ],  # $s1
                     [ $s1,$no,$no, $no,$no,$no, $no,$no,$no ],  # $s2
                     [ $h, $s5,$no, $no,$no,$no, $no,$no,$no ],  # $s4
                     [ $s4,$no,$no, $no,$no,$no, $no,$no,$no ],  # $s5

                     [ $no,$no,$no, $no,$no,$no, $no,$e2,$h  ],  # $e1
                     [ $no,$no,$no, $no,$no,$no, $no,$no,$e1 ],  # $e2
                     [ $no,$no,$no, $no,$no,$no, $no,$e5,$h  ],  # $e4
                     [ $no,$no,$no, $no,$no,$no, $no,$no,$e4 ],  # $e5

                     [ $no,$no,$no, $no,$no,$no, $no,$no,$no ],  # $no
                    );
  require MyFLAT;
  my $f = MyFLAT::aref_to_FLAT_DFA(\@state_table, name => 'NcentrePred',
                                   accepting_list=>[$h, $s1,$s2,$s4,$s5]);
  $f->MyFLAT::view;
  $f = $f->MyFLAT::minimize;
  $f->MyFLAT::view;
  # MyFLAT::FLAT_show_breadth($f,2);
  # MyFLAT::FLAT_show_transitions($f,'0000');

  for (my $k=0; $k <= 10; $k+=2) {
    my $path = Math::PlanePath::TerdragonCurve->new;
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my $nk_on_centreline_by_path = sub {
      my ($n) = @_;
      my ($x,$y) = $path->n_to_xy($n);
      ($x,$y) = transform_to_horizontal($x,$y, $k);
      ### path horiz: "$n  $x,$y"
      return $y == 0;
    };
    foreach my $n ($n_lo .. $n_hi-1) {
      my $len = $k/2;
      my $n9 = sprintf '%0*s', $len, cnv($n,10,9);
      my $by_flat = $f->contains($n9) ? 1 : 0;
      my $by_path = $nk_on_centreline_by_path->($n) ? 1 : 0;
      unless ($by_path == $by_flat) {
        print "k=$k n=$n [$n9] path $by_path flat $by_flat\n";
        print "  "; MyFLAT::FLAT_show_transitions($f,$n9);
        exit 0;
      }
    }
  }
  exit 0;
}



{
  # Turns Tree -- Rights in Triangle
  # k=3 https://hog.grinvin.org/ViewGraphInfo.action?id=26977
  # k=4 https://hog.grinvin.org/ViewGraphInfo.action?id=26979

  require MyGraphs;
  unshift @INC, "../gmaker/devel/lib";
  require Graph::Maker::StarOfStars;
  my @graphs;
  foreach my $k (3,4) {
    my $graph = triangle_turn_tree_Graph($k);
    MyGraphs::Graph_view($graph, xy=>1);
    MyGraphs::Graph_xy_print($graph);
    push @graphs, $graph;

    if ($k >= 1) {
      my $stars = Graph::Maker->new('star_of_stars', undirected => 1,
                                    level=>$k-1,
                                    N=>4);
      MyGraphs::Graph_is_isomorphic($stars,$graph) or die;
    }
  }
  MyGraphs::hog_searches_html(@graphs);
  exit 0;
}
{
  # Graphs

  require MyGraphs;
  require Graph::Maker::PlanePath;
  my @graphs;
  foreach my $k (2) {
    # triangle of terdragons graph
    #   k=0  3-cycle
    #   k=1  https://hog.grinvin.org/ViewGraphInfo.action?id=922
    #          3-triangles, vertex in common
    #   k=2  hog not
    #   k=3  hog not
    my $graph = Graph::Maker->new('planepath', undirected => 1,
                                  level=>$k,
                                  planepath=>'TerdragonCurve',
                                  n_lo => 2 * 3**$k,
                                  n_hi => 5 * 3**$k,
                                 );
    $graph->set_graph_attribute('vertex_name_type','xy-triangular');

    # # neighbours6 adjacent points, shortcuts across boundary triangles
    # # edges between points of the curve a unit distance apart
    # #   k=0  single line segment
    # #   k=1 https://hog.grinvin.org/ViewGraphInfo.action?id=28  diamond
    # #   k=2 https://hog.grinvin.org/ViewGraphInfo.action?id=25189
    # #   k=3 https://hog.grinvin.org/ViewGraphInfo.action?id=25191
    # #
    # my $graph = Graph::Maker->new('planepath', undirected => 1,
    #                               planepath=>'TerdragonCurve',
    #                               level=>$k,
    #                               type => 'neighbours6');

    push @graphs, $graph;
  }
  # MyGraphs::graph6_view('F`?Nw');
  MyGraphs::hog_searches_html(@graphs);
  exit 0;
}
{
  # sub-curve segments on line start to end

  my $k = 6;
  my $path = Math::PlanePath::TerdragonCurve->new;
  my ($n_lo, $n_hi) = $path->level_to_n_range($k);
  my ($ex,$ey) = $path->n_to_xy($n_hi);
  my $dir6 = ($k/2) % 6;
  my $dx = $dir6_to_dx[$dir6];
  my $dy = $dir6_to_dy[$dir6];
  my %segs;
  foreach my $i (0 .. 3**($k/2)) {
    my $x = $dx * $i;
    my $y = $dy * $i;
    my @n_list = $path->xy_to_n_list($x,$y);
    push @n_list, map {$_-1} @n_list;
    @n_list = grep {$_>=$n_lo && $_<=$n_hi} @n_list;
    @segs{@n_list} = ();
  }
  my @segs = sort {$a <=> $b} keys %segs;
  print join(',',@segs),"\n";

  foreach my $n (@segs) {
    my ($x,$y) = $path->n_to_xy($n);
    ($x,$y) = transform_to_horizontal($x,$y, $k);
    my $n3 = int($n / 3**($k/2)) * 2;
    my $within = abs($n3-$x) < 6 ? '***' : '';
    print "$n    $n3 $x  $within\n";
  }
  exit 0;
}

{
  # boundary point numbers N

  #         -1,1 ----> 1,1
  #          / ^       . ^
  #         /   .     .   \
  #        v     .   v     \
  #    -2,0  ...> 0,0 ===> 2,0
  #        ^     .   ^     /
  #         \   .     .   /
  #          \ v       . v
  #        -1,-1 ----> 1,-1

  my $point_enclosed_segs =  MyTerdragonConf->new
    (name => 'point enclosed',
     segs => [ "-1,-1,1",
               "-1,-1,0",
               "2,0,2",
               "2,0,1",
               "-1,1,0",
               "-1,1,2" ]);
  sub conf_is_boundary_point {
    my ($conf) = @_;
    $point_enclosed_segs
  }

  fBpred_hullsaround_init();
  my @states = grep { my $state = $_;
                      my $conf = $hullsaround_state_to_conf[$state];
                      $point_enclosed_segs->is_subset($conf)
                    } 0 .. $#hullsaround_state_to_conf;
  print "point enclosed states  ".join(',',@states),"\n";
  fBpred_hullsaround_init();

  my $PnonB_pred = MyFLAT::aref_to_FLAT_DFA(\@hullsaround_state_table,
                                            name => 'point enclosed',
                                            starting => 0,
                                            accepting_list => \@states,
                                           );
  $PnonB_pred = $PnonB_pred->MyFLAT::minimize;
  my $PB_pred = $PnonB_pred->complement;
  $PB_pred = $PB_pred->MyFLAT::minimize;
  my $PB_LtoH_pred = $PB_pred->MyFLAT::reverse;
  MyFLAT::view($PB_LtoH_pred);

  my $TB_pred = $PB_pred->intersect(Tpred_k_flat());
  $TB_pred = $TB_pred->MyFLAT::minimize;
  # MyFLAT::view($TB_pred);

  my $TR_pred = $TB_pred->intersect(Tpred_k_flat());
  $TB_pred = $TB_pred->MyFLAT::minimize;
  # MyFLAT::view($TB_pred);

  my $TB_LtoH_pred = $TB_pred->MyFLAT::reverse;
  MyFLAT::view($TB_LtoH_pred);

  exit 0;

}
{
  # triple visited on boundary
  # 2,8,20,44,92,188,380,764,1532,3068,

  my @values;
  my $path = Math::PlanePath::TerdragonCurve->new;
  foreach my $k (0 .. 14) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);

    my %seen;
    foreach my $n ($n_lo .. $n_hi) {
      $path->_UNDOCUMENTED__n_segment_is_boundary($n,$k) or next;
      my ($x,$y) = $path->n_to_xy($n);
      my @n_list = $path->xy_to_n_list($x,$y);
      @n_list = grep {$_ <= $n_hi} @n_list;
      if (@n_list == 3) {
        ### found: "n=$n  at $x,$y  n_list ".join(',',@n_list)
        $seen{"$x,$y"} = 1;
      }
    }

    my $count = keys %seen;
    print "$count,";
    push @values, $count;
  }

  splice @values, 0,3;
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}

{
  # turns tree - widths
  require MyGraphs;
  my $graph = triangle_turn_tree_Graph(6);
  print join(',',MyGraphs::Graph_width_list($graph, '1,1')), "\n";
  exit 0;
}
{
  # turn tree padded
  require MyGraphs;
  my $graph = triangle_turn_tree_Graph(3);
  MyGraphs::Graph_pad_degree($graph, 3);
  $graph = MyGraphs::Graph_line_graph($graph);
  MyGraphs::Graph_view($graph);
  exit 0;
}

{
  # Area Graphs
  # k=0 single vertex
  # k=1 claw
  # k=2 hog not
  # k=3 hog not

  require MyGraphs;
  my @graphs;
  foreach my $k (0 .. 3) {
    my $graph = triangle_area_Graph($k);
    MyGraphs::Graph_view($graph);
    push @graphs, $graph;
  }
  MyGraphs::hog_searches_html(@graphs);
  exit 0;
}
{
  # turns tree widths
  unshift @INC, "../gmaker/devel/lib";
  require MyGraphs;
  foreach my $k (0 .. 5) {
    my $graph = triangle_turn_tree_Graph($k);
    print join(',',MyGraphs::Graph_width_list($graph)),"\n";
  }
  exit 0;
}

{
  # Turns Tree -- adjacent rights in level

  require Graph;
  require Math::NumSeq::PlanePathTurn;
  my $path = Math::PlanePath::TerdragonCurve->new;
  my $seq = Math::NumSeq::PlanePathTurn->new(planepath_object => $path,
                                             turn_type => 'LSR');

  my $turn_tree_Graph = sub {
    my ($k) = @_;
    my $graph = Graph->new(undirected => 1);
    $graph->set_graph_attribute (name => "Terdragon Turn Tree $k");
    $graph->set_graph_attribute('vertex_name_type','xy-triangular');
    my $n_lo = 1;
    my $n_hi = 3**$k-1;
    my %seen;
    foreach my $n ($n_lo .. $n_hi-1) {
      my $turn = turn($n);
      if ($n==$n_lo) { $turn = 1; }
      if ($turn == -1) {
        my ($x,$y) = $path->n_to_xy($n);
        my $v = "$x,$y";
        $seen{$v} = 1;
        ### $v
        $graph->add_vertex($v);
      }
    }

    foreach my $v (keys %seen) {
      my ($x,$y) = split /,/, $v;
      foreach my $dir (0 .. $#dir6_to_dx) {
        my $x2 = $x+$dir6_to_dx[$dir];
        my $y2 = $y+$dir6_to_dy[$dir];
        my $v2 = "$x2,$y2";
        if ($seen{$v2}) {
          $graph->add_edge($v,$v2);
        }
      }
    }
    return $graph;
  };

  unshift @INC, "../gmaker/devel/lib";
  require MyGraphs;
  require Graph::Maker::PlanePath;
  my @graphs;
  foreach my $k (5) {
    my $graph = $turn_tree_Graph->($k);
    MyGraphs::Graph_view($graph, xy=>1);
    MyGraphs::Graph_xy_print($graph);
    push @graphs, $graph;
  }
  MyGraphs::hog_searches_html(@graphs);
  exit 0;
}
{
  # waist, middle to boundary nearest

  # k=0 dx=1  norm=dx^2 = 1
  # k=1 same norm = 1

  my @values;
  my $path = Math::PlanePath::TerdragonCurve->new;
  foreach my $k (0 .. 15) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    my ($xm,$ym) = ($x_hi/2, $y_hi/2);

    my $min_norm = 1e38;
    my ($min_x,$min_y);
    my @min_n;
    foreach my $n ($n_lo .. $n_hi) {
      $path->_UNDOCUMENTED__n_segment_is_right_boundary($n) or next;
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);

      foreach my $midpoint (0,1) {
        my $tx = $x - $xm;
        my $ty = $y - $ym;
        my $norm = $tx*$tx + 3*$ty*$ty;
        # print "  n=$n  norm=$norm\n";
        if ($norm < $min_norm) {
          $min_norm = $norm;
          @min_n = ();
          ($min_x,$min_y) = ($x,$y);
        }
        if ($norm == $min_norm) {
          push @min_n, ($midpoint?"mid ":"")."n=$n xy=$x,$y";
        }

        $x += $dx/2;
        $y += $dy/2;
      }
    }

    my $angle = atan2(($min_y-$ym)*sqrt(3), ($min_x-$xm)) * 180/3.141592;
    $angle -= 30*$k;
    $angle %= 360;
    if ($angle > 180) { $angle -= 360; }
    print "k=$k norm=$min_norm angle=$angle   ",join('  ',@min_n),"\n";
    push @values, $min_norm;
  }

  splice @values, 0,4;
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}



{
  # N triple point list
  my $path = Math::PlanePath::TerdragonCurve->new;
  for (my $n = 0; $n < 81; $n++) {
    my ($x,$y) = $path->n_to_xy($n);
    my @n_list = $path->xy_to_n_list($x,$y);
    if (@n_list==3) {
      print " $n,";
    }
  }
  print "\n";
  exit 0;
}
{
  # Rpred / Lpred on OEIS sample values
  # nothing much ternary related

  require Encode::Locale;
  binmode(STDOUT, ':encoding(console_out)');
  require Math::OEIS::Stripped;
  my $stripped = Math::OEIS::Stripped->new;
  require Math::OEIS::Names;
  my $names = Math::OEIS::Names->new;

  my $fh = $stripped->fh;
  my $count = 0;
  my $count_Rpred = 0;
  my $count_Lpred = 0;
  while (defined(my $line = readline $fh)) {
    $line =~ /^(A\d+)\s+,?(([012],)+)\s*$/ or next;
    my $anum = $1;
    my $str = $2;
    $str =~ s/,//g;
    $count++;

    my $Rpred = Rpred_of_ternary_str($str);
    my $Lpred = Lpred_k_of_ternary_str($str);
    if ($Rpred || $Lpred) {
      my $name = $names->anum_to_name($anum);
      print "$anum  $name\n";
      print " $str\n";
      print " ",($Rpred ? "Rpred" : ""),($Lpred ? "  Lpred" : ""),"\n";
      print "\n";
    }
    $count_Rpred += $Rpred;
    $count_Lpred += $Lpred;
  }
  print "count binaries $count Rpred $count_Rpred Lpred $count_Lpred\n";
  exit 0;
}

{
  my $f = MyFLAT::eventually_accepting(NonRpred_flat());
  MyFLAT::view($f);
  exit 0;
}
{
  $hullsaround_segs_outer->print;
  $hullsaround_segs_inner->print;
  $hullsaround_segs_all->print;
  my $conf = MyTerdragonConf->initial (within => $hullsaround_segs_all);
  $conf->print;
  foreach my $digit (0,1,2) {
    my $e = $conf->expand($digit);
    print join(' ',$e->bbox),"\n";
    $e->print;
  }

  $L_triangle_segs->clone(within=>$hullsaround_segs_all)->print;
  $R_triangle_segs->clone(within=>$hullsaround_segs_all)->print;
  $B_triangles_segs->clone(within=>$hullsaround_segs_all)->print;

  foreach my $digit (0,1,2) {
    $hullsaround_segs_all->expand($digit)->print;
  }

  exit 0;
}
{
  # other n state transitions, digits low to high

  require MyFSM;
  my %table;
  my $state_to_name = sub {
    my ($state) = @_;
    my ($delta,$ap) = @$state;
    my $out = ''; # ($bit ^ ($delta_x+$delta_y)) & 1;
    my $flip = ($delta*$ap) % 3;
    if ($flip==2) { $flip = -1; }
    return "$delta,ap=$ap flip $flip";
  };
  my @pending = ([1,0],
                 [2,0]);
  my %initials = map {$state_to_name->($_) => 1} @pending;
  my %accepting;
  my $transition_use_new_digit = 0;
  my %seen;
  while (@pending) {
    my $state = shift @pending;
    my ($delta,$ap) = @$state;
    my $from = $state_to_name->($state);
    next if $seen{$from}++;

    print "$from\n";
    if ($delta == 0) {
      print "  is accepting\n";
      $accepting{$from} = "same";
    }

    foreach my $digit (0,1,2) {
      ($delta,$ap) = @$state;
      my ($new_digit,$new_delta);
      if ($ap == 0) {
        $ap = $digit;
        $new_digit = $digit;
        $new_delta = $delta;
      } else {
        $new_digit = ($digit + $delta*$ap) % 3;
        $new_delta = ($delta - ($digit==1) + ($new_digit==1)) % 3;
      }

      my $new_state = [$new_delta,$ap];
      my $to = $state_to_name->($new_state);
      $table{$from}->{$transition_use_new_digit ? $new_digit : $digit} = $to;
      my $digit_diff = ($new_digit - $digit) % 3;
      my $flip = ($digit_diff == 0 ? "same" : "flip +$digit_diff");
      print "  $digit -> $to  digits old=$digit new=$new_digit $flip to delta=$new_delta\n";

      push @pending, $new_state;
    }
  }

  my $fsm = MyFSM->new(table => \%table,
                       initials => \%initials,
                       accepting => \%accepting,
                       title => 'N Other Transitions'
                       . ($transition_use_new_digit ? ", New Digit" : ""),
                       flow => 'west',
                      );

  # my @accepting = keys %accepting;
  # foreach my $i (1 .. $#accepting) {
  #   $fsm->merge_state($accepting[0],$accepting[1]);
  # }

  # $fsm = $fsm->digit_delta;
  $fsm->show;
  $fsm->simplify (verbose => 1);

  #  $fsm = $fsm->reverse;
  # $fsm->simplify (verbose => 1);
  # $fsm->renumber_states;

  # $fsm->print_tikz;
  $fsm->view;
  exit 0;
}

{
  # graph distance between symmetric ends
  #     *---*
  #      \
  #   *---*
  #
  #          0 1  2  3  4  5  6   7
  # distance   2, 3, 5, 9,15,26, 44
  #        add    1  2  4  6  9, 18

  my $path = Math::PlanePath::TerdragonCurve->new;
  my @values;
  foreach my $k (1..2) {
    my $graph = terdragon_Graph($k);
    my ($n_lo, $n_hi) = $path->level_to_n_range($k-1);
    my ($x_hi,$y_hi) = $path->n_to_xy(3*$n_hi);
    my ($x_shift,$y_shift) = $path->n_to_xy(2*$n_hi);
    my @distances;
    foreach my $n ($n_lo .. $n_hi) {
      my ($x1,$y1) = $path->n_to_xy($n);
      my ($x2,$y2) = ($x1 + $x_shift, $y1 + $y_shift);
      my $v1 = Graph_xy_to_vertex($x1,$y1);
      my $v2 = Graph_xy_to_vertex($x2,$y2);
      my $distance = $graph->path_length($v1,$v2);
      push @distances, $distance;
    }
    my $max_distance = max(@distances);
    my $min_distance = min(@distances);
    @distances = sort {$a<=>$b} @distances;
    print "k=$k  $min_distance to $max_distance\n";
    push @values, $max_distance;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}
{
  # right boundary convex hull

  # k=1 area=0 points=4   2,0 3,1 1,1 0,0
  # k=2 area=7 points=5   2,0 3,1 3,3 1,3 0,0
  # k=3 area=19 points=6   3,3 0,6 -2,6 0,0 2,0 3,1
  # k=4 area=55 points=8   -1,7 -9,9 -11,9 -8,6 0,0 2,0 3,1 3,3
  # k=5 area=181 points=10   -10,10 -12,10 -29,9 -26,6 -23,5 0,0 2,0 3,1 3,3 -1,7
  # k=6 area=559 points=11   -10,10 -30,10 -33,9 -56,0 -53,-3 -50,-4 -44,-4 2,0 3,1 3,3 -1,7
  # k=7 area=1668 points=13   -30,10 -60,0 -64,-4 -83,-27 -80,-30 -77,-31 -71,-31 -62,-28 2,0 3,1 3,3 -1,7 -10,10
  # k=8 area=4941 points=14   -60,0 -91,-31 -91,-37 -83,-81 -80,-84 -77,-85 -71,-85 -62,-82 -50,-70 3,1 3,3 -1,7 -10,10 -30,10
  # k=9 area=14897 points=14   -91,-91 -81,-101 1,-165 4,-166 10,-166 19,-163 31,-151 31,-131 3,3 -1,7 -10,10 -30,10 -60,0 -91,-31
  # k=10 area=44809 points=14   0,-182 30,-192 247,-247 253,-247 262,-244 274,-232 274,-212 244,-182 -1,7 -10,10 -30,10 -60,0 -91,-31 -91,-91
  # k=11 area=134340 points=14   273,-273 335,-273 739,-247 748,-244 760,-232 760,-212 730,-182 640,-152 -10,10 -30,10 -60,0 -91,-31 -91,-91 0,-182
  # k=12 area=402417 points=14   1489,11 1489,31 1459,61 1369,91 1183,91 -30,10 -60,0 -91,-31 -91,-91 0,-182 273,-273 821,-273 911,-243 1477,-1
  # k=13 area=1207901 points=14   1640,0 1731,91 2218,740 2218,760 2188,790 2098,820 1912,820 1639,729 -60,0 -91,-31 -91,-91 0,-182 273,-273 821,-273
  # k=14 area=3624739 points=14   1640,0 2460,820 2460,1002 2218,2218 2188,2248 2098,2278 1912,2278 1639,2187 1366,1914 -91,-31 -91,-91 0,-182 273,-273 821,-273
  # k=15 area=10873428 points=14   2460,2460 2186,2734 1,4435 -89,4465 -275,4465 -548,4374 -821,4101 -821,3555 -91,-91 0,-182 273,-273 821,-273 1640,0 2460,820
  # k=16 area=32614821 points=14   -1,4921 -820,5194 -6650,6652 -6836,6652 -7109,6561 -7382,6288 -7382,5742 -6560,4920 0,-182 273,-273 821,-273 1640,0 2460,820 2460,2460
  # k=17 area=97850297 points=14   -7381,7381 -9021,7381 -19958,6652 -20231,6561 -20504,6288 -20504,5742 -19682,4920 -17222,4100 273,-273 821,-273 1640,0 2460,820 2460,2460 -1,4921

  require Math::Geometry::Planar;
  my $path = Math::PlanePath::TerdragonCurve->new;
  my @values;
  foreach my $k (11) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    my $points = [];

    foreach my $n ($n_lo .. $n_hi) {
    # foreach my $n ($n_hi .. 3*$n_hi) {  # of V part
      $path->_UNDOCUMENTED__n_segment_is_right_boundary($n) or next;
      push @$points, [ $path->n_to_xy($n) ];
    }
    print scalar(@$points),"\n";

    my $planar = Math::Geometry::Planar->new;
    $planar->points($points);
    if (@$points >= 5) {
      $planar = $planar->convexhull2;
      $points = $planar->points;
    }
    my $area = (@$points >= 3 ? $planar->area : 0);
    my $num_points = scalar(@$points);
    $points = hull_points_sort_by_angle($points, $x_hi,$y_hi);

    print "k=$k area=$area points=$num_points   ",points_str($points),"\n";
    push @values, $area;
  }
  splice @values, 0,6;
  print join(',',@values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}

{
  # graph V diameter
  #          0 1  2  3  4  5  6  7
  # diameter 2,3, 6,10,16,27,45
  # cf whole 1,3, 5, 8,13,22,39,66
  # sum      3 6 11 18 29 49

  my $path = Math::PlanePath::TerdragonCurve->new;
  my @values;
  foreach my $k (0 .. 8) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);

    my $graph = terdragon_Graph($k, n_hi => 2*$n_hi);
    my ($x_hi,$y_hi) = $path->n_to_xy(2*$n_hi);

    my $diameter = $graph->diameter;
    my @longest_path = $graph->longest_path;

    print "k=$k  diameter=$diameter  $longest_path[0] to $longest_path[-1]    cf end $x_hi,$y_hi\n";
    push @values, $diameter;
  }
  shift @values;
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}

{
  # graph neighbours for graph-diameter.c
  #  7    8    9   10   11    12
  # 66, 113, 194, 337, 580, 1005

  my $path = Math::PlanePath::TerdragonCurve->new;
  print "{\n";
  foreach my $k (12) {
    my %xy_to_v = ('0,0' => 0);
    my $num_vertices = 1;
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my @neighbours;

    my $prev_v = 0;
    foreach my $n (1 .. $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      my $xy = "$x,$y";
      my $v = ($xy_to_v{$xy} //= $num_vertices++);
      push @{$neighbours[$v]}, $prev_v;
      push @{$neighbours[$prev_v]}, $v;
      $prev_v = $v;
    }

    $num_vertices == P($k) or die;
    print "{\n";
    foreach my $v (0 .. $num_vertices-1) {
      print "  { ",join(',',@{$neighbours[$v]},-1)," },\n";
    }
    print "},\n";
  }
  print "},\n";
  sleep 1;
  exit 0;
}



{
  # terdragon graph end to end distance
  #          0 1 2 3  4  5  6  7
  # distance 1,3,5,8,13,22,39,66

  my $path = Math::PlanePath::TerdragonCurve->new;
  my @values;
  foreach my $k (4, 8) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);

    # my %neighbour;
    # foreach my $n ($n_lo .. $n_hi) {
    #   my ($x,$y) = $path->n_to_xy($n);
    #   my $key = "$x,$y";
    #   $

    my $graph = terdragon_Graph($k);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    my $u = Graph_xy_to_vertex(0,0);
    my $v = Graph_xy_to_vertex($x_hi,$y_hi);
    my $distance = $graph->path_length($u,$v);

    print "k=$k  distance=$distance\n";
    push @values, $distance;
  }
  shift @values;
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}
{
  # Graph diameter k=7 start to end -- generate tikz
  #
  #          0 1 2 3  4  5  6  7
  # diameter 1,3,5,8,13,22,39,66
  #            2 2 3  5  9 17 27
  # even
  # 0                              indent=0
  # 2  3 + 2*(1)     = 5           indent=1
  # 4  9 + 2*(1 + 1) = 13
  # 6  27 + 2*(4 + 1 + 1) = 39
  # indentS(h) = if(h==0,0, indentV(h-1));
  # indentV(h) = if(h==0,1, 3^(h-1) + indentS(h-1));
  # indent(h) = if(h==0,0, h%2==0, 3*indent(h-1)+1, 3*indent(h-1)+1);
  # vector(10,h,h--; indentS(h))
  # A006342
  # indent(h) = 1/8*(3^h - vector_modulo([1,-5],h));
  # vector(10,h,h--; indent(h))
  # sum_indent(h) = sum(j=0,h, indent(j));
  # sum_indent(h) = 1/16*(3^(h+1) - 8*h + vector_modulo([-3,15,21,7],h));
  # vector(10,h,h--; sum_indent(h))
  # vector(10,h,h--; 3^h + 2*sum_indent(h))
  # diameter_even(h) = 11/8*3^h + 1/2*h + 1/8*vector_modulo([-3,3],h);
  # vector(10,h,h--; diameter_even(h))
  # diameter_odd(h) = diameter_even(h) + 3^h;
  # vector(10,h,h--; diameter_odd(h))
  # diameter(k) = vector_modulo([11/8,19/8],k)*3^floor(k/2) + k/4 + 1/8*vector_modulo([-3,-5,3,1],k);
  # vector(10,k,k--; diameter(k))

  # even same dir as b^k is w12^k = w6^(k/2)
  # odd b^k+90deg = w12^(k+3) = w6^(k+3)/2
  # Mleft(k) = 1/2*b^k + if(k%2==0, -1/2*w6^(k/2), 1/2*w6^((k+3)/2));
  # Mleft(1) == w6
  # Mleft(2) == w6
  # Mleft(3) == 1/2*b^3 - 1/2    \\ towards X neg
  # Mleft(4) == 4*w3
  # Mleft(5) == 4*w3 - 5
  # w3_abs_sum(z) = my(v=z_to_w3(z)); abs(v[1])+abs(v[2]);
  # trihatten(z1,z2=0) = my(z = z1-z2); \
  #   min(w3_abs_sum(z),min(w3_abs_sum(z*w3),w3_abs_sum(z*w3^2)));
  # trihatten(-5) == 5
  # z_to_w3(z) = my(y=2*imag(z), x = z-w3*y); \
  #   if(imag(x),error("z_to_w3() oops ",x)); [real(x),y];
  #
  # Mleft_sum(k) = sum(j=1,k-1, if(j==1,2, trihatten(Mleft(j),Mleft(j-1))));
  # vector(20,k,k--; floor(2*Mleft_sum(k)))
  # diameter(k) = if(k==0,1, k==1,3,  2*Mleft_sum(k) - !(k%2) + 1);
  # vector(10,k,k--; diameter(k))
  #
  # for(k=1,8,print(z_to_w3(Mleft(k))," ",trihatten(Mleft(k),Mleft(k-1))))
  # for(i=1,8,print(trihatten(Mleft(i),Mleft(i+1))))
  # for(i=1,8,print(1+2*sum(j=1,i-1, trihatten(Mleft(j),Mleft(j+1)))))
  #
  # dpow(k) = if(k==0,1, k==1,3, 3^floor(k/2)*vector_modulo([3/2,5/2],k) -1/4*(-1)^k - 1/2*k + 7/4);
  # vector(10,k,k--; dpow(k))

  # dpow_by_recurrence(k) = dpow(k-2) + vector_modulo([1,5/3],k)*3^floor(k/2)-1;
  # vector(10,k,k--; dpow_by_recurrence(k))
  # vector(10,k,k--; k*=2; dpow(k)-dpow(k-2))
  # vector(10,k,k--; k*=2;k++; 5*3^floor(k/2)-1)

  my @vertices = (
                  '0+0', '2+0', '1+1', '0+2', '-1+3', '-2+4', '-4+4',
                  '-6+4', '-8+4', '-10+4', '-11+3', '-13+3', '-14+2',
                  '-15+1', '-17+1', '-19+1', '-20+0', '-21-1', '-22-2',
                  '-23-3', '-24-4', '-25-5', '-26-6', '-27-7', '-29-7',
                  '-30-8', '-31-9', '-32-10', '-33-11', '-34-12', '-36-12',
                  '-37-13', '-38-14', '-39-15', '-41-15', '-42-16',
                  '-43-17', '-44-18', '-45-19', '-46-20', '-47-21',
                  '-48-22', '-49-23', '-50-24', '-52-24', '-54-24',
                  '-55-25', '-56-26', '-58-26', '-59-27', '-60-28',
                  '-61-29', '-63-29', '-65-29', '-67-29', '-69-29',
                  '-70-30', '-71-31', '-73-31', '-75-31', '-77-31',
                  '-79-31', '-80-30', '-81-29', '-82-28', '-83-27', '-81-27'
                 );
  my $level = 6;
  my $path = Math::PlanePath::TerdragonCurve->new;
  my ($n_lo, $n_hi) = $path->level_to_n_range($level);
  require MyPrintwrap;
  MyPrintwrap::printwrap("  \\draw[my gray]");
  my $join = '';
  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    MyPrintwrap::printwrap("$join($x,$y)");
    $join = '--';
  }
  MyPrintwrap::printwrap(";\n");
  MyPrintwrap::printwrap("  \\draw[my path]");
  foreach my $v (@vertices) {
    my ($x,$y) = Graph_vertex_to_xy($v);
    MyPrintwrap::printwrap("$join($x,$y)");
    $join = '--';
  }
  MyPrintwrap::printwrap(";\n");

  my ($ex,$ey) = $path->n_to_xy($n_hi);
  {
    my $x = $ex/2;
    my $y = $ey/2;
    print "  \\coordinate (M) at ($ex,$ey);\n";
  }
  {
    my ($n_lo, $n_hi) = $path->level_to_n_range($level-1);
    my ($x,$y) = $path->n_to_xy($n_hi);
    print "  \\coordinate (end0) at ($x,$y);\n";
  }
  {
    my ($n_lo, $n_hi) = $path->level_to_n_range($level-1);
    my ($x,$y) = $path->n_to_xy($n_hi);
    $x /= 2; $y /= 2;
    $x = $ex - $x;
    $y = $ey - $y;
    print "  \\coordinate (M2) at ($x,$y);\n";
  }
  foreach my $offset (1 .. 3) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($level-$offset);
    my ($x,$y) = $path->n_to_xy($n_hi);
    $x /= 2; $y /= 2;
    print "  \\node at ($x,$y) [my dot];\n";
  }
  exit 0;
}


{
  # convex hull inertia by points
  # horizontal extent 1/24.0 == 0.041666666
  # hull area sqrt3/4 * 29/24.0 == 0.52322368
  my $k = 6 *20;
  my $resolution = 100;

  my ($points,$ex,$ey) = hull_and_end($k);
  {
    my $scale = 1/sqrt(3)**$k;
    my $xfactor = $scale * 1/2;
    my $yfactor = $scale * sqrt(3)/2;
    foreach my $p (@$points) {
      $p->[0] *= $xfactor;
      $p->[1] *= $yfactor;
    }
  }

  require Math::Complex;
  my $w3 = Math::Complex::root(1,3,1);
  my $w6 = Math::Complex::root(1,6,1);
  my $b = $w6 + 1;
  print "w3=$w3 w6=$w6 b=$b\n";
  $points = [
             complex_to_point(-1/24        ),   # P1
             complex_to_point(-1/24 * $b**1),   # P2
             complex_to_point(-1/24 * $b**2),   # P3
             complex_to_point(-1/24 * $b**3),   # P4
             complex_to_point(-1/24 * $b**4),   # P5
             complex_to_point(-1/24 * $b**5),   # P6
             complex_to_point(-1/24 * $b**5 * (1+1/9*$w6)),  # P7

             complex_to_point(1 + 1/24        ),   # P1
             complex_to_point(1 + 1/24 * $b**1),   # P2
             complex_to_point(1 + 1/24 * $b**2),   # P3
             complex_to_point(1 + 1/24 * $b**3),   # P4
             complex_to_point(1 + 1/24 * $b**4),   # P5
             complex_to_point(1 + 1/24 * $b**5),   # P6
             complex_to_point(1 + 1/24 * $b**5 * (1+1/9*$w6)),  # P7
            ];

  require Math::Geometry::Planar;
  my $planar = Math::Geometry::Planar->new;
  $planar->points($points);
  my $G = $planar->centroid;
  foreach my $p (@$points) {
    $p->[0] -= $G->[0];
    $p->[1] -= $G->[1];
  }
  print "centroid $G->[0]  $G->[1]\n";

  foreach my $p (@$points) {
    print "  $p->[0], $p->[1]\n";
  }
  # print points_str($points),"\n";

  foreach my $p (@$points) {
    $p->[0] *= $resolution;
    $p->[1] *= $resolution;
  }

  $planar = Math::Geometry::Planar->new;
  $planar->points($points);

  require MyPlanar;
  my ($xmin,$ymin, $xmax,$ymax) = MyPlanar::planar_bbox_xyxy($planar);
  print "bbox X $xmin to $xmax  and Y $ymin to $ymax\n";
  my $Ix = 0;
  my $Iy = 0;
  my $Ixy = 0;
  my $mass = 0;
  foreach my $x ($xmin .. $xmax) {
    foreach my $y ($ymin .. $ymax) {
      if ($planar->isinside([$x,$y])) {
        $Ix += $y*$y;
        $Iy += $x*$x;
        $Ixy += -2 * $x*$y;
        $mass++;
      }
    }
  }
  # $Ix *= 1/4;         # (x/2)^2 = x^2 * 1/4
  # $Iy *= 3/4;         # (y*sqrt(3)/2)^2 = y^2 * 3/4
  # $Ixy *= sqrt(3)/4;  # (x/2 * y*sqrt(3)/2) = x*y * sqrt(3)/4

  $Ix /= $resolution**4;
  $Iy /= $resolution**4;
  $Ixy /= $resolution**4;
  printf "inertia %.9f,  %.9f, %.9f   mass %d\n", $Ix, $Iy, $Ixy, $mass;

  my $Ix_numerator = $Ix * 884736 / sqrt(3);
  my $Iy_numerator = $Iy * 2654208 / sqrt(3);
  my $Ixy_numerator = $Ixy * 27648;
  print "numerators $Ix_numerator, $Iy_numerator, $Ixy_numerator\n";
  exit 0;

  sub complex_to_point {
    my ($z) = @_;
    my $re = $z;
    my $im = 0;
    if (ref $z) {
      $re = $z->Re;
      $im = $z->Im;
    }
    return [ round_maybe($re), round_maybe($im) ];
  }
  sub round_maybe {
    my ($x) = @_;
    my $int = int($x+($x>=0 ? .5 : -.5));
    if (abs($x - $int) < 1e-10) { return $int; }
    return $x;
  }
}

{
  # convex hull P1 samples

  my $ex = 2;
  my $ey = 0;
  my $points = [
                [0,0],
                [$ex,$ey],
               ];

  print "0,0,0,0,0,0, ";
  foreach my $k (0 .. 60) {
    if ($k >= 6) {
      my $P = $points->[0];
      my ($x,$y) = @$P;
      print " $x/2 + $y*sqrt3i/2,";
    }
    ($points,$ex,$ey) = hull_expand($points,$ex,$ey);
  }
  print "\n";
  exit 0;
}

{
  # convex hull 12
  my $level = 12;

  my $path = Math::PlanePath::TerdragonCurve->new;
  my ($n_lo, $n_hi) = $path->level_to_n_range($level);
  my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
  my $ex = 2;
  my $ey = 0;
  my $points = [
                [0,0],
                [$ex,$ey],
               ];

  foreach (1 .. $level) {
    ($points,$ex,$ey) = hull_expand($points,$ex,$ey);
    # print "  % count ",scalar(@$points),"\n";
  }

  @$points = reverse @$points;
  $points = hull_points_sort_by_angle($points, -$ex,-$ey);
  foreach my $p (@$points) {
    my ($x,$y) = @$p;
    $x /= 100; $y /= 100;
    print "    --($x,$y)\n";
  }
  print "\n";
  print "% end ($ex,$ey)\n";

  require MyPrintwrap;
  MyPrintwrap::printwrap("  \\draw[my gray]");
  my $count = 0;
  my $join = '';
  for (my $n = $n_lo; $n <= $n_hi; $n += 3**6) {
    my ($x,$y) = $path->n_to_xy($n);
    $x /= 100; $y /= 100;
    MyPrintwrap::printwrap("$join($x,$y)");
    $join = '--';
    $count++;
  }
  MyPrintwrap::printwrap(";\n");
  print "  % $count points\n";
  exit 0;
}

{
  # convex hull by repeating

  require Math::Geometry::Planar;
  my $points = [ [0,0], [2,0] ];
  print "k=0  ",points_str($points),"\n";
  my $x_end = 2;
  my $y_end = 0;
  my @values;
  foreach my $k (1 .. 20) {
    my @new_points = (@$points,
                      (map {part1($_)} @$points),
                      (map {part2($_)} @$points),
                     );
    ### end: "$x_end, $y_end"
    ### @new_points
    my $planar = Math::Geometry::Planar->new;
    $planar->points(\@new_points);
    $planar = $planar->convexhull2;
    $points = $planar->points;
    my $area = $planar->area;
    my $num_points = scalar(@$points);
    $points = [ sort {point_cmp_by_angle($a,$b)} @$points ];

    print "k=$k area=$area points=$num_points   ",points_str($points),"\n";
    ($x_end,$y_end) = mul($x_end,$y_end);
    push @values, $area;
  }
  splice @values, 0,7;
  print join(',',@values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;

  sub part1 {
    my ($p) = @_;
    my ($x, $y) = xy_rotate_plus120(@$p);
    return [ $x_end + $x, $y_end + $y ];
  }
  sub part2 {
    my ($p) = @_;
    my ($x, $y) = @$p;
    my ($xm,$ym) = xy_rotate_plus60($x_end,$y_end);
    return [ $xm + $x, $ym + $y ];
  }
  sub point_rotate_plus120 {
    my ($p) = @_;
    return [ xy_rotate_plus120(@$p) ];
  }
  sub point_cmp_by_angle {
    my ($a,$b) = @_;
    return (Math::NumSeq::PlanePathDelta::_dxdy_to_dir4(@$a)
            <=>
            Math::NumSeq::PlanePathDelta::_dxdy_to_dir4(@$b));
  }
}

{
  # V two expansions

  my $level = 4;
  my $path = Math::PlanePath::TerdragonCurve->new;
  my ($n_lo, $n_hi) = $path->level_to_n_range($level);
  require MyPrintwrap;
  MyPrintwrap::printwrap("  \\draw");
  my $join = '';
  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    MyPrintwrap::printwrap("$join($x,$y)");
    $join = '--';
  }
  MyPrintwrap::printwrap(";\n");
  exit 0;
}


{
  # terdragon graph diameter
  #          0 1 2 3  4  5  6  7
  # diameter 1,3,5,8,13,22,39,66
  # smaller than A052968 2^(n-1)+n+1 at k=7
  # A052968(k) = 2^(k-1) + k + 1;
  # vector(10,k,k--; A052968(k))
  # smaller than A213710 iterate n - count1bits(n) at k=7

  # k=7 diameter=66 radius=66 shortest=1 longest=66 avg=19.9223751744172
  # longest 0+0 2+0 1+1 ... -82-28 -83-27 -81-27 is start to end

  my @values;
  foreach my $k (7) {
    my $graph = terdragon_Graph($k);
    my $apsp = $graph->APSP_Floyd_Warshall;
    my $diameter      = $apsp->diameter;
    my $radius        = $apsp->diameter;
    my $shortest_path = $apsp->shortest_path;
    my $average_path  = $apsp->average_path_length;
    my $longest_path  = $apsp->longest_path;
    my @longest_path  = $apsp->longest_path;
    my @centre        = $apsp->center_vertices;
    print "k=$k  diameter=$diameter radius=$radius shortest=$shortest_path longest=$longest_path avg=$average_path\n";
    print "  centre  ",join(' ',@centre),"\n";
    print "  longest ",join(' ',@longest_path),"\n";
    push @values, $diameter;
  }
  shift @values;
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}

{
  # graph edge degree counts
  # EdgeCount11(k) =          if(k==0,1,0);
  # EdgeCount12(k) =          if(k>=1,2,0);
  # EdgeCount22(k) =       2^(k-2);        \\ k>=3
  # EdgeCount24(k) =       2^(k-1) + 2;    \\ k>=3
  # EdgeCount26(k) =       2^k     - 4;    \\ k>=3
  # EdgeCount44(k) =       2^(k-2) + 4;    \\ k>=3
  # EdgeCount46(k) =       3*2^k   - 18;   \\ k>=3
  # EdgeCount66(k) = 3^k - 5*2^k   + 14;   \\ k>=3
  # vector(9,k,k--; EdgeCount46(k))

  my $path = Math::PlanePath::TerdragonCurve->new;

  my @type_list = ('1,1', '1,2',
                   '2,2', '2,4', '2,6',
                   '4,4', '4,6',
                   '6,6');
  print "     ";
  foreach my $type (@type_list) {
    printf " %6s", $type;
  }
  print "\n";

  my @values;
  foreach my $k (0 .. 8) {
    my %degree = ('0,0' => 1);
    my %count;
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my ($prev_x,$prev_y) = $path->n_to_xy($n_lo);
    foreach my $n ($n_lo+1 .. $n_hi) {
      ### $n
      my ($x,$y) = $path->n_to_xy($n);

      my $from_degree = $degree{"$prev_x,$prev_y"};
      my $to_degree = ($degree{"$x,$y"}
                       //= do {
                         my @n_list = $path->xy_to_n_list($x,$y);
                         @n_list = grep {$_ <= $n_hi} @n_list;
                         2*scalar(@n_list) - ($n==$n_hi);
                       });

      my $d1 = min($from_degree,$to_degree);
      my $d2 = max($from_degree,$to_degree);
      my $type = "$d1,$d2";
      $count{$type}++;
      ### $type

      ($prev_x,$prev_y) = ($x,$y);
    }
    my $i = 0;
    my $total = 0;
    print "k=$k  ";
    foreach my $type (@type_list) {
      my $value = (delete $count{$type}) || 0;
      printf " %6d", $value;
      push @{$values[$i++]}, $value;
      $total += $value;
    }
    printf "  = %6d\n", $total;
    if (%count) { die keys %count; }
  }
  foreach my $i (2 .. $#values) {
    require Math::OEIS::Grep;
    Math::OEIS::Grep->search(array => $values[$i], verbose=>1,
                             name => $type_list[$i]);
  }
  exit 0;
}




{
  # multi-N at X,Y generating
  my $path = Math::PlanePath::TerdragonCurve->new;
  my $width = 11;
  my $count = 0;

  sub i_to_n_list {
    my ($i) = @_;
    my @digits = digit_split_lowtohigh($i,3);
    ### digits: join(' ',reverse @digits)
    my @n_list;
    foreach my $offset (0, 1, 2) {
      ### $offset
      my @new_digits;
      my $ones = $offset;
      $new_digits[0] = $digits[0];
      foreach my $i (1 .. $#digits+3) {
        my $digit = $digits[$i] || 0;
        my $new_digit = ($digit + $ones) % 3;
        ### at: "i=$i digit=$digit ones=$ones   new_digit=$new_digit"
        $new_digits[$i] = $new_digit;
        $ones += ($new_digit == 1);
      }
      ### final ones: $ones
      my $new_n = digit_join_lowtohigh(\@new_digits,3);
      if ($ones % 3 != 0) {
        $new_n = -$new_n;
      }
      push @n_list, $new_n;
    }
    return @n_list;
  }

  foreach my $i (Math::BaseCnv::cnv('210202',3,10),
                 Math::BaseCnv::cnv('210211',3,10), # other_n
                 3**5 .. 3**6,
                ) {
    next if $i % 3 == 0;
    next if $i % 3 == 1;
    my @n_list = i_to_n_list($i);
    next unless @n_list;

    my $count = grep {$_>=0} @n_list;
    if ($count == 0) {
      printf "SKIP  i= %*s\n", $width, Math::BaseCnv::cnv($i,10,3);
      # next;
    } else {
      printf "i= %*s\n", $width, Math::BaseCnv::cnv($i,10,3);
    }
    foreach my $n (@n_list) {
      my ($x,$y) = ($n >= 0 ? $path->n_to_xy($n) : (-1,-1));
      printf "n= %*s  at $x,$y\n", $width, Math::BaseCnv::cnv($n,10,3);
    }
    print "count=$count\n\n";
    last if $count++ > 6;
  }
  exit 0;
}

{
  my @list = i_to_n_list(2);
  ### @list
  exit 0;
}

{
  # Eboth
  my %table = (0  => { 0 =>  0,  1 => 1,  2 =>  2 },
               1  => { 0 =>  1,  1 => 4,  2 =>  2 },
               2  => { 0 =>  6,  1 => 1,  2 =>  2 },
               3  => { 0 =>  7,  1 => 4,  2 =>  2 },
               4  => { 0 =>  1,  1 => 4,  2 =>  5 },
               5  => { 0 =>  7,  1 => 4,  2 =>  5 },
               6  => { 0 =>  6,  1 => 3,  2 =>  6 },
               7  => { 0 =>  7,  1 => 5,  2 =>  6 },
              );
  require MyFSM;
  my $fsm = MyFSM->new(table => \%table,
                       initial => 0,
                       accepting => { 4 => 'L', 5 => 'L',
                                      6 => 'R', 7 => 'R' },
                      );
  $fsm->simplify;
  print $fsm->as_graph6;
  print $fsm->as_sparse6;
  $fsm->show;
  $fsm->view;
  print "\n";

  $fsm = $fsm->reverse;
  $fsm->simplify;
  $fsm->show;
  $fsm->view;
  print $fsm->as_graph6;
  print $fsm->as_sparse6;
  # MyFSM::fsm_replace($fsm, '10,10,10,10,10,10,10,10','right');


  # STATE: foreach my $state ($fsm->states) {
  #    next STATE unless $state =~ /^(right|(10|11)(,(10|11))*)$/;
  #    foreach my $edge (MyFSM::fsm_edges($fsm)) {
  #      my $new_state = $fsm->{$state}->{$edge};
  #      next STATE unless $new_state =~ /^(right|(10|11)(,(10|11))*)$/;
  #    }
  #    MyFSM::fsm_replace($fsm, $state,'right');
  #  }
  #  $fsm->show;
  #  print "\n";

  # STATE: foreach my $state (MyFSM::fsm_states($fsm)) {
  #    next STATE unless $state =~ /^(left|(5|7)(,(5|7))*)$/;
  #    foreach my $edge (MyFSM::fsm_edges($fsm)) {
  #      my $new_state = $fsm->{$state}->{$edge};
  #      next STATE unless $new_state =~ /^(left|(5|7)(,(5|7))*)$/;
  #    }
  #    MyFSM::fsm_replace($fsm, $state,'left');
  #  }
  #  $fsm->show;
  #  print "\n";
  #
  # STATE: foreach my $state (MyFSM::fsm_states($fsm)) {
  #    next STATE unless $state =~ /^(non|(0|1|2|3)(,(0|1|2|3))*)$/;
  #    foreach my $edge (MyFSM::fsm_edges($fsm)) {
  #      my $new_state = $fsm->{$state}->{$edge};
  #      next STATE unless $new_state =~ /^(non|(0|1|2|3)(,(0|1|2|3))*)$/;
  #    }
  #    MyFSM::fsm_replace($fsm, $state,'non');
  #  }
  #  $fsm->show;
  #  print "\n";

  exit 0;
}

{
  # reverse fsm as an NFA -- not working

  my $forward = { 0  => { 0 =>  0,  1 => 1,  2 =>  2 },
                  1  => { 0 =>  1,  1 => 5,  2 =>  2 },
                  2  => { 0 => 10,  1 => 1,  2 =>  2 },
                  3  => { 0 => 11,  1 => 5,  2 =>  2 },
                  5  => { 0 =>  1,  1 => 5,  2 =>  7 },
                  7  => { 0 => 11,  1 => 5,  2 =>  7 },
                  10 => { 0 => 10,  1 => 3,  2 => 10 },
                  11 => { 0 => 11,  1 => 7,  2 => 10 },
                };
  my $reverse = {};
  foreach my $state (keys %$forward) {
    my $row = $forward->{$state};
    foreach my $digit (keys %$row) {
      my $new_state = $row->{$digit};
      $reverse->{$new_state}->{$digit} = $state;
    }
  }
  require MyFSM;
  my $fsm = MyFSM->new(table => $reverse,
                       initial => 0);
  $fsm->view;
  exit 0;
}

{
  # EpredL or EpredR for middle segment recognition

  require MyFSM;
  my $EpredL_fsm = MyFSM->new
    (table => { ls0  => { 0 => 'ls0',  1 => 'ls1',  2 => 'ls0' },
                ls1  => { 0 => 'ls1',  1 => 'ls2',  2 => 'ls0' },
                ls2  => { 0 => 'ls1',  1 => 'ls2',  2 => 'ls2' },
              },
     accepting => { ls1 => 'middle' },
     initial   => 'ls0',
    );
  $EpredL_fsm->show;

  my $EpredL_rev = $EpredL_fsm->reverse;
  $EpredL_rev->simplify;
  $EpredL_rev->show;
  #  $EpredL_rev->view;


  # EpredR
  my $EpredR_fsm = MyFSM->new
    (table => { rs0  => { 0 => 'rs0',  1 => 'rs0',  2 => 'rs1' },
                rs1  => { 0 => 'rs2',  1 => 'rs0',  2 => 'rs1' },
                rs2  => { 0 => 'rs2',  1 => 'rs1',  2 => 'rs2' },
              },
     accepting => { rs2 => 'right' },
     initial   => 'rs0',
    );
  $EpredR_fsm->show;

  my $EpredR_rev = $EpredR_fsm->reverse;
  $EpredR_rev->simplify;
  $EpredR_rev->show;
  #  $EpredR_rev->view;


  # middle_R
  my $middle_R_fsm = MyFSM->new
    (table => { rs0  => { 0 => 'rs0',  1 => 'rs0',  2 => 'rs1' },
                rs1  => { 0 => 'rs2',  1 => 'rs0',  2 => 'rs1' },
                rs2  => { 0 => 'rs2',  1 => 'rs1',  2 => 'rs2' },
              },
     accepting => { rs1 => 'middle' },
     initial   => 'rs0',
     title     => 'R middle',
    );
  $middle_R_fsm->view;

  my $R_fsm = MyFSM->new
    (table => { R  => { 0 => 'R',  1 => 'M',   2 => 'E' },
                M  => { 0 => 'R',  1 => 'nb',  2 => 'nb' },
                E  => { 0 => 'nb', 1 => 'M',   2 => 'E' },
                nb => { 0 => 'nb', 1 => 'nb',  2 => 'nb' },
              },
     accepting => { nb => 'nb' },
     initial   => 'R',
     title     => 'right boundary',
    );
  $R_fsm->show;
  my $R_rev = $R_fsm->reverse;

  print "\nunion\n";
  my $fsm = $middle_R_fsm->union($R_fsm);
  $fsm->check;
  $fsm->simplify;
  $fsm->show;
  $fsm->view;

  # foreach my $state ($fsm->states) {
  #   if ($state =~ /^middle\+/) {
  #     $fsm->{'accepting'}->{$state} = 'middle+nb';
  #   }
  # }
  my $fsm_rev = $fsm->reverse;
  $fsm_rev->simplify;
  $fsm_rev->show;
  $fsm_rev->view;

  exit 0;
}

{
  # right boundary by FSM
  # no 11, 12, 20

  require MyFSM;
  my $R_fsm = MyFSM->new
    (table => { R  => { 0 => 'R',  1 => 'M',   2 => 'E' },
                M  => { 0 => 'R',  1 => 'nb',  2 => 'nb' },
                E  => { 0 => 'nb', 1 => 'M',   2 => 'E' },
                nb => { 0 => 'nb', 1 => 'nb',  2 => 'nb' },
              },
     accepting => { nb => 'nb' },
     initial   => 'R',
    );
  $R_fsm->simplify;
  $R_fsm->show;
  $R_fsm->view;

  my $R_rev = $R_fsm->reverse;
  $R_rev->simplify;
  $R_rev->show;
  $R_rev->view;


  my $path = Math::PlanePath::TerdragonCurve->new;
  foreach my $n (0 .. 100_000) {
    my $f = is_right_boundary_by_fsm($n) ? 1 : 0;
    my $p = $path->_UNDOCUMENTED__n_segment_is_right_boundary($n) ? 1 : 0;
    if ($f != $p || $n <= 9) {
      print "n=$n  $f $p\n";
    }
  }
  exit 0;

  sub is_right_boundary_by_fsm {
    my ($n) = @_;
    my $state = $R_fsm->{'initial'};
    foreach my $digit (reverse digit_split_lowtohigh($n,3)) {
      $state = $R_fsm->transition($state,$digit);
    }
    return $state ne 'nb';
  }
}

{
  # how many cordial labellings
  # start k=1 is 4,40,13824,

  my @values;
  my $path = Math::PlanePath::TerdragonCurve->new;
  foreach my $k (1 .. 4) {
    print "k=$k\n";
    my $P = P($k);
    my $E = 3**$k;  # segments

    my @v_list;
    my %v_hash;
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    foreach my $n ($n_lo .. $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      my $v = "$x,$y";
      if (! defined $v_hash{$v}) {
        push @v_list, $v;
        $v_hash{$v} = $#v_list;
      }
    }
    my $v_count = scalar(@v_list);
    $v_count == $P or die "k=$k  v_count=$v_count but P=$P";

    my @v_adjacent;
    foreach my $i (0 .. $#v_list) {
      my $v = $v_list[$i];
      my ($x,$y) = split /,/, $v;
      my @adjacent;
      foreach my $dir6 (0 .. $#dir6_to_dx) {
        my $dx = $dir6_to_dx[$dir6];
        my $dy = $dir6_to_dy[$dir6];
        my $x2 = $x + $dx;
        my $y2 = $y + $dy;
        my $n = $path->xyxy_to_n_either($x,$y,$x2,$y2);
        next unless defined $n && $n <= $n_hi;
        my $v2 = "$x2,$y2";
        if (defined $v_hash{$v2}) {
          push @adjacent, $v2;
        }
      }
      $v_adjacent[$i] = \@adjacent;
    }

    my $e_count = 0;
    foreach my $i (0 .. $#v_adjacent) {
      my $aref = $v_adjacent[$i];
      @$aref = map {$v_hash{$_}} @$aref;
      @$aref = grep {$_ < $i} @$aref;
      $e_count += scalar(@$aref);
    }
    if ($k < 3) {
      foreach my $i (0 .. $#v_adjacent) {
        my $aref = $v_adjacent[$i];
        print "v=$i  $v_list[$i]  adjacent ";
        foreach my $a (@$aref) {
          print " ",$a;
          if ($a !~ /,/) {
            print "=$v_list[$a]";
          }
        }
        print "\n";
      }
    }
    $e_count == $E or die "e_count = $e_count";

    my $ef_max = ceil($E/2);
    my @ef_count = (0,0);
    my @vf_initial = (floor($P/2),ceil($P/2));
    my @vf_remaining = @vf_initial;
    my $pos = 0;
    my @v_label = (-1);

    my $cordial_count = 0;
    while ($pos >= 0) {
      ### at: "$pos  vfrem=$vf_remaining[0],$vf_remaining[1]  efc=$ef_count[0],$ef_count[1]  ".join('',@v_label[0..$pos])

      # check
      # {
      #   my $vf1_r = $vf_initial[1] - $vf_remaining[1];
      #   my $vf1_c = sum(0,map{max(0,$v_label[$_])} 0 .. $pos);
      #   $vf1_r == $vf1_c
      #     or die "oops, $vf1_r $vf1_c";
      # }
      # {
      #   my @ef_c = (0,0);
      #   foreach my $i (0 .. $pos) {
      #     next if $v_label[$i] < 0;
      #     foreach my $j (@{$v_adjacent[$i]}) {
      #       my $e_label = $v_label[$i] ^ $v_label[$j];
      #       $ef_c[$e_label]++
      #     }
      #   }
      #   ($ef_count[0] == $ef_c[0] && $ef_count[1] == $ef_c[1])
      #     or die "oops ef, @ef_count  recount @ef_c";
      # }

      my $l = $v_label[$pos];
      if ($l >= 0) {
        ### increment: "l=$l"
        $vf_remaining[$l]++;
        foreach my $j (@{$v_adjacent[$pos]}) {
          my $e_label = $l ^ $v_label[$j];
          $ef_count[$e_label]--;
        }
      }
      $l++;
      if ($l <= 1 && $vf_remaining[$l] == 0) { $l++; }
      if ($l <= 1 && $vf_remaining[$l] == 0) { $l++; }
      if ($l >= 2) {
        $pos--;
        ### backtrack to: "pos=$pos"
        next;
      }

      $v_label[$pos] = $l;
      $vf_remaining[$l]--;
      foreach my $j (@{$v_adjacent[$pos]}) {
        my $e = $l ^ $v_label[$j];
        $ef_count[$e]++;
      }
      if ($ef_count[0] > $ef_max || $ef_count[1] > $ef_max) {
        next;
      }

      if ($pos == $#v_list) {
        ### found: "vfrem=$vf_remaining[0],$vf_remaining[1]"

        my $is_cordial = (abs($ef_count[0] - $ef_count[1]) <= 1 ? 1 : 0);
        if ($is_cordial) {
          if ($k < 3) {
            my $sym = aref_is_symmetric(\@v_label) ? "  sym" : "";
            my $bfsym = aref_is_bitflip_symmetric(\@v_label) ? "  bfsym" : "";
            print "  found ",join('',@v_label),"  $ef_count[0],$ef_count[1]  $is_cordial$sym$bfsym\n";
          }
          $cordial_count++;
        }

        ### backtrack from found ...
        $vf_remaining[$l]++;
        foreach my $j (@{$v_adjacent[$pos]}) {
          my $e_label = $l ^ $v_label[$j];
          $ef_count[$e_label]--;
        }
        $pos--;
        next;
      }

      $pos++;  # forward
      $v_label[$pos] = -1;
    }

    print "  cordial $cordial_count\n";
    push @values, $cordial_count;
  }

  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;

  sub aref_is_symmetric {
    my ($aref) = @_;
    foreach my $i (0 .. int($#$aref/2)) {
      if ($aref->[$i] != $aref->[$#$aref-$i]) { return 0; }
    }
    return 1;
  }

  sub aref_is_bitflip_symmetric {
    my ($aref) = @_;
    foreach my $i (0 .. int($#$aref/2)) {
      if ($aref->[$i] != 1 - $aref->[$#$aref-$i]) { return 0; }
    }
    return 1;
  }
}

{
  # EpredL low to high
  #
  my @table;
  BEGIN {
    @table = ([3,1,0], # t0
              [1,2,0], # t1
              [2,2,2], # right
              [3,3,3]);
  }
  sub EpredL_by_low_to_high_sep {
    my ($n) = @_;
    my $m = 0;
    foreach my $digit (digit_split_lowtohigh($n,3)) {
      $m = $table[$m][$digit];
    }
    return $m == 2;
  }

  my $bad = 0;
  my $path = Math::PlanePath::TerdragonCurve->new;
  foreach my $n (0 .. 3**10) {
    my $penc = EpredL_by_path($path,$n);
    # my $denc = EpredL_by_low_to_high_sep($n);
    my $denc = EpredL_by_digits($n);
    if ($penc != $denc) {
      my $n3 = cnv($n,10,3);
      print "n=$n [$n3]  path=$penc  digits $denc\n";
      die if ++$bad > 10;
    }
  }
  print "ok\n";
  exit 0;
}

{
  # EpredL separate fsm

  require MyFSM;
  my $fsm = MyFSM->new
    (table => { ls0  => { 0 => 'ls0',  1 => 'ls1',  2 => 'ls0' },
                ls1  => { 0 => 'ls1',  1 => 'ls2',  2 => 'ls0' },
                ls2  => { 0 => 'ls1',  1 => 'ls2',  2 => 'ls2' },
              },
     accepting => { ls2 => 'left' },
     initial   => 'ls0',
    );
  $fsm->show;
  $fsm->view;

  $fsm = $fsm->reverse;
  $fsm->show;
  $fsm->view;

  $fsm->simplify;
  $fsm->show;
  $fsm->view;

  print $fsm->as_graphviz;
  exit 0;
}
{
  # EpredR low to high
  #
  my @table;
  BEGIN {
    @table = ([1,3,0], # m0
              [1,0,2], # m1
              [2,2,2], # right
              [3,3,3]);
  }
  sub EpredR_by_low_to_high_sep {
    my ($n) = @_;
    my $m = 0;
    foreach my $digit (digit_split_lowtohigh($n,3)) {
      $m = $table[$m][$digit];
    }
    return $m == 2;
  }
  sub EpredR_by_digits {
    my ($n) = @_;
    my $n3 = cnv($n,10,3);
    if ($n3 =~ /1$/) { return 0; }
    if ($n3 =~ /.*20/) {
      $n3 = substr($n3,$+[0]);
      if ($n3 =~ /11|12/) { return 0; }
      return 1;
    }
    return 0;
  }

  my $bad = 0;
  my $path = Math::PlanePath::TerdragonCurve->new;
  foreach my $n (0 .. 3**10) {
    my $penc = EpredR_by_path($path,$n);
    # my $denc = EpredR_by_low_to_high_sep($n);
    my $denc = EpredR_by_digits($n);
    if ($penc != $denc) {
      my $n3 = cnv($n,10,3);
      print "n=$n [$n3]  path=$penc  digits $denc\n";
      die if ++$bad > 10;
    }
  }
  print "ok\n";
  exit 0;
}



{
  # non-enclosing and non-boundary

  my $path = Math::PlanePath::TerdragonCurve->new;
  my %pending;
  my $peak_pending = 0;
  foreach my $n (0 .. 3**20) {
    my $enc = EpredR($n);
    my $boundary = $path->_UNDOCUMENTED__n_segment_is_right_boundary($n);
    if ($boundary) {
      if (%pending) {
        die "oops", keys %pending;
      }
      if ($enc) {
        die "oops, both boundary and enc";
      }

    } elsif ($enc) {
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);

      my $pending = scalar(keys %pending);
      if ($pending > $peak_pending) {
        $peak_pending = $pending;
        my $n3 = cnv($n,10,3);
        print "n=$n [$n3]  peak pending $peak_pending\n";
      }

      foreach (1,2) {
        $x += $dx;
        $y += $dy;
        ($dx,$dy) = xy_rotate_minus120($dx,$dy);
        my $n = $path->xyxy_to_n_either($x,$y, $x+$dx,$y+$dy)
          // die "oops, n=$n not enclosing";
        delete $pending{$n};
      }

    } else {
      # non-boundary and non-enc
      $pending{$n} = 1;
    }
  }
  exit 0;
}

{
  # enclosure state counts to level
  my @state_to_index;
  $state_to_index[ 0] = 0;
  $state_to_index[ 1] = 1;
  $state_to_index[ 2] = 2;
  $state_to_index[ 3] = 3;
  $state_to_index[ 5] = 4;
  $state_to_index[ 7] = 5;
  $state_to_index[10] = 6;
  $state_to_index[11] = 7;

  my $path = Math::PlanePath::TerdragonCurve->new;
  foreach my $level (0 .. 8) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($level);
    my @count = (0,0,0,0, 0,0,0,0);
    foreach my $n ($n_lo .. $n_hi-1) {
      my $state = n_to_enc_state_by_path($path, $n);
      $count[$state_to_index[$state]]++;
    }
    printf "k=%2d ", $level;
    foreach my $i (0 .. $#count) {
      printf " %3d", $count[$i];
    }
    print "\n";
  }
  exit 0;
}

{
  # pictures of join tree
  #
  # AA(k) = sum(j=0,k-2, 3^j*2^(k-2-j));
  # k=2  3^0*2^0
  # k=3  3^1*2^0 + 3^0*2^1
  # vector(8,k,my(k=k-1); AA(k))
  # vector(8,k,my(k=k-1); AR(k))
  # 1
  # 1 + 3 + 1 == 5
  # 1 + 3 + 1 + 9 + 1 + 3 + 1 == 19
  # 1 + 3 + 1 + 9 + 1 + 3 + 1 + 27 + 1 + 3 + 1 + 9 + 1 + 3 + 1 == 65
  # triple and new 1 in between and ends
  # every V 2-side triangle is a join in the next level

  require Image::Base::GD;
  require Math::Geometry::Planar;
  my $size = 800;
  my $scale = .75;

  my $path = Math::PlanePath::TerdragonCurve->new;
  my @values;
  my $b;
  BEGIN { $b = Math::Complex->make(3/2, 1/2*sqrt(3)) }

  foreach my $k (2 .. 9) {
    my $filename = sprintf "/tmp/join-tree-%02d.png", $k;
    my $image = Image::Base::GD->new (-height => $size,
                                      -width  => $size);
    $image->rectangle(0,0, $size-1,$size-1, 'black');

    my ($n_lo, $n_hi) = $path->level_to_n_range($k+1);
    ### $n_hi

    #     *            *-------*
    #    / \            \     /
    #   / + \            \ + /
    #  /     \            \ /
    # *---+---*            +
    #   X+Y odd         X+Y even
    #
    my $is_enclosed_triangle = sub {
      my ($x,$y) = @_;    # centre
      if (($x+$y) % 2) {
        return (path_xyxy_is_traversed_either($path, $x-1,$y, $x+1,$y, $n_hi)
                && path_xyxy_is_traversed_either($path, $x-1,$y, $x,$y+1, $n_hi)
                && path_xyxy_is_traversed_either($path, $x+1,$y, $x,$y+1, $n_hi));
      } else {
        return (path_xyxy_is_traversed_either($path, $x,$y, $x-1,$y+1, $n_hi)
                && path_xyxy_is_traversed_either($path, $x,$y, $x+1,$y+1, $n_hi)
                && path_xyxy_is_traversed_either($path, $x-1,$y+1, $x+1,$y+1, $n_hi));
      }
    };

    my $consecutive = sub {
      my ($n1, $n2) = @_;
      return (defined $n1
              && defined $n2
              && $n1 < $n_hi
              && $n2 < $n_hi
              && abs($n1-$n2) == 1);
    };

    my $triangle_neighbours = sub {
      my ($x,$y) = @_;
      ### triangle_neighbours(): "$x, $y"
      my @ret;
      if (($x+$y) % 2) {

        #            *       *
        #           / \     / \
        #          /   \   /   \
        #         /             \
        #        *-------*-------*
        #       / \     / \     / \
        #      /   \   / + \   /   \
        #     /     \ /     \ /     \
        #    *-----  *---+---*  -----*         X+Y odd
        #             \     /
        #          /   \   /   \
        #         /     \ /     \
        #        *-------*-------*
        {
          my $n = $path->xyxy_to_n_either($x-1,$y, $x+1,$y);   # bottom
          my $n2 = $path->xyxy_to_n_either($x-1,$y, $x,$y-1);  # left
          if ($consecutive->($n,$n2)) { push @ret, $x-1,$y-1; }
        }
        {
          my $n = $path->xyxy_to_n_either($x-1,$y, $x+1,$y);   # bottom
          my $n2 = $path->xyxy_to_n_either($x+1,$y, $x,$y-1);  # right
          if ($consecutive->($n,$n2)) { push @ret, $x+1,$y-1; }
        }

        {
          my $n = $path->xyxy_to_n_either($x+1,$y, $x,$y+1);    # right
          my $n2 = $path->xyxy_to_n_either($x+1,$y, $x+2,$y+1); # lower
          if ($consecutive->($n,$n2)) { push @ret, $x+2,$y; }
        }
        {
          my $n = $path->xyxy_to_n_either($x+1,$y, $x,$y+1);    # right
          my $n2 = $path->xyxy_to_n_either($x,$y+1, $x+2,$y+1); # upper
          if ($consecutive->($n,$n2)) { push @ret, $x+1,$y+1; }
        }

        {
          my $n = $path->xyxy_to_n_either($x-1,$y, $x,$y+1);    # left
          my $n2 = $path->xyxy_to_n_either($x-1,$y, $x-2,$y+1); # lower
          if ($consecutive->($n,$n2)) { push @ret, $x-2,$y; }
        }
        {
          my $n = $path->xyxy_to_n_either($x-1,$y, $x,$y+1);    # left
          my $n2 = $path->xyxy_to_n_either($x,$y+1, $x-2,$y+1); # upper
          if ($consecutive->($n,$n2)) { push @ret, $x-1,$y+1; }
        }

      } else {
        #       *-------*-------*
        #        \     / \     /
        #         \   /   \   /
        #          \ /     \ /
        #   *-------*-------*-------*
        #    \     / \     / \     /
        #     \   /   \ + /   \   /
        #      \ /     \ /     \ /
        #       *-------+-------*
        #        \     / \     /
        #         \   /   \   /
        #          \ /     \ /
        #           *       *
        {
          my $n = $path->xyxy_to_n_either($x-1,$y+1, $x+1,$y+1);  # top
          my $n2 = $path->xyxy_to_n_either($x-1,$y+1, $x,$y+2);   # left
          if ($consecutive->($n,$n2)) { push @ret, $x-1,$y+1; }
        }
        {
          my $n = $path->xyxy_to_n_either($x-1,$y+1, $x+1,$y+1);  # top
          my $n2 = $path->xyxy_to_n_either($x+1,$y+1, $x,$y+2);   # right
          if ($consecutive->($n,$n2)) { push @ret, $x+1,$y+1; }
        }

        {
          my $n = $path->xyxy_to_n_either($x,$y, $x-1,$y+1);  # left
          my $n2 = $path->xyxy_to_n_either($x,$y, $x-2,$y);   # lower
          if ($consecutive->($n,$n2)) { push @ret, $x-1,$y-1; }
        }
        {
          my $n = $path->xyxy_to_n_either($x,$y, $x-1,$y+1);    # left
          my $n2 = $path->xyxy_to_n_either($x-1,$y+1, $x-2,$y); # upper
          if ($consecutive->($n,$n2)) { push @ret, $x-2,$y; }
        }

        {
          my $n = $path->xyxy_to_n_either($x,$y, $x+1,$y+1); # right
          my $n2 = $path->xyxy_to_n_either($x,$y, $x+2,$y);  # lower
          if ($consecutive->($n,$n2)) { push @ret, $x+1,$y-1; }
        }
        {
          my $n = $path->xyxy_to_n_either($x,$y, $x+1,$y+1);    # right
          my $n2 = $path->xyxy_to_n_either($x+1,$y+1, $x+2,$y); # upper
          if ($consecutive->($n,$n2)) { push @ret, $x+2,$y; }
        }
      }
      ### return triangles: scalar(@ret)/2
      return @ret;
    };

    my $bk_x = 2;
    my $bk_y = 0;
    {
      for (2 .. $k) {
        ($bk_x,$bk_y) = mul($bk_x,$bk_y);
      }
    }
    my $transform = sub {
      my ($x,$y) = @_;
      $y += .5;
      {
        # centre on b^k, fixed scale
        $x -= $bk_x;
        $y -= $bk_y;
        $y *= sqrt(3);
        my $f = $scale * $size / sqrt(3)**8;
        $x *= $f;
        $y *= $f;
        return ($size/2 + $x, $size/2 - $y);
      }
      {
        # unrotated centre on b^k
        $x -= $bk_x;
        $y -= $bk_y;
        $y *= sqrt(3);
        my $f = $scale * $size / sqrt(3)**$k;
        $x *= $f;
        $y *= $f;
        return ($size/2 + $x, $size/2 - $y);
      }
    };

    {
      # blob at b^(k-1)
      my ($x,$y) = $transform->($bk_x,$bk_y);
      $image->ellipse($x-2,$y-2, $x+2,$y+2, 'white', 1);
    }

    my %seen;
    my ($jet_x,$jet_y) = JET($k-1);
    ### JET: "$jet_x, $jet_y"
    {
      my $jet_enc = $is_enclosed_triangle->($jet_x,$jet_y);
      ### $jet_enc
    }

    my $line_between_triangles = sub {
      my ($x1,$y1, $x2,$y2, $colour) = @_;
      if ($x1 > $x2) {
        ($x1,$y1, $x2,$y2) = ($x2,$y2, $x1,$y1);
      }
      if ((($x1+$y1) % 2) != (($x1+$y1) % 2)) {
        die;
      }
      my ($xm,$ym);
      if (($x1+$y1) % 2) {
        if ($y1 == $y2) {
          #   *   *
          #  / \ / \
          # *-+-*-+-*
          $xm = $x1+1;
          $ym = $y1;
        } elsif ($y2 > $y1) {
          #     *
          #    / \
          #   *-+-*
          #  / \
          # *-+-*
          $xm = $x1;
          $ym = $y2;
        } else {
          #   *
          #  / \
          # *-+-*
          #    / \
          #   *-+-*
          $xm = $x2;
          $ym = $y1;
        }
      } else {
        if ($y1 == $y2) {
          #   *---*---*
          #    \ / \ /
          #     +   +
          $xm = $x1+1;
          $ym = $y1+1
        } elsif ($y2 > $y1) {
          #   *---*
          #    \ /
          # *---+
          #  \ /
          #   +
          $xm = $x2;
          $ym = $y2;
        } else {
          # *---*
          #  \ /
          #   +---*
          #    \ /
          #     +
          $xm = $x1;
          $ym = $y1;
        }
      }
      $y1 += .5;
      $y2 += .5;
      ($x1,$y1) = $transform->($x1,$y1);
      ($x2,$y2) = $transform->($x2,$y2);
      ($xm,$ym) = $transform->($xm,$ym);
      $image->line($x1,$y1, $xm,$ym, $colour);
      $image->line($x2,$y2, $xm,$ym, $colour);
    };

    my @pending = ($jet_x,$jet_y);
    my $nodes = 0;
    my $colour = 'green';
    while (@pending) {
      ### @pending
      my $x = shift @pending;
      my $y = shift @pending;
      next if $seen{"$x,$y"}++;
      $nodes++;
      ### node at: "$x, $y"

      my @new = $triangle_neighbours->($x,$y);
      my $new_count = scalar(@new) / 2;
      while (@new) {
        my $x2 = shift @new;
        my $y2 = shift @new;
        next unless $is_enclosed_triangle->($x2,$y2);

        # if ($colour eq 'green') {
        #   $J_direction = Math::Complex->make($x2-$x, $y2-$y);
        # }

        $line_between_triangles->($x,$y, $x2,$y2, $colour);
        $colour = 'red';
        push @pending, $x2,$y2;
      }
    }

    my $nodes3 = cnv($nodes,10,3);
    print "k=$k  $nodes [$nodes3]\n";
    $image->save($filename);
    push @values, $nodes;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;

  sub JET {
    my ($k) = @_;
    my ($x,$y) = (2,0);
    for (1 .. $k) {
      ($x,$y) = mul($x,$y);
    }
    return ($x-1,$y);
  }
  # CHECK {
  #   ### JEC(2): JEC(2)
  #   ### JEQ(2): "".JEQ_complex(2)
  #   my ($x,$y) = JEC(0);   $x == 0 && $y == 0 or die;
  #   ($x,$y) = JEC(1);   $x == 0 && $y == 0 or die;
  #   ($x,$y) = JEC(2);   $x == -1 && $y == 1 or die;
  #   # -2, \;
  #   # -3 {-} i, \;
  #   # -2 {-} 4 i, \;
  #   #  1 {-} 5 i, \;
  #   #  6 {-} 4 i, \;
  # }
  # sub JEQ_complex {
  #   my ($k) = @_;
  #   return (Math::Complex->make(3/5, 1/5) * $b**$k
  #           - Math::Complex->make(1/10, -3/10) * (-1)**$k);
  # }
}

{
  # segment enclosure run lengths which occur

  # Epred
  # n=385  ones=12
  # n=871  ones=12
  # n=1114  ones=12
  # n=1141  ones=12

  # EpredR
  # n=15        zeros=7..14 [21]..[112]       zeros max 8
  # n=33        zeros=25..32 [221]..[1012]
  # n=58  ones=56..57 [2002]..[2010]

  my @seen_ones;
  my @seen_zeros;
  my $prev = 0;
  my $ones = 0;
  my $ones_start = 0;
  my $ones_end = 0;
  my $zeros = 0;
  my $zeros_start = 0;
  my $zeros_end = 0;
  for (my $n = 0; ; $n++) {
    my $enc = n_to_enc_by_digits_low($n);
    # my $enc = EpredR_by_digits_low($n);
    if ($prev) {
      if ($enc) {
        # 1,1
        $ones++;
        $ones_end = $n;
      } else {
        # 1,0
        unless ($seen_ones[$ones]++ > 3) {
          my $start3 = cnv($ones_start,10,3);
          my $end3 = cnv($ones_end,10,3);
          print "n=$n  ones=$ones  $ones_start..$ones_end [$start3]..[$end3]\n";
        }
        $zeros = 1;
        $zeros_start = $zeros_end = $n;
      }
    } else {
      if ($enc) {
        # 0,1
        unless ($seen_zeros[$zeros]++ > 3) {
          my $start3 = cnv($zeros_start,10,3);
          my $end3 = cnv($zeros_end,10,3);
          print "n=$n        zeros=$zeros  $zeros_start..$zeros_end [$start3]..[$end3]\n";
        }
        $ones = 1;
        $ones_start = $ones_end = $n;
      } else {
        # 0,0
        $zeros++;
        $zeros_end = $n;
      }
    }
    $prev = $enc;
  }
  print "\n";
  exit 0;

  sub EpredR_by_digits_low {
    my ($n) = @_;
    return n_to_encRL_by_digits_low($n) == 1 ? 1 : 0;
  }
}

{
  # segment enclosure states -- low to high by mappings, incl graph

  #        11222  right  11
  #     10001222  right  10
  # 022220001222  non
  #      2000222  left   20
  #     11000222  non    11
  #  12221000222  non    12

  my @all_states = (0,1,2,3,5,7,10,11);

  my @identity = (0) x 12;
  foreach my $state (@all_states) {
    $identity[$state] = $state;
  }

  my @expand;
  $expand[ 0] = [  0, 1, 2];
  $expand[ 1] = [  1, 5, 2];
  $expand[ 2] = [ 10, 1, 2];
  $expand[ 3] = [ 11, 5, 2];
  $expand[ 5] = [  1, 5, 7];
  $expand[ 7] = [ 11, 5, 7];
  $expand[10] = [ 10, 3,10];
  $expand[11] = [ 11, 7,10];

  my @digit_map = ([(0) x 12], [(0) x 12], [(0) x 12]);
  foreach my $digit (0,1,2) {
    foreach my $state (@all_states) {
      $digit_map[$digit]->[$state] = $expand[$state]->[$digit];
    }
  }

  sub state_map_product {
    my ($s, $t) = @_;
    my @new = (0) x 12;
    foreach my $state (@all_states) {
      my $n = $state;
      $n = $s->[$n];
      $n = $t->[$n];
      $new[$state] = $n;
    }
    return \@new;
  }
  sub state_map_decided {
    my ($s) = @_;
    {
      # enclosure left and right
      my $any_non = grep {$_>0 && $_ < 5} @$s;
      my $any_right = grep {$_ >= 10} @$s;
      my $any_left  = grep {$_ == 5 || $_ == 7} @$s;
      return ($any_non && ! $any_left && ! $any_right ? " =non"
              : $any_left && ! $any_non && ! $any_right ? " =left"
              : $any_right && ! $any_non && ! $any_left ? " =right"
              : '');
    }
    {
      # enclosure either side
      my $any_non = grep {$_>0 && $_ < 5} @$s;
      my $any_enc = grep {$_ >= 5} @$s;
      return ($any_non && ! $any_enc ? " =non"
              : $any_enc && ! $any_non ? " =enc"
              : '');
    }
  }
  sub state_map_string {
    my ($s) = @_;
    return join(',',@$s) . state_map_decided($s);

  }
  sub state_map_uniqs {
    my ($s) = @_;
    my %s;
    foreach my $state (@$s) {
      $s{$state} = 1;
    }
    delete $s{0};
    return join(',', sort {$a<=>$b} keys %s);
  }

  my %seen;
  my @m;
  sub state_map_to_number {
    my ($s) = @_;
    my $key = state_map_string($s);
    my $num = $seen{$key};
    if (! defined $num) {
      push @m, $s;
      $num = $#m;
      $seen{$key} = $num;
    }
    return $num;
  }

    my @pending = (\@identity);
    while (@pending) {
      my $s = shift @pending;
      my $num = state_map_to_number($s);
      my $key = state_map_string($s);
      my $uniq = state_map_uniqs($s);
      my $decided = state_map_decided($s);
      printf "m%-2d %-37s [%s]\n", $num, $key, $uniq;
      next if $decided;

      foreach my $digit (0,1,2) {
        my $t = state_map_product($digit_map[$digit], $s);
        my $tkey = state_map_string($t);
        my $already = '';
        if (defined $seen{$tkey}) {
          $already = '  (seen)';
        } else {
          push @pending, $t;
        }
        my $tnum = state_map_to_number($t);
        print "  $digit -> m$tnum$already\n";
      }
      # print "pending $#pending\n";
    }
    print "total maps ",scalar(@m),"\n";

    # Graph
    require Graph::Easy;
    my $graph = Graph::Easy->new(timeout => 9999);
    my $node_number = 0;
    foreach my $i (0 .. $#m) {
      my $s = $m[$i];
      my $sdecided = state_map_decided($s);
      next if $sdecided;
      my $sname = state_map_name($i);
      foreach my $digit (0,1,2) {
        my $t = state_map_product($digit_map[$digit], $s);
        my $tdecided = state_map_decided($t);
        if ($tdecided) {
          my $node = $graph->add_node($node_number++);
          $node->set_attribute('label',$tdecided);
          $graph->add_edge($sname, $node, $digit);
        } else {
          my $ti = state_map_to_number($t);
          my $tname = state_map_name($ti);
          $graph->add_edge($sname, $tname, $digit);
        }
      }
    }
  sub state_map_name {
    my ($i) = @_;
    my $s = $m[$i];
    my $decided = state_map_decided($s);
    my $zero = ($decided ? '' : "[$s->[0]] ");
    return "m$i -- $zero".state_map_uniqs($s).$decided;
  }

  {
    my $graphviz = $graph->as_graphviz();
    require File::Slurp;
    File::Slurp::write_file('/tmp/terdragon-enc.dot', $graphviz);
    require IPC::Run;
    IPC::Run::run(['dot','-Tps','/tmp/terdragon-enc.dot'],'>','/tmp/terdragon-enc.ps');
    # IPC::Run::run(['dotty','/tmp/terdragon-enc.dot'],'&');
    IPC::Run::run(['gv','--scale=.7','/tmp/terdragon-enc.ps'],'&');
  }

  #print $graph->as_ascii;

  print "type ",$graph->type,"\n";
  print "is_simple: ",$graph->is_simple ? "yes\n" : "no\n";
  print "roots: ",join(' ', map{$_->name} $graph->source_nodes), "\n";

  exit 0;
}

{
  # segment enclosure unit triangle -- check by low, right and left
  #
  my @is_enc = (0) x 16;
  $is_enc[5] = -1;   # left
  $is_enc[7] = -1;   # left
  $is_enc[10] = 1;   # right
  $is_enc[11] = 1;   # right

  foreach my $want (1, -1) {
    foreach my $n (0 .. 17) {
      print n_to_encRL_by_digits_low($n) == $want ? 1 : 0, ",";
      if ($n % 3 == 2) { print " \\; "; }
    }
    print "\n";
    my $count = 0;
    for (my $n = 0; $count < 10; $n++) {
      if (n_to_encRL_by_digits_low($n) == $want) {
        $count++;
        print "$n,";
      }
    }
    print "\n";
  }

  my $bad = 0;
  my $path = Math::PlanePath::TerdragonCurve->new;
  foreach my $n (0 .. 3**10) {
    my $ps = n_to_enc_state_by_path($path, $n);
    my $penc = $is_enc[$ps];
    my $denc = n_to_encRL_by_digits_low($n);
    if ($penc != $denc) {
      my $n3 = cnv($n,10,3);
      print "n=$n [$n3]  path [$ps]=$penc  digits $denc\n";
      die if ++$bad > 0;
    }
  }
  print "ok\n";
  exit 0;

  sub n_to_encRL_by_digits_low {
    my ($n) = @_;
    ### n_to_encRL_by_digits_low(): $n

    {
      # strip all low 2s
      my $low;
      do {
        $low = _divrem_mutate($n,3);
      } while ($low == 2);
      ### $low
      ### $n

      my $prev = $low;
      if ($low == 0) {
        ### right 20 non 11,12 ...
        while ($n) {
          my $digit = _divrem_mutate($n,3);
          if ($digit == 1 && $prev != 0) { return 0; }
          if ($digit == 2 && $prev == 0) { return 1; }
          $prev = $digit;
        }
      } else {
        ### left 11,10 non 02 ...
        while ($n) {
          my $digit = _divrem_mutate($n,3);
          if ($digit == 1 && $prev != 2) { return -1; }
          if ($digit == 0 && $prev == 2) { return 0; }
          $prev = $digit;
        }
      }
      return 0;
    }
  }
}

{
  # boundary triangles in direction
  #
  # dir=0: match 2,8,29,85,211,463,926
  #
  # dir=1: match 3,6,10,15,21,29,45,90,220,561
  # A227430  x^2*(1-x)^3 / ((1-2*x)*(1-x+x^2)*(1-3*x+3*x^2)).
  # A227430 ,0,0,1,3,6,10,15,21,29,45,90,220,561,1365,3095,6555,13110,25126,46971,87381,164921,320001,640002,1309528,2707629,5592405,11450531,
  # x^2*(1-x)^3 / ((1-2*x)*(1-x+x^2)*(1-3*x+3*x^2)) \
  # == 1/6 - (1/12)/(x - 1/2) + (3*x - 1)/(18*x^2 - 18*x + 6) - (x + 1)/(6*x^2 - 6*x + 6)
  #
  # w = quadgen(4*-3); x^2*(1-x)^3 / ((1-2*x)*(1-x+x^2)*(1-3*x+3*x^2)) \
  # == 1/6 + (1/6)/(1-2*x) \
  #    + (-1/12 - 1/12*w)/(1 - (1/2-1/2*w)*x) \
  #    + (-1/12 + 1/12*w)/(1 - (1/2+1/2*w)*x) \
  #    - ( 1/12 - 1/12*w)/(1 - (3/2-1/2*w)*x) \
  #    - ( 1/12 + 1/12*w)/(1 - (3/2+1/2*w)*x)
  # roots 2^k  and period-6 of  sqrt(3)^k  1^k
  #                           3^floor(k/2)

  # dir=2: match 5,15,35,70,126,211,341,561
  #
  # dir=3: match 4,10,20,35,56,85,130,220,440
  #
  # dir=4: match 6,21,56,126,252,463,804
  # A192080 Expansion of 1/((1-x)^6 - x^6).
  #
  # dir=5: match 2,3,4,5,6,8,16,45,130,341,804
  # A119336 Expansion of (1-x)^4/((1-x)^6 - x^6)

  # k= 0   1   0   0    0   0   0
  # k= 1   1   0   0    0   0   1
  # k= 2   1   1   0    0   0   2
  # k= 3   1   3   0    1   0   3
  # k= 4   1   6   1    4   0   4
  # k= 5   1  10   5   10   1   5
  # k= 6   2  15  15   20   6   6
  # k= 7   8  21  35   35  21   8
  # k= 8  29  29  70   56  56  16
  # k= 9  85  45 126   85 126  45
  # k=10 211  90 211  130 252 130
  # k=11 463 220 341  220 463 341
  # k=12 926 561 561  440 804 804
  #

  my $path = Math::PlanePath::TerdragonCurve->new;
  my $side = 'right';
  my @values;
  foreach my $level (0 .. 10) {
    my %seen;
    my ($n_lo, $n_hi) = $path->level_to_n_range($level);
    my @count = (0,0,0, 0,0,0);
    foreach my $n ($n_lo .. $n_hi - 1) {
      my ($x1,$y1) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my ($orig_dx,$orig_dy) = ($dx,$dy);
      if ($side eq 'right') {
        $path->_UNDOCUMENTED__n_segment_is_right_boundary($n) or next;
      } else {
        $path->_UNDOCUMENTED__n_segment_is_right_boundary($n_hi-1 - $n) or next;
        ### left boundary ...
        $x1 += $dx;
        $y1 += $dy;
        $dx = -$dx;
        $dy = -$dy;
      }

      my $x2 = $x1 + $dx;
      my $y2 = $y1 + $dy;

      ($dx,$dy) = xy_rotate_minus120($dx,$dy);
      ### rotated: "$dx,$dy"
      my $x3 = $x2 + $dx;
      my $y3 = $y2 + $dy;

      ($x1,$y1, $x2,$y2, $x3,$y3) = triangle_normalize ($x1,$y1, $x2,$y2, $x3,$y3);
      if ($seen{"$x1,$y1"}++) {
        ### duplicate ...
        next;
      }

      my $num_sides = (path_xyxy_is_traversed_either($path, $x1,$y1, $x2,$y2, $n_hi)
                       + path_xyxy_is_traversed_either($path, $x2,$y2, $x3,$y3, $n_hi)
                       + path_xyxy_is_traversed_either($path, $x3,$y3, $x1,$y1, $n_hi));
      my $dir3;
      if ($num_sides == 1) {
        # 1-side by its segment direction
        $dir3 = dxdy_to_dir3 ($orig_dx,$orig_dy);
      } else {
        # 2-side by its missing segment direction
        # dx,dy of the missing side
        if (! path_xyxy_is_traversed_either($path, $x1,$y1, $x2,$y2, $n_hi)) {
          $dx = $x2 - $x1;
          $dy = $y2 - $y1;
        } elsif (! path_xyxy_is_traversed_either($path, $x2,$y2, $x3,$y3, $n_hi)) {
          $dx = $x3 - $x2;
          $dy = $y3 - $y2;
        } elsif (! path_xyxy_is_traversed_either($path, $x3,$y3, $x1,$y1, $n_hi)) {
          $dx = $x3 - $x1;
          $dy = $y3 - $y1;
        }
        if ($dx > 0 && $dy != 0) { ($dx,$dy) = (-$dx,-$dy); }
        $dir3 = dxdy_to_dir3 ($dx,$dy);
        ### $dir3
      }
      if ($num_sides == 2) {
        $dir3 += 3;
      }
      # $dir3 += $level;
      $dir3 %= 6;
      $count[$dir3]++;
    }
    my $total = sum(@count);

    # RT1 + RT2
    foreach my $i (0 .. 2) {
      push @count, $count[0+$i] + $count[3+$i];
    }

    # cumulative
    foreach my $i (0 .. 5) {
      push @count, $count[$i] + ($values[$i]->[-1] || 0);
    }

    printf "k=%2d  %3d %3d %3d  %3d %3d %3d     %3d %3d %3d     %3d %3d %3d  %3d %3d %3d\n", $level, @count;
    foreach my $i (0 .. $#count) {
      push @{$values[$i]}, $count[$i];
    }
  }

  require Math::OEIS::Grep;
  foreach my $i (0 .. $#values) {
    my $aref = $values[$i];
    while (@$aref && $aref->[0] < 2) {
      shift @$aref;
    }
    Math::OEIS::Grep->search (array => $aref, verbose => 1,
                              name => "dir=$i");
  }
  exit 0;
}
{
  # Single and double-visited points in direction
  #
  # d=0     0,0,0,0, 2,10,30,70,140,252,422
  #
  # d=1     0,0,0,0, 0, 0, 2,14, 56,168,420
  # 2*A008489 Expansion of (1-x^7)/(1-x)^7.
  #
  # singles
  # d=2     0,2,4,6,8,10,12,16,32,90
  # 2*A119336
  # 1/3/(1-2*x) + (1/2-1/2*x)/(1-3*x+3*x^2) + (1/6+1/6*x)/(1-x+x^2)
  # terms in w12*sqrt(3)
  # vector(10,k, 2*sum(i=0,k,binomial(k,6*i+4)))
  #

  my $path = Math::PlanePath::TerdragonCurve->new;
  my @values;
  foreach my $k (0 .. 9) {
    (my $n_lo, my $n_hi) = $path->level_to_n_range($k);
    my @count = (0,0,0, 0,0,0);
    foreach my $n ($n_lo+1 .. $n_hi - 1) {
      my ($x,$y) = $path->n_to_xy($n);
      my @n_list = $path->xy_to_n_list($x,$y);
      next unless $n == $n_list[0];
      @n_list = grep {$_ <= $n_hi} @n_list;
      if (@n_list == 1) {
        # direction of absent cross
        #
        #  *   *   *   d=0
        #     / \
        #    *   *
        #
        my ($dx,$dy) = $path->n_to_dxdy($n);
        foreach (0 .. 5) {
          ($dx,$dy) = xy_rotate_minus60($dx,$dy);
          if (! path_xyxy_is_traversed_either($path, $x,$y, $x+$dx,$y+$dy, $n_hi)
              && ! path_xyxy_is_traversed_either($path, $x,$y, $x-$dx,$y-$dy, $n_hi)) {
            last;
          }
        }
        my $d = (dxdy_to_dir3_maybe($dx,$dy)
                 // dxdy_to_dir3_maybe(-$dx,-$dy)
                 // die);
        $count[$d]++;
        ### single: "n=$n d=$d"
      } elsif (@n_list == 2) {
        # direction of cross piece
        #
        #  *---*---*   d=0
        #     / \
        #    *   *
        #
        my ($dx,$dy) = $path->n_to_dxdy($n);
        foreach (0 .. 5) {
          ($dx,$dy) = xy_rotate_minus60($dx,$dy);
          if (path_xyxy_is_traversed_either($path, $x,$y, $x+$dx,$y+$dy, $n_hi)
              && path_xyxy_is_traversed_either($path, $x,$y, $x-$dx,$y-$dy, $n_hi)) {
            last;
          }
        }
        my $d = (dxdy_to_dir3_maybe($dx,$dy)
                 // dxdy_to_dir3_maybe(-$dx,-$dy)
                 // die);
        $d += 3;
        $count[$d]++;
      }
    }
    #      %3d %3d %3d  %3d %3d %3d

    # S + D
    foreach my $i (0 .. 2) {
      push @count, $count[0+$i] + $count[3+$i];
    }

    # # S+D twice
    # foreach my $i (0 .. 2) {
    #   push @count, ($count[(1+$i)%3] + $count[(2+$i)%3] + $count[(1+$i)%3 + 3] + $count[(2+$i)%3 + 3] + ($i==0?2:0))
    #     / 2;
    # }

    printf "k=%2d  %3d %3d %3d  %3d %3d %3d     %3d %3d %3d\n", $k, @count;
    foreach my $i (0 .. $#count) {
      push @{$values[$i]}, $count[$i];
    }

    # my $got_S = sum(@count[0..2]);
    # my $want_S = S($k);
    # $got_S == $want_S or die "total $got_S want S($k)=$want_S";
    my $got_D = sum(@count[3..5]);
    my $want_D = D($k);
    $got_D == $want_D or die "total $got_D want S($k)=$want_D";
  }

  require Math::OEIS::Grep;
  foreach my $i (0 .. $#values) {
    my $aref = $values[$i];
    while (@$aref && $aref->[0] < 2) {
      shift @$aref;
    }
    Math::OEIS::Grep->search (array => $aref, verbose => 1,
                              name => "dir=$i");
  }
  exit 0;
}


{
  # segment enclosure run lengths sequence
  my $func = sub {
    my ($n) = @_;
    return 1 - n_to_enc_by_digits_low($n);
  };
  my @values;
  my $n = 0;
  my $prev = $func->($n);
  my $ones = ($prev ? 1 : 0);
  my $count = 0;
  for ( ; ; $n++) {
    my $enc = $func->($n);
    if ($prev) {
      if ($enc) {
        # 1,1
        $ones++;
      } else {
        # 1,0
        print "$ones,";
        push @values, $ones;
        last if @values > 50;
      }
    } else {
      if ($enc) {
        # 0,1
        $ones = 1;
      } else {
        # 0,0
      }
    }
    $prev = $enc;
  }
  print "\n";
  require Math::OEIS::Grep;
  shift @values;
  shift @values;
  Math::OEIS::Grep->search (array => \@values, verbose => 1);
  exit 0;
}

{
  # segment enclosure predicate samples
  # 0,0,0, 0,1,0, 1,0,0,  0,1,0, 0,1,1,1,0,0,1,0,1,0,1,0,1,0,0,0,1,0,

  for (my $n = 0; $n < 30; $n++) {
    my $enc = n_to_enc_by_digits_low($n);
    print "$enc,";
  }
  print "\n";
  exit 0;
}
{
  # segment enclosure unit triangle -- check by low
  #
  my @is_enc = (0) x 16;
  $is_enc[5] = 1;
  $is_enc[7] = 1;
  $is_enc[10] = 1;
  $is_enc[11] = 1;

  my $path = Math::PlanePath::TerdragonCurve->new;
  foreach my $n (0 .. 3**10) {
    my $ps = n_to_enc_state_by_path($path, $n);
    my $penc = $is_enc[$ps];
    my $denc = n_to_enc_by_digits_low($n);
    if ($penc != $denc) {
      my $n3 = cnv($n,10,3);
      die "n=$n [$n3]  path [$ps]=$penc  digits $denc";
    }
  }
  print "ok\n";
  exit 0;

  # cf Tperm 10 <-> 20
  #    12,11,20 becomes 12,11,10   seek lowest 1
  # with flip  02 <-> 12
  #    10,11,02 becomes 10,11,12   seek lowest 1

  sub n_to_enc_by_digits_low {
    my ($n) = @_;
    ### n_to_enc_by_digits_low(): $n

    {
      # strip all low 2s
      my $low;
      do {
        $low = _divrem_mutate($n,3);
      } while ($low == 2);

      my $prev = $low;
      while ($n) {
        my $digit = _divrem_mutate($n,3);
        if ($low == 1) {
          $digit = 2 - $digit;  # flip 0<->2
        }
        # 12,11 -> non,   02 -> enc
        # 01,11 -> enc,   20 -> non   flipped
        # is high to low 12,11 and 20
        if ($digit == 1 && $prev != 0) { return $low; }
        if ($digit == 2 && $prev == 0) { return 1-$low; }
        $prev = $digit;
      }
      return 0;
    }

    {
      # WRONG ...
      while ($n) {
        ### at: "n=".cnv($n,10,3)
        if ($n < 3) {
          return 0;
        }
        if ($n % 3 == 2) {
          $n -= 2;
          $n /= 3;
          next;
        }
        # high to low  21,11 -> non,   02 -> enc   low 0
        # high to low  10,11 -> enc,   02 -> non   low 1
        if ($n % 27 == cnv('210',3,10)
            || $n % 27 == cnv('110',3,10)
            || $n % 27 == cnv('21',3,10)) {
          return 0;
        }
        if  ($n % 27 == cnv('20',3,10)
             || $n % 27 == cnv('101',3,10)
             || $n % 27 == cnv('111',3,10)) {
          return 1;
        }
        $n = ($n % 3) + ($n - ($n%9))/3;
      }
      return 0;
    }

    {
      my @digits = digit_split_lowtohigh($n,3);
      ### digits: (reverse @digits).""

      # strip all low 2s
      while ($#digits >= 0 && $digits[0] == 2) {
        shift @digits;
      }

      my $low = shift @digits || 0;
      my $prev = $low;
      foreach my $digit (@digits) {
        if ($low == 1) {
          $digit = 2 - $digit;  # flip 0<->2
        }
        # 21,11 -> non,   02 -> enc
        if ($prev > 0  && $digit == 1) { return $low; }
        if ($prev == 0 && $digit == 2) { return 1-$low; }
        $prev = $digit;
      }
      return 0;
    }

    {
      my @digits = digit_split_lowtohigh($n,3);
      ### digits: (reverse @digits).""

      # strip all low 2s
      while ($#digits >= 0 && $digits[0] == 2) {
        shift @digits;
      }

      my $low = $digits[0] || 0;
      foreach my $i (0 .. $#digits-1) {
        if ($low == 1) {
          $digits[$i+1] = 2 - $digits[$i+1];  # flip 0<->2
        }
        # 21,11 -> non,   02 -> enc
        if ($digits[$i] > 0  && $digits[$i+1] == 1) { return $low; }
        if ($digits[$i] == 0 && $digits[$i+1] == 2) { return 1-$low; }
      }
      return 0;
    }

    {
      if ($n < 3) { return 0; }
      my @digits = digit_split_lowtohigh($n,3);
      ### digits: (reverse @digits).""

      # strip all low 2s
      while ($#digits && $digits[0] == 2) {
        shift @digits;
      }

      my $low = $digits[0];
      if ($low == 1) {
        # low to high  01,11 -> enc,   20 -> non
        foreach my $i (0 .. $#digits-1) {
          if ($digits[$i] < 2  && $digits[$i+1] == 1) { return 1; }
          if ($digits[$i] == 2 && $digits[$i+1] == 0) { return 0; }
        }

      } elsif ($low == 0) {
        # low to high  21,11 -> non,   02 -> enc
        foreach my $i (0 .. $#digits-1) {
          if ($digits[$i] == 0 && $digits[$i+1] == 2) { return 1; }
          if ($digits[$i] > 0  && $digits[$i+1] == 1) { return 0; }
        }
      }

      return 0;
    }

    {
      if ($n < 3) { return 0; }
      my @digits = digit_split_lowtohigh($n,3);
      ### digits: (reverse @digits).""

      my $low = $digits[0];

      if ($low == 2) {
        # strip all low 2s
        while ($#digits && $digits[0] == 2) {
          shift @digits;
        }

        if ($#digits && $digits[0] == 1) {
          ### at: "222..1   ".join('',@digits)
          # look upwards "01" or "11" -> enc, "20" or end -> non
          foreach my $i (0 .. $#digits-1) {
            ### pair: "$digits[$i] $digits[$i+1]"
            if ($digits[$i]  < 2 && $digits[$i+1] == 1) { return 1; }
            if ($digits[$i] == 2 && $digits[$i+1] == 0) { return 0; }
          }

        } elsif ($#digits && $digits[0] == 0) {
          # look upwards "02" -> enc, "11" or "21" -> non
          foreach my $i (0 .. $#digits-1) {
            if ($digits[$i] == 0 && $digits[$i+1] == 2) { return 1; }
            if ($digits[$i] > 0  && $digits[$i+1] == 1) { return 0; }
          }
        }

      } elsif ($low == 1) {
        if ($#digits >= 1 && $digits[1] == 1) {
          return 1;
        }
        foreach my $i (1 .. $#digits-1) {
          if ($digits[$i] < 2  && $digits[$i+1] == 1) { return 1; }
          if ($digits[$i] == 2 && $digits[$i+1] == 0) { return 0; }
        }

        # if ($#digits >= 1 && $digits[1] == 2) {
        #   ### at: "12"
        #   # look upwards "11" or "01" -> enc, "20" -> non
        #   foreach my $i (1 .. $#digits-1) {
        #     if ($digits[$i] < 2  && $digits[$i+1] == 1) { return 1; }
        #     if ($digits[$i] == 2 && $digits[$i+1] == 0) { return 0; }
        #   }
        #
        # } elsif ($#digits >= 1 && $digits[1] == 0) {
        #   ### at: "10"
        #   # look upwards "01" or "11" -> enc, "20" -> non
        #   foreach my $i (1 .. $#digits-1) {
        #     ### pair: "$digits[$i] $digits[$i+1]"
        #     if ($digits[$i] < 2  && $digits[$i+1] == 1) { return 1; }
        #     if ($digits[$i] == 2 && $digits[$i+1] == 0) { return 0; }
        #   }
        # }

      } elsif ($low == 0) {
        # look upwards "02" -> enc, "11" or "21" or end -> non
        foreach my $i (0 .. $#digits-1) {
          if ($digits[$i] == 0 && $digits[$i+1] == 2) { return 1; }
          if ($digits[$i] > 0  && $digits[$i+1] == 1) { return 0; }
        }
      }

      return 0;
    }
    no Smart::Comments;
  }
}
{
  # segment enclosure direction sequence
  #
  my @values;
  for (my $n = 0; @values < 120; $n++) {
    # my $n = Tperm($n);   # in Tperm order
    my $enc = n_to_enc_by_digits_low($n) || next;
    my $d = $n % 3;
    # next unless $d == 2;
    # push @values, $d;
    push @values, $n;
  }
  print join('',@values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search (array => \@values, verbose => 1);
  exit 0;
}
{
  # ternary permutation digit pairs 10 <-> 20
  # not in OEIS decimal or ternary
  my @values;
  foreach my $n (0 .. 26) {
    my $t = Tperm($n);
    # $t = cnv($t,10,3);
    push @values, $t;
  }
  splice @values, 0,4;
  print join(',',@values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search (array => \@values);
  exit 0;
}

{
  # segment enclosure direction counts
  #
  # d=0 A027649   2*3^n - 2^n
  # d=1 A027649   2*3^n - 2^n
  # d=2 A056182   2*3^n - 2*2^n   at n-1
  # total A056182    2 * (3^n - 2^n)
  #
  my $path = Math::PlanePath::TerdragonCurve->new;
  my @values;
  foreach my $k (0 .. 10) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my @count = (0,0,0);
    foreach my $n ($n_lo .. $n_hi - 1) {
      my $enc = n_to_enc_by_digits_low($n) || next;
      my $d = $n % 3;
      $count[$d]++;
    }
    my $total = sum(@count);
    push @count, $total;

    print "k=$k  ",join('  ',@count),"\n";
    foreach my $i (0 .. $#count) {
      push @{$values[$i]}, $count[$i];
    }
  }

  require Math::OEIS::Grep;
  foreach my $i (0 .. $#values) {
    my $aref = $values[$i];
    while (@$aref && $aref->[0] < 2) {
      shift @$aref;
    }
    Math::OEIS::Grep->search (array => $aref, verbose => 1,
                              name => "d=$i");
  }
  exit 0;
}

{
  # centroid centre of gravity of boundary triangles
  #
  #                    1--------3
  #                     \   +    2, 1-1/3=2/3
  #                      \
  #                       \    mean G[1] = 3/2, 1/6
  # 0-------2      0-------2
  #   \ + /            +     G[0] = 1, -1/3
  #    \ /
  #     *
  #
  # -> 12/16 + 1/13*sqrt(3)*I

  # terdragon curve segment centroid
  #
  #     w      b=w+1
  #    /      /
  #   /      /
  #  o------*
  #
  # w = e^(i*theta)    b = 1+w
  #   = 1/2+sqrt(3)/2*I
  # G(k) = 1/2*G(k-1) + 1/2*(b^k + (-w)*G(k-1))
  # G(k) = (1/2-1/2*w)*G(k-1) + 1/2*b^k
  # G(k) = 1/2*b^k + f*1/2*b^(k-1) + ... + f^(k-1)*1/2*b + f^k*G(0)
  # G(k) = 1/2*b * (b^(k-1) + f*b^(k-2) + ... + f^(k-1))  + f^k*G(0)
  # G(k) = 1/2*b/(b-f) * (b^k - f^k) + f^k*G(0)
  #   f = 1/2-1/2*w  = 1/2*conj(b)
  #     = 1/2 - 1/2*(1/2+sqrt(3)/2)
  #     = 1/4 - sqrt(3)/4
  # 1/2*b/(b-f)
  #    1/2*b / (b - 1/2*conj(b))
  #    (conj(b) - 1/2*b) * 1/2*b / ((b - 1/2*conj(b)) * (conj(b) - 1/2*b))
  #    (conj(b) - 1/2*b) * 1/2*b / abs(b - 1/2*conj(b))^2
  #    (conj(b) - 1/2*b) * 1/2*b / abs(real(b)+I*imag(b) - 1/2*(real(b)-I*imag(b)))^2
  #    (conj(b) - 1/2*b) * 1/2*b / abs(real(b)+I*imag(b) - 1/2*real(b) + 1/2*I*imag(b))^2
  #    (conj(b) - 1/2*b) * 1/2*b / abs(1/2*real(b) + 3/2*I*imag(b))^2
  #    (conj(b) - 1/2*b) * 1/2*b / abs(1/2*b + I*imag(b))^2

  require Math::Complex;
  my $w = Math::Complex->make(-2,sqrt(3));
  my $b = $w + 2;
  print "w=$w b=$b\n";

  my $type = 'segments';
  my $part = 'V';

  my $third = to_bigrat(1) / 3;
  my $half = to_bigrat(1) / 2;
  my $factor = ($type eq 'triangles' ? $third : $half);

  my $path = Math::PlanePath::TerdragonCurve->new;
  my @xvalues;
  my @yvalues;
  my @counts;
  foreach my $k (0 .. 10) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my $x_vstart = 0;
    my $y_vstart = 0;
    if ($part eq 'V') {
      # V part
      $n_lo = $n_hi;
      ($x_vstart,$y_vstart) = $path->n_to_xy($n_lo);
      $n_hi *= 3;
      print "V n=$n_lo to n=$n_hi  $x_vstart,$y_vstart\n";
    }
    my @count = (0,0,0);
    my $x_total = to_bigrat(0);
    my $y_total = to_bigrat(0);
    my %seen_triangle;
    my $count = 0;
    foreach my $n ($n_lo .. $n_hi-1) {
      next unless $path->_UNDOCUMENTED__n_segment_is_right_boundary($n);
      my ($x1,$y1) = $path->n_to_xy($n);
      $x1 -= $x_vstart;
      $y1 -= $y_vstart;
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my $x2 = $x1 + $dx;
      my $y2 = $y1 + $dy;
      ($dx,$dy) = xy_rotate_minus60($dx,$dy);
      my ($x3,$y3) = ($x1+$dx, $y1+$dy);

      if ($type eq 'triangles') {
        ($x1,$y1, $x2,$y2, $x3,$y3) = triangle_normalize($x1,$y1, $x2,$y2, $x3,$y3);
        next if $seen_triangle{"$x1,$y1--$x2,$y2--$x3,$y3"}++;
        $x_total += ($x1+$x2+$x3);
        $y_total += ($y1+$y2+$y3);
      } elsif ($type eq 'segments') {
        if ($k <= 1) {
          print "$x1,$y1 to $x2,$y2\n";
        }
        $x_total += ($x1+$x2);
        $y_total += ($y1+$y2);
      } else {
        die;
      }
      $count++;
    }
    if ($part eq 'V') {
      $count == V($k) or die;
    } else {
      $count == R($k) or die;
    }

    my $gx = $factor * $x_total / $count;
    my $gy = $factor * $y_total / $count;

    push @xvalues, $x_total;
    push @yvalues, $y_total;
    push @counts, $count;

    my ($ex,$ey) = $path->n_to_xy($n_hi);
    my $e = Math::Complex->make($ex, $ey * sqrt(3));
    my $g = Math::Complex->make($gx->numify, $gy->numify * sqrt(3));
    $g /= $e;

    my ($fx,$fy) = GT_by_sum($k);

    print "$k total=$x_total, $y_total  scaled=$gx, $gy  [$fx $fy] $g\n";
  }

  foreach my $k (0 .. $#xvalues) {
    print "($xvalues[$k]/2 + $yvalues[$k]*sqrt3i/2) * $factor/$counts[$k],\n";
  }
  print "\n";

  shift @xvalues;
  shift @xvalues;
  shift @yvalues;
  shift @yvalues;
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search (array => \@xvalues, verbose => 1);
  Math::OEIS::Grep->search (array => \@yvalues, verbose => 1);
  exit 0;

  sub GT_by_parts {
    my ($k) = @_;
    my $gx = to_bigrat(1);
    my $gy = to_bigrat(-1) / 3;
    my $ex = 2;
    my $ey = 0;
    foreach (1 .. $k) {
      ($ex,$ey) = mul($ex,$ey);
      my ($rx,$ry) = xy_rotate_minus120($gx,$gy);
      $rx += $ex;
      $ry += $ey;
      ($gx,$gy) = (($gx+$rx)/2, ($gy+$ry)/2);
    }
    return ($gx,$gy);
  }

  # G(0) = (0+1+(-sqrt(3)/2*I))/3
  #      = 1/3 - sqrt(3)/6*I
  #      = (0+1-w)/3 = (1-w)/3
  # w = -1/2 + sqrt(3)/2*I
  # b = w + 2 = 3/2 + sqrt(3)/2*I
  # sqrt(3)*I = 2w+1
  # G(k) = 1/2*G(k-1) + 1/2*(b^k + w^2*G(k-1))
  # G(k) = (1/2 + w^2/2)*G(k-1) + 1/2*b^k
  #          with f = (1/2 + w^2/2) = -b/2+1 = -w/2
  # G(k) = 1/2*b^k + f*1/2*b^(k-1) + ... + f^(k-1)*1/2*b + f^k*G(0)
  # G(k) = 1/2*b * (b^(k-1) + f*b^(k-2) + ... + f^(k-1))  + f^k*G(0)
  # G(k) = 1/2*b/(b-f) * (b^k - f^k) + f^k*(1-w)/3
  # 1/2*b/(b-f) = 6/13 - sqrt(3)/13*I
  #             = 6/13 - (2*w+1)/13
  #             = 5/13 - 2/13*w
  #             = 5/13 - 2/13*(b-2)
  #             = 9/13 - 2/13*b
  # G(k) = (5/13 - 2/13*w)* (b^k - (-w/2)^k) + (-w/2)^k*(1-w)/3
  # G(k) = (5/13 - 2/13*w)* b^k + (-(5/13 - 2/13*w) +(1-w)/3) *(-w/2)^k)
  # G(k) = (5/13 - 2/13*w)* b^k + (-2/39 - 7/39*w) *(-w/2)^k
  # G(k) = 1/39 * ( (15 - 6*w)* b^k - (2 + 7*w) *(-w/2)^k )
  # G(k) = (5 - 2*w)/13* b^k - (2 + 7*w)/39 *(-w/2)^k
  #        5/13 - 2/13*w
  #        5/13 - 2/13*(-1/2 + sqrt(3)/2 *I)
  #        6/13 - 1/13*sqrt(3)*I
  #        6/13 - 2/13* sqrt(3)/2 *I
  # vector(8,k,my(k=k-1);G(k)/b^k)
  sub GT_by_sum {
    my ($k) = @_;

    {
      my $ex = to_bigrat(2);
      my $ey = to_bigrat(0);
      foreach (1 .. $k) {
        ($ex,$ey) = mul($ex,$ey);
      }
      my ($wex,$wey) = xy_rotate_plus120($ex,$ey);

      my $w2x = to_bigrat(2);
      my $w2y = to_bigrat(0);
      foreach (1 .. $k) {
        ($w2x,$w2y) = xy_rotate_minus60($w2x,$w2y);
      }
      $w2x /= 2**$k;
      $w2y /= 2**$k;

      my ($w2ox,$w2oy) = xy_rotate_plus120($w2x,$w2y);

      return ((5*$ex - 2*$wex )/13 - (2*$w2x + 7*$w2ox) / 39,
              (5*$ey - 2*$wey )/13 - (2*$w2y + 7*$w2oy) / 39);
    }

    {
      if ($k == 0) {
        return (to_bigrat(1),
                to_bigrat(-1) / 3);
      }
      my ($gx,$gy) = GT_by_parts($k-1);
      # G(k) = 1/2*G(k-1) + 1/2*(b^k - w^2*G(k-1))
      #      = (1/2+w^2/2)*G(k-1) + 1/2*b^k
      #      = (1-b/2) * G(k-1) + 1/2*b^k
      my $ex = 2;
      my $ey = 0;
      foreach (1 .. $k) {
        ($ex,$ey) = mul($ex,$ey);
      }
      my ($rx,$ry) = mul($gx,$gy);
      return ($gx - $rx/2 + $ex/2,
              $gy - $ry/2 + $ey/2);
    }
  }
  # G(k) = 1/2*b/(b-f) * b^k - 1/2*b/(b-f) * f^k + f^k*(1-w)/3
  # G(k) = 1/2*b/(b-f) * b^k + (- 1/2*b/(b-f) + (1-w)/3 ) * f^k
  # G(k) = b^k + (1 + (1-w)/3 ) * f^k
}


{
  # segment enclosure states -- low to high

  #
  # 0,1,2,3,5,7,10,11
  #   [0] -> 0,1,10,11
  #   [1] -> 1,3,5,7
  #   [2] -> 2,7,10
  #
  # 0,1,2,3,5,7,10,11
  #   [00]  goes to 0,1,10,11
  #   [01]  goes to 1,3,5,7
  #   [02]  goes to 2,10
  #   [10]  goes to 1,11
  #   [11]  goes to 5
  #   [12]  goes to 2,7
  #   [20]  goes to 10,11
  #   [21]  goes to 1,3,5
  #   [22]  goes to 2,7,10

  my @all_states = (0,1,2,3,5,7,10,11);
  my @digits;
  #                      00   22    001 002 010 012  221
  my @decided_modulus   = (9, 9,     27, 27, 27, 27, 27,       );
  my @decided_remainder = (0, 2*3+2,  1,  2,  3,  5, 2*9+2*3+1, );

  push @decided_modulus, 81;  # 0102
  push @decided_remainder, cnv('102',3,10);

  push @decided_modulus, 81;  # 0121
  push @decided_remainder, cnv('121',3,10);

  push @decided_modulus, 81;  # 0210
  push @decided_remainder, cnv('210',3,10);

  push @decided_modulus, 81;  # 1201
  push @decided_remainder, cnv('1201',3,10);
  push @decided_modulus, 81;  # 2201
  push @decided_remainder, cnv('2201',3,10);

  push @decided_modulus, 81;  # 1212
  push @decided_remainder, cnv('1212',3,10);
  push @decided_modulus, 81;  # 2212
  push @decided_remainder, cnv('2212',3,10);

  push @decided_modulus, 243;  # 02102
  push @decided_remainder, cnv('2102',3,10);

  push @decided_modulus, 243;  # 02210
  push @decided_remainder, cnv('2210',3,10);

  push @decided_modulus, 243;  # 12121
  push @decided_remainder, cnv('12121',3,10);
  push @decided_modulus, 243;  # 22121
  push @decided_remainder, cnv('22121',3,10);

  push @decided_modulus, 243;  # 22210
  push @decided_remainder, cnv('22210',3,10);

  push @decided_modulus, 729;  # 022102
  push @decided_remainder, cnv('22102',3,10);
  push @decided_modulus, 729;  # 222102
  push @decided_remainder, cnv('222102',3,10);

  foreach my $len (1 .. 7) {
    my $tried = 0;
  N: foreach my $n (0 .. 3**$len-1) {
      foreach my $i (0 .. $#decided_modulus) {
        if ($n % $decided_modulus[$i] == $decided_remainder[$i]) {
          next N;
        }
      }

      my @digits = digit_split_lowtohigh($n,3);

      my $states = [ @all_states ];
      my $str = '';
      foreach my $i (reverse 0 .. $len-1) {
        my $digit = $digits[$i] || 0;
        $str .= $digit;
        $states = states_apply_digit($states, $digit);
      }

      my $any_non = grep {$_ < 10} @$states;
      my $any_enc = grep {$_ >= 10} @$states;
      my $decided = ($any_non && ! $any_enc ? "non"
                     : ! $any_non && $any_enc ? "enc"
                     : '');
      if ($decided) {
        push @decided_modulus, 3**$len;
        push @decided_remainder, $n;
      }

      print "[$str]  goes to ",join(',',@$states),"   $decided\n";
      $tried++;
    }
    print "tried $tried  decided $#decided_modulus\n\n";
  }
  exit 0;
}
{
  # segment enclosure states -- low to high

  my @expand;
  BEGIN {
    $expand[ 0] = [  0, 1, 2];
    $expand[ 1] = [  1, 5, 2];
    $expand[ 2] = [ 10, 1, 2];
    $expand[ 3] = [ 11, 5, 2];
    $expand[ 5] = [  1, 5, 7];
    $expand[ 7] = [ 11, 5, 7];
    $expand[10] = [ 10, 3,10];
    $expand[11] = [ 11, 7,10];
  }
  my @rev = ({},{},{});
  foreach my $i (0 .. $#expand) {
    my $aref = $expand[$i] // next;
    foreach my $digit (0,1,2) {
      $rev[$digit]->{$aref->[$digit]} = 1;
    }
  }
  my @all_states = (0,1,2,3,5,7,10,11);

  sub states_apply_digit {
    my ($states, $digit) = @_;
    my %new_states;
    foreach my $state (@$states) {
      $new_states{$expand[$state]->[$digit]} = 1;
    }
    return [ sort {$a<=>$b} keys %new_states ];
  }

  {
    # where 0 goes
    my $states = [ @all_states ];
    $states = states_apply_digit($states, 0);
    print "0 goes to ",join(',',@$states),"\n";
    $states = states_apply_digit($states, 0);
    print "  goes to ",join(',',@$states),"\n";
  }
  {
    # where 2 goes
    my $states = [ @all_states ];
    $states = states_apply_digit($states, 2);
    print "2 goes to ",join(',',@$states),"\n";
    $states = states_apply_digit($states, 2);
    print "  goes to ",join(',',@$states),"\n";
  }


  my @pending = ([ [], [@all_states] ]);
  for (1 .. 7) {
    print scalar(@pending)," pending\n";
    my @new_pending;
    foreach my $elem (@pending) {
      my ($digits,$states) = @$elem;
      my $any_non = grep {$_ < 5} @$states;
      my $any_enc = grep {$_ >= 5} @$states;
      my $decided = ($any_non && ! $any_enc ? "non"
                     : ! $any_non && $any_enc ? "enc"
                     : '');
      print "[",@$digits,"]  ",join(',',@$states)," $decided\n";

      next if $decided;
      next if @$digits > 7;

      foreach my $digit (0,1,2) {
        my @new_digits = ($digit, @$digits);
        if (count_low_zeros_of_aref(\@new_digits) > 2) {
          next;
        }
        if (count_low_twos_of_aref(\@new_digits) > 2) {
          next;
        }
        ### @new_digits
        my %new_states;
        foreach my $start_state (@all_states) {
          my $state = $start_state;
          foreach my $digit (@new_digits) {
            $state = $expand[$state]->[$digit];
          }
          ### state: "$start_state to $state"
          $new_states{$state} = 1;
        }
        my @new_states = sort {$a<=>$b} keys %new_states;
        if (join(',',@$states) eq join(',',@new_states)) {
          print "      digit $digit states unchanged\n";
          # next;
        }
        push @new_pending, [ \@new_digits, \@new_states ];
      }
    }
    @pending = @new_pending;
    print "\n";
  }
  exit 0;

  sub states_determinall_enc {
    my ($aref) = @_;
    return
  }
}

{
  # segment enclosure states check
  #
  my $path = Math::PlanePath::TerdragonCurve->new;
  foreach my $n (0 .. 3**10) {
    my $ps = n_to_enc_state_by_path($path, $n);
    my $ds = n_to_enc_state_by_digits($n);
    if ($ps != $ds) {
      die "path $ps  digits $ds";
    }
  }
  print "ok\n";
  exit 0;
}

{
  # segment enclosure states expansion
  #
  my @expand;
  BEGIN {
    $expand[ 0] = [  0, 1, 2];
    $expand[ 1] = [  1, 5, 2];
    $expand[ 2] = [ 10, 1, 2];
    $expand[ 3] = [ 11, 5, 2];
    $expand[ 5] = [  1, 5, 7];
    $expand[ 7] = [ 11, 5, 7];
    $expand[10] = [ 10, 3,10];
    $expand[11] = [ 11, 7,10];
  }
  my @is_left = (0) x 16;
  $is_left[5] = 1;
  $is_left[7] = 1;

  my @is_right = (0) x 16;
  $is_right[10] = 1;
  $is_right[11] = 1;

  my @is_enc = map {$is_left[$_] | $is_right[$_]} 0 .. 15;

  my @array = (0);
  foreach my $k (1 .. 4) {
    @array = map { @{$expand[$_]} } @array;
    print((map{$is_enc[$_]}@array),"\n\n");
    # print join(' ',@array),"\n\n";
  }
  require Math::OEIS::Grep;
  # @array = map {$is_left[$_]} @array;
  # @array = map {$is_right[$_]} @array;
  @array = map {$is_enc[$_]} @array;
  Math::OEIS::Grep->search (array => \@array, verbose => 1);
  exit 0;

  sub n_to_enc_state_by_digits {
    my ($n) = @_;
    my $state = 0;
    foreach my $digit (reverse digit_split_lowtohigh($n,3)) { # high to low
      $state = $expand[$state]->[$digit];
    }
    return $state;
  }
}

{
  # segment enclosure vs area
  #
  my @is_left = (0) x 16;
  $is_left[5] = 1;
  $is_left[7] = 1;

  my @is_right = (0) x 16;
  $is_right[10] = 1;
  $is_right[11] = 1;

  my $path = Math::PlanePath::TerdragonCurve->new;
  foreach my $k (0 .. 10) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my $AL = 0;
    my $AR = 0;
    foreach my $n ($n_lo .. $n_hi-1) {
      my $state = n_to_enc_state_by_digits($n);
      $AL += $is_left[$state];
      $AR += $is_right[$state];
    }
    my $AR_by_formula = ($k == 0 ? 0 : 3**($k-1) - 2**($k-1));
    print "$k  $AL  $AR  cf $AR_by_formula\n";
  }
  exit 0;
}

{
  # segment enclosure states descent
  #
  my $level = 6;
  my $path = Math::PlanePath::TerdragonCurve->new;
  my ($n_lo, $n_hi) = $path->level_to_n_range($level);

  my @descent;
  foreach my $n ($n_lo .. $n_hi - 1) {
    my $state = n_to_enc_state_by_path($path,$n);
    my $d = join(',', map {sprintf('%2d',n_to_enc_state_by_path($path,3*$n+$_))} 0,1,2);
    if (defined $descent[$state]) {
      if ($descent[$state] ne $d) {
        die "oops $descent[$state] vs $d";
      }
    }
    $descent[$state] = $d;
  }
  foreach my $state (0 .. $#descent) {
    my $d = $descent[$state] // next;
    printf "%2d -> %s\n", $state, $d;
  }
  exit 0;
}
{
  # segment enclosure states
  #
  #        *               none,1,1+4    = 0,1,5  = 3
  #       / \                   2,2+8    =   2,10 = 2
  #      1   4             1 with 2,2+8  = 3,11   = 2
  #     /     \            2 with   1+4  =   7    = 1
  #    *------->                                  ----
  #     \     /                                     8
  #      2   8
  #       \ /
  #        *
  #
  # -->    0 ->  0, 1, 2
  #
  # /
  # -->    1 ->  1, 5, 2
  #
  # -->    2 -> 10, 1, 2
  # \
  #
  # /\
  # -->    5 -> 1, 5, 7                left
  #
  # -->   10 -> 10, 3, 10              right
  # \/
  #
  # /
  # -->    3 -> 11, 5, 2
  # \
  #
  #
  # /\
  # -->    7 -> 11, 5, 7               left
  # \
  #
  # /
  # -->   11 -> 11, 7, 10              right
  # \/
  #
  # states 0, 1, 2, 3,  5, 7, 10, 11 = 8of

  # k=      0    1    2    3   4     5   6    7   8   9    10   11  12  13  14  15
  # k=0     1    0    0    0   0     0   0    0   0   0     0    0   0   0   0   0       1  [1]
  # k=1     1    1    1    0   0     0   0    0   0   0     0    0   0   0   0   0       3  [3]
  # k=2     1    3    3    0   0     1   0    0   0   0     1    0   0   0   0   0       9  [5]
  # k=3     1    8    7    1   0     4   0    1   0   0     5    0   0   0   0   0      27  [7]
  # k=4     1   20   17    5   0    14   0    5   0   0    17    2   0   0   0   0      81  [8]
  # k=5     1   52   43   17   0    44   0   21   0   0    53   12   0   0   0   0     243  [8]
  # k=6     1  140  113   53   0   134   0   77   0   0   161   50   0   0   0   0     729  [8]
  # k=7     1  388  307  161   0   404   0  261   0   0   485  180   0   0   0   0    2187  [8]
  # k=8     1 1100  857  485   0  1214   0  845   0   0  1457  602   0   0   0   0    6561  [8]
  # k=9     1 3172 2443 1457   0  3644   0 2661   0   0  4373 1932   0   0   0   0   19683  [8]
  # k=10    1 9260 7073 4373   0 10934   0 8237   0   0 13121 6050   0   0   0   0   59049  [8]

  # state=1: match 3,8,20,52,140,388,1100,3172,9260
  # [QUARTER]
  # A007689 2^n + 3^n.
  # A007689   ,2,5,13,35,97,275,793,2315,6817,20195,60073,179195,535537,1602515,4799353,14381675,43112257,129271235,387682633,1162785755,3487832977,10462450355,31385253913,94151567435,282446313697,847322163875,

  # state=2: match 3,7,17,43,113,307,857,2443,7073
  # A085279 Expansion of (1-2x-2x^2)/((1-2x)(1-3x))
  # = -1/3 + -1/(2*x - 1) + -1/(9*x - 3)
  # vector(10,k, 2^k + 1/3*3^k)

  # state=3: match 5,17,53,161,485,1457,4373
  # A048473 a(n) = 2*3^n - 1.
  # A048473 ,1,5,17,53,161,485,1457,4373,13121,39365,118097,354293,1062881,3188645,9565937,28697813,86093441,258280325,774840977,2324522933,6973568801,20920706405,62762119217,188286357653,564859072961,

  # state=5: match 4,14,44,134,404,1214,3644,10934
  # A198643 5*3^n - 1
  # A198643 ,4,14,44,134,404,1214,3644,10934,32804,98414,295244,885734,2657204,7971614,23914844,71744534,215233604,645700814,1937102444,5811307334,17433922004,52301766014,156905298044,470715894134,1412147682404,4236443047214,

  # state=7: match 5,21,77,261,845,2661,8237
  # A126645 a(n) is the number of integers k less than 10^n such that the decimal representation of k lacks the digits 1,2,3,4 and 5, at least one of digits 6,7 and at least one of digits 8,9.
  # A126645 ,5,21,77,261,845,2661,8237,25221,76685,232101,700397,2109381,6344525,19066341,57264557,171924741,516036365,1548633381,4646948717,13942943301,41833024205,125507461221,376539160877,1129651037061,

  # state=10: match 5,17,53,161,485,1457,4373,13121
  # A048473 a(n) = 2*3^n - 1.
  # same as state=3

  # state=11: match 2,12,50,180,602,1932,6050
  # A028243 3^(n-1) - 2*2^(n-1) + 1 (essentially 2 * Stirling numbers of second kind).
  # A028243 ,0,0,2,12,50,180,602,1932,6050,18660,57002,173052,523250,1577940,4750202,14283372,42915650,128878020,386896202,1161212892,3484687250,10456158900,31372671002,94126401612,

  my $path = Math::PlanePath::TerdragonCurve->new;
  print "k=   ",(map {sprintf ' %3d', $_} 0..15),"\n";
  my @values;
  foreach my $level (0 .. 10) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($level);

    my @count = (0) x 16;
    foreach my $n ($n_lo .. $n_hi - 1) {
      my $state = n_to_enc_state_by_path($path,$n);
      $count[$state]++;
      # if ($state == 7) { print "7 at n=$n\n"; }
    }
    my $distinct = grep {$_} @count;
    my $total = sum(@count);
    push @count, $total;
    print "k=$level  ",(map {sprintf ' %3d', $_} @count),"  [$distinct]\n";
    foreach my $i (0 .. $#count) {
      push @{$values[$i]}, $count[$i];
    }
  }

  require Math::OEIS::Grep;
  foreach my $i (0 .. $#values) {
    my $aref = $values[$i];
    while (@$aref && $aref->[0] < 2) {
      shift @$aref;
    }
    next unless @$aref;
    Math::OEIS::Grep->search (array => $aref, verbose => 1,
                              name => "state=$i");
  }
  exit 0;

  sub EpredR_by_path {
    my ($path, $n) = @_;
    my $state = n_to_enc_state_by_path($path,$n);
    return $state==10 || $state==11;
  }
  sub EpredL_by_path {
    my ($path, $n) = @_;
    my $state = n_to_enc_state_by_path($path,$n);
    return $state==5 || $state==7;
  }
  sub n_to_enc_state_by_path {
    my ($path, $n) = @_;
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);

    my ($dxR,$dyR) = xy_rotate_minus60($dx,$dy);
    my ($dxL,$dyL) = xy_rotate_plus60($dx,$dy);

    my $state = 0;
    if (path_xyxy_is_traversed_either($path, $x,$y, $x+$dxL,$y+$dyL, $n)) {
      $state |= 1;
    }
    if (path_xyxy_is_traversed_either($path, $x,$y, $x+$dxR,$y+$dyR, $n)) {
      $state |= 2;
    }

    $x += $dx;
    $y += $dy;
    if (path_xyxy_is_traversed_either($path, $x,$y, $x-$dxR,$y-$dyR, $n)) {
      $state |= 4;
    }
    if (path_xyxy_is_traversed_either($path, $x,$y, $x-$dxL,$y-$dyL, $n)) {
      $state |= 8;
    }
    return $state;
  }
}

{
  # boundary segments in direction

  # right boundary
  # dir3=0    1,2,3,4,5,7,14,37,101,256,593
  # A131023 a(1) = 1, a(2) = 2, a(3) = 3, a(4) = 4; for n > 4, a(n) = 5*a(n-1) - 9*a(n-2) + 6*a(n-3)
  #
  # dir3=1    0,1,3,7,14,25,41,64,101,175,350
  #
  # dir3=2    0,0,0,1,5,16,41,91,182,337,593
  # one row of A131022

  unshift @INC, "$FindBin::Bin/../../path/xt";
  require MyOEIS;
  my $path = Math::PlanePath::TerdragonCurve->new;
  my (@v0,@v1,@v2);
  foreach my $level (0 .. 10) {
    (my $n_lo, my $n_hi) = $path->level_to_n_range($level);

    my $points = MyOEIS::path_boundary_points ($path, $n_hi,
                                               lattice_type => 'triangular',
                                               side => 'right');
    my @count = (0,0,0);
    foreach my $i (1 .. $#$points) {
      my $p1 = $points->[$i-1];
      my $p2 = $points->[$i];
      my ($x1,$y1) = @$p1;
      my ($x2,$y2) = @$p2;
      my $dx = $x2-$x1;
      my $dy = $y2-$y1;
      my $dir = dxdy_to_dir3($dx,$dy);
      $count[$dir]++;
    }
    print "level=$level  $count[0] $count[1] $count[2]\n";

    push @v0, $count[0];
    push @v1, $count[1];
    push @v2, $count[2];
  }

  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@v0, name=>'v0');
  Math::OEIS::Grep->search(array => \@v1, name=>'v1');
  Math::OEIS::Grep->search(array => \@v2, name=>'v2');
  exit 0;

  {
    # boundary segments in direction

    my $path = Math::PlanePath::TerdragonCurve->new;
    my $side = 'right';
    my @values;
    foreach my $level (0 .. 10) {
      my %seen;
      my ($n_lo, $n_hi) = $path->level_to_n_range($level);
      my @count = (0,0,0);
      foreach my $n ($n_lo .. $n_hi - 1) {
        my ($x1,$y1) = $path->n_to_xy($n);
        my ($dx,$dy) = $path->n_to_dxdy($n);
        if ($side eq 'right') {
          $path->_UNDOCUMENTED__n_segment_is_right_boundary($n) or next;
        } else {
          $path->_UNDOCUMENTED__n_segment_is_right_boundary($n_hi-1 - $n) or next;
          ### left boundary ...
          $x1 += $dx;
          $y1 += $dy;
          $dx = -$dx;
          $dy = -$dy;
        }

        my $dir3 =dxdy_to_dir3 ($dx,$dy);
        $count[$dir3]++;
      }
      my $total = sum(@count);
      print "k=$level  $count[0] $count[1] $count[2]    total $total\n";
      foreach my $i (0 .. $#count) {
        push @{$values[$i]}, $count[$i];
      }
    }

    require Math::OEIS::Grep;
    foreach my $i (0 .. $#values) {
      my $aref = $values[$i];
      while (@$aref && $aref->[0] < 2) {
        shift @$aref;
      }
      Math::OEIS::Grep->search (array => $aref, verbose => 1,
                                name => "dir=$i");
    }
    exit 0;
  }
}


{
  # moment of inertia
  #
  #        *
  #       /                 __*
  #      /              __--  |
  #     *--+--*        *      |    __*
  #          /                |__--
  #         /                 *
  #        *
  #
  # I(k,alpha+90) = 1/3 * I(k-1,alpha)
  #               + 2/3 * I(k-1,alpha+60deg)
  #               + 2 * 3^(k-1) * (1/2*sqrt(3)^(k-1) * sin() )^2
  # 90+60 = 150 = -30 mod 180
  #
  # I(k,alpha) = 1/3 * I(k-1,alpha-30deg)        alpha+150
  #            + 2/3 * I(k-1,alpha+90deg)
  #            + 2 * 3^(k-1) * (1/2*sqrt(3)^(k-1) * sin(alpha-30deg) )^2
  #
  # P(k,alpha    ) = 1/3 * P(k-1,alpha+150) + 2/3 * P(k-1,alpha+ 90) + 1/2*9^(k-1) * sin^2(alpha+150)
  # P(k,alpha+ 30) = 1/3 * P(k-1,alpha    ) + 2/3 * P(k-1,alpha+120) + 1/2*9^(k-1) * sin^2(alpha    )
  # P(k,alpha+ 60) = 1/3 * P(k-1,alpha+ 30) + 2/3 * P(k-1,alpha+150) + 1/2*9^(k-1) * sin^2(alpha+ 30)
  # P(k,alpha+ 90) = 1/3 * P(k-1,alpha+ 60) + 2/3 * P(k-1,alpha    ) + 1/2*9^(k-1) * sin^2(alpha+ 60)
  # P(k,alpha+120) = 1/3 * P(k-1,alpha+ 90) + 2/3 * P(k-1,alpha+ 30) + 1/2*9^(k-1) * sin^2(alpha+ 90)
  # P(k,alpha+150) = 1/3 * P(k-1,alpha+120) + 2/3 * P(k-1,alpha+ 60) + 1/2*9^(k-1) * sin^2(alpha+120)
  #
  #  0 30 60 90 120 150

  #       *-------*
  #        \
  #         +
  #          \
  #   *---+---*
  # R = abs(b^k/2 - b^(k-1)/2)
  #   = 1/2*abs(b^(k-1)*(b-1))
  #   = 1/2*sqrt(3)^(k-1)
  # R = e^(i*(alpha+60deg))
  #    ____
  #   /    \
  #  /      \
  # *--------180
  # sin(alpha+150) = sin(alpha)*cos(150) + cos(alpha)*sin(150)
  # sin^2(alpha+150) = sin^2(alpha)*cos^2(150) + cos^2(alpha)*sin^2(150) + 2*sin(alpha)*cos(150)
  # sin^2(alpha+150) = sin^2(alpha)*3/4 + cos^2(alpha)*1/4 + 2*sin(alpha)*cos(150)
  # sin^2(alpha+150) = sin^2(alpha)*3/4 + cos^2(alpha)*1/4 + 2*sin(alpha)*cos(150)
  # if alpha multiple of 30 then simplify

  exit 0;
}


{
  # single, double, triple visited
  #
  # singles = 2,4,6,10,18,34,66,130,258,514,1026
  # A133140 a(0) = 2, a(n) = 2^n + 2 for n>=1.
  # 2^k+2
  #
  # doubles = 0,0,2,6,14,30,62,126,254,510,1022
  # A000918 2^n - 2
  #
  # triples = 0,0,0,2,12,50,180,602,1932,6050,18660
  # A028243 3^(n-1) - 2*2^(n-1) + 1
  # = 2*A000392
  #
  # distinct = S+D+T = 2,4,8,18,44,114,308,858,2444,7074,20708
  #
  #-----------
  # permanently remaining single/double/triple
  #
  # singles   = 2,3,4,7,13,25,49,97,193
  # A004119 = 3*2^n + 1
  # singles-1 = 1,2,3,6,12,24,48,96,192
  # is A098011 + 1
  # A098011 = 2^(n-4)+2^(n-3), initial then 3*2^n
  #
  # doubles 0,1,3,6,12,24,48,96,192
  # 3*2^n
  #
  # triples 0,0,1,5,19,65,211,665,2059
  # 3^n - 2^n

  my $path = Math::PlanePath::TerdragonCurve->new;
  my @values;
  foreach my $level (0 .. 10) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($level);
    my @count = (0,0,0);
    my $n_count = 0;
    foreach my $n ($n_lo .. $n_hi) {
      ### $n
      $n_count++;
      my ($x,$y) = $path->n_to_xy($n);
      my @n_list = $path->xy_to_n_list($x,$y);
      @n_list = grep {$_<=$n_hi} @n_list;
      ### @n_list
      next unless $n_list[0] == $n;
      my $num = scalar(@n_list);
      ### $num
      $count[$num-1]++;
    }
    # $count[0]--;
    my $total = sum(@count);

    printf "%2d  %12b %12b %14b  %d of %d\n", $level, @count, $total, $n_count;

    foreach my $i (0 .. $#count) {
      push @{$values[$i]}, $count[$i];
    }
    push @{$values[scalar(@count)]}, $total;
  }
  foreach my $i (0 .. $#values) {
    my $aref = $values[$i];
    require Math::OEIS::Grep;
    Math::OEIS::Grep->search (array => \@$aref, name => $i, verbose => 1);
  }
  exit 0;
}

{
  # ternary permutation at digit 1 add +1 mod 3 to digits below
  my @values;
  foreach my $n (0 .. 80) {
    foreach my $add (1, 2) {
      my @digits = digit_split_lowtohigh($n,3);
      foreach my $i (1 .. $#digits) {
        next unless $digits[$i] == 1;
        foreach my $j (0 .. $i-1) { $digits[$j] += $add; }
      }
      foreach (@digits) { $_ %= 3; }  # mutate array
      my $nperm = digit_join_lowtohigh(\@digits,3);
      my $nperm3 = Math::BaseCnv::cnv($nperm,10,3);
      push @{$values[$add]}, $nperm;   # in decimal
      # push @values, $nperm3;  # in ternary
    }
  }
  my $num_values = scalar(@{$values[1]});
  my $end_values = $#{$values[1]};
  foreach my $n (0 .. 26) {
    my $nperm = $values[1][$n];
    my $nperm3 = Math::BaseCnv::cnv($nperm,10,3);
    my $inv = -1;
    foreach my $i (0 .. $end_values) {
      if ($values[1][$i] == $n) { $inv = $i; }
    }
    my $inv3 = Math::BaseCnv::cnv($inv,10,3);
    push @{$values[3]}, $inv;

    my $two_nperm = $values[2][$n];
    my $two_nperm3 = Math::BaseCnv::cnv($two_nperm,10,3);
    my $two_inv = -1;
    foreach my $i (0 .. $end_values) {
      if ($values[2][$i] == $n) { $two_inv = $i; }
    }
    my $two_inv3 = Math::BaseCnv::cnv($two_inv,10,3);
    push @{$values[4]}, $two_inv;

    printf "%3d  %3d  %3s  inv %3d %3s  two %3d  %3s  inv %3d %3s\n",
      $n, $nperm, $nperm3, $inv, $inv3,
        $two_nperm, $two_nperm3, $two_inv, $two_inv3;
  }

  foreach my $add (1,2, 3,4) {
    my $aref = $values[$add];
    shift @$aref;
    shift @$aref;
    require Math::OEIS::Grep;
    Math::OEIS::Grep->search (array => \@$aref, verbose => 1);
  }
  exit 0;
}

{
  # other N at same X,Y
  my $path = Math::PlanePath::TerdragonCurve->new;
  my $to3 = \&to_balanced3;
  $to3 = sub { Math::BaseCnv::cnv($_[0],10,3) };
  foreach my $n (3**5 .. 3**6) {
    next if $n % 3 == 0;
    my ($x,$y) = $path->n_to_xy($n);
    my @n_list = $path->xy_to_n_list($x,$y);
    next unless @n_list == 3;
    next unless $n == $n_list[0];
    my @n3_list = map {$to3->($_)} @n_list;
    my $width = max(map {length} @n3_list) + 1;
    foreach my $n (@n_list) {
      my $n3 = $to3->($n);
      printf "%*s\n", $width, $n3;
    }
    foreach my $n (@n_list) {
      my ($other,$sign) = other_n($n);
      my $other3 = $to3->($other);
      my $lowdigit = $n % 3;
      # $lowdigit == 1 &&  $lowdigit == 2 &&
      my $diff = ($other3 eq $n3_list[0] ? "  a"
                  : $other3 eq ($n3_list[1]//-1) ? "  b"
                  : $other3 eq ($n3_list[2]//-1) ? "  c"
                  : "************");
      printf "%*s (%s) %s\n", $width, $other3, $sign, $diff;
    }
    my $xyt = xy_to_ternary($x,$y);
    printf "%*s xy ternary\n", $width, $xyt;

    print "\n";
  }
  exit 0;

  # other N
  # add = 0,1,2
  # add += digit==1        ones below
  # add -= new_digit==1    ones below
  #
  #   1  0  1  0  2  1  low=1 sign=-1
  #                  1  low unchanged, add is  -1,  0,  0 = -1
  #               1     new=1          add now -1,  0, +1 =  0
  #            0        old=0,new=0    add same             0
  #         1          old=1,new=1    add now -1, -1, +2 = 0
  #      0
  #   1
  #
  #  -1 -1 -1 -1 -1 0  sign add
  #  -1 -1  0  0  0 0  old add
  #  +2 +2 +1 +1  0 0  new add
  #  ----------------
  #   0  0  0  0 -1 0  total add
  #
  #   0  2  1  0  2 1  old plus its add  \ +2 in each
  #   2  1  0  2  1 1  new sub its add   /
  #                      delta -1 each
  #
  #   1  0  1  0  2 1  old
  # - 2  2  1  1  1 0  ones below, not including self
  # = 2  1  0  2  1 1
  #
  #   1  0  1  0  1 1
  # - 3  3  2  2  1 0  ones below, not including self
  # = 1  0  2  1  0      delta -1 each
  #
  #
  # 1  0  2  1  1  1   total = 4 = 1
  # 1  0  2  0  1      plus
  # 1  0  2  2  0      minus
  #
  # 1  0  2  2  0  1   total = 2
  # 2  1  0  0  1      plus
  # 0  2  1  1  2      minus
  #
  sub other_n {
    my ($n) = @_;
    my @ret;
    my $debug = 0;
    my @digits = digit_split_lowtohigh($n,3);
    while (@digits && $digits[0] == 0) {
      push @ret, shift @digits;
    }
    my $lowdigit = shift @digits;
    push @ret, $lowdigit;
    my $sign = ($lowdigit == 1 ? -1 : 1);
    my $add = $sign;
    if ($debug) { print "sign=$sign\n"; }
    foreach my $digit (@digits) {
      my $new_digit = ($digit + $add) % 3;
      if ($debug) { print "add=$add\n"; }
      push @ret, $new_digit;
      if ($digit == 1) {
        $add += $sign;
        if ($debug) { print "old 1\n"; }
      }
      # if ($new_digit == 1) {
      #   if ($debug) { print "new 1\n"; }
      #   $add -= $sign;
      # }
    }
    if ($debug) { print "final add=$add\n"; }
    my $new_digit = (0 + $add) % 3;
    push @ret, $new_digit;
    if ($new_digit == 1) {
      $add -= $sign;
    }
    $add %= 3;
    return (digit_join_lowtohigh(\@ret,3, $n*0),
            "$sign,$add");
  }

  sub to_balanced3 {
    my ($n) = @_;
    my $str = '';
    while ($n) {
      my $digit = $n % 3;
      if ($digit == 0) {
        $str = "0$str";
      } elsif ($digit == 1) {
        $str = "1$str";
        $n -= 1;
      } else { # $digit == 2
        $str = "n$str";
        $n += 1;
      }
      $n % 3 == 0 or die "$n";
      $n /= 3;
    }
    return $str;
  }

  sub xy_to_ternary {
    my ($x,$y) = @_;
    my $str = '';
    while ($x || $y) {
      my $digit = (($x+3*$y)/2) % 3;
      $str = "$digit$str";
      if ($digit == 1) {
        $x -= 2;
      } elsif ($digit == 2) {
        $x -= 1;
        $y -= 1;
      }
      ($x,$y) = div($x,$y);
    }
    return $str;
  }
}

{
  # N triple point relations
  require Math::BaseCnv;
  my $path = Math::PlanePath::TerdragonCurve->new;
  my %seen;
  for (my $n = 0; $n < 2000; $n++) {
    my ($x,$y) = $path->n_to_xy($n);
    my $key = "$x,$y";
    push @{$seen{$key}}, $n;
    if (@{$seen{$key}} == 3) {
      next if all_multiples_3 (@{$seen{$key}});
      @{$seen{$key}} = n_sort_by_prev_dir (@{$seen{$key}});
      my @v3;
      foreach my $v (@{delete $seen{$key}}) {
        my $dir3 = n_prev_dir($v);
        my $v3 = Math::BaseCnv::cnv($v,10,3);
        push @v3, $v3;
        printf "%4s %7s   dir3=%d\n", $v, $v3, $dir3;
      }
      my $lenmatch = 0;
      foreach my $i (1 .. length($v3[0])) {
        my $want = substr ($v3[0], -$i);
        if ($v3[1] =~ /$want$/ && $v3[2] =~ /$want$/) {
          next;
        } else {
         $lenmatch = $i-1;
          last;
          last;
        }
      }
      my $zeros = ($v3[0] =~ /(0*)$/ && $1);
      my $lenzeros = length($zeros);
      my $same = ($lenmatch == $lenzeros+1 ? "same" : "diff");
      print "low same $lenmatch zeros $lenzeros   $same\n";
      print "\n";
    }
  }
  exit 0;

  sub n_sort_by_prev_dir {
    my @n_list = @_;
    return sort {n_prev_dir($a) <=> n_prev_dir($b)} @n_list;
  }
  sub n_prev_dir {
    my ($n) = @_;
    my ($dx,$dy) = $path->n_to_dxdy($n-1);
    # ($dx,$dy) = (-$dx,-$dy);  # rotate 180
    return dxdy_to_dir3 ($dx,$dy);
  }

  sub all_multiples_3 {
    foreach my $n (@_) {
      if ($n % 3 != 0) {
        return 0;
      }
    }
    return 1;
  }
}
{
  # boundary increment
  my $path = Math::PlanePath::TerdragonCurve->new;
  my $i = 0;
  my $n_prev = $path->_UNDOCUMENTED__right_boundary_i_to_n($i);
  $i++;
  my $run = 0;
  my $five = 0;
  my $five_max = -1;
  for ( ; $i < 1540000; $i++) {
    my $n = $path->_UNDOCUMENTED__right_boundary_i_to_n($i);
    my $n_diff = $n - $n_prev;
    my $n_diff3 = Math::BaseCnv::cnv($n_diff,10,3);
    my $n3 = Math::BaseCnv::cnv($n,10,3);

    # print "$n_diff3, ";
    # if ($n_diff != 1) {
    #   print "\n";
    # }
    # print "$n_diff,";
    # print "$n3,";

    if ($n_diff == 1) {
      $run++;
    } else {
      # print "$run,";
      if ($run == 5) {
        if ($five > $five_max) {
         print "  # at n=$n3\n";
          print "$five\n";
          $five_max = $five;
        }
        $five = 0;
      } else {
        $five++;
      }
      $run = 0;
    }

    my $n_prev3 = Math::BaseCnv::cnv($n_prev,10,3);
    my $got = _UNDOCUMENTED__right_boundary_n_to_dn($path,$n_prev);
    my $got3 = Math::BaseCnv::cnv($got,10,3);
    if ($got != $n_diff) {
      die "prev=$n_prev3 n=$n3  want=$n_diff3 got=$got3";
    }

    $n_prev = $n;
  }
  print "\n";
  exit 0;

  sub _UNDOCUMENTED__right_boundary_n_to_dn {
    my ($self, $n) = @_;
    my $add = ($n*0) + 1;
    my $q = $n;
    while ($q % 3 == 2) {
      $q = int(($q - 2) / 3);
      $add *= 3;
    }
    return ($q % 9 == 3 ? 3*$add+1 : 1);
  }
}

{
  # segments by direction relative to middle segment
  # d=0  A101990
  require Math::PlanePath::TerdragonCurve;
  my $path = Math::PlanePath::TerdragonCurve->new;
  my @count_arrays;
  my $width = 36;
  foreach my $k (0 .. 10) {
    my $n_end = 3**$k;
    my $dir3_base = dxdy_to_dir3($path->n_to_dxdy(($n_end-1)/2));
    # $dir3_base = 0;
    # $dir3_base = -$k;
    my @count = (0,0,0);
    for (my $n = 0; $n < $n_end; $n++) {
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my $dir3 = dxdy_to_dir3($dx,$dy);
      $dir3 = ($dir3 - $dir3_base) % 3;
      $dir3 >= 0 && $dir3 <= 2 or die;
      $count[$dir3]++;
      ### seg: "n=$n dir=$dir3"
    }
    ### @count
    foreach my $i (0 .. 2) {
      push @{$count_arrays[$i]}, $count[$i];
    }
    printf "k=%2d N=$n_end dir3_base=%d", $k, $dir3_base;
    # foreach my $dxdy (@dxdy_strs) {
    #   my $a = _UNDOCUMENTED__level_to_segments_dxdy($path, $k, split(/,/, $dxdy));
    #   my $ar = Math::BaseCnv::cnv($a,10,3);
    #   printf " %*s", $width, $ar;
    # }
    # print "\n";
    print "\n";
  }
  my $trim = 1;
  foreach my $i (0 .. 2) {
    my $aref = $count_arrays[$i] || [];
    # splice @$aref, 0, $trim;
    # @$aref = MyOEIS::first_differences(@$aref);
    print "$i is ", join(',',@$aref),"\n";
    require Math::OEIS::Grep;
    Math::OEIS::Grep->search (array => \@$aref, name => "d=$i");
  }

  exit 0;
}

{
  # segments by direction relative to middle segment
  # calculated by offset from 3^k

  # terdragon mid
  # A101990 a(n) = 3*a(n-1) - 3*a(n-2) + 9*a(n-3), a(1) = a(2) = 1, a(3) = 9.
  # a(k) = if(k==0,1, k==1,1, k==2,9, 3*a(k-1) - 3*a(k-2) + 9*a(k-3))
  # polroots(x^3 - 3*x^2 + 3*x - 9)
  # x^3 - 3*x^2 + 3*x - 9
  # (x-3)*(x - sqrt(3)*I)*(x + sqrt(3)*I)
  # ap(k) = 3^k + 3^floor(k/2)
  # g = x*(1-2*x+9*x^2) / ((1-3*x)*(1+3*x^2))
  # SM(k,d)=S(k,d-k)
  #        = 3^(k-1) + s(k-4*(d-k))*3^floor((k-1)/2)
  #        = 3^(k-1) + s(-3k-4d)*3^floor((k-1)/2)
  # d=0 d=1 d=2
  #  2,  1,  1,
  #  0, -1, -1, d=4
  # -2, -1, -1, d=8
  #  0,  1,  1
  # sm0(k) = k=k%4; if(k==0,2, k==0,0, k==0,-2, 0)
  # SM0(k) = 3^(k-1) + sm0(k)*3^floor((k-1)/2)
  # vector(15,k,SM0(k-1))


  my @s = (2,  1,  1,
           0, -1, -1,
           -2, -1, -1,
           0,  1,  1);
  foreach my $d (0 .. 2) {
    foreach my $k (0 .. 11) {
      print ' ',$s[(3*$k-4*$d)%12];
    }
    print "\n";
  }
  my @sm0 = (2,0,-2,0);
  sub sm0 {
    my ($m) = @_;
    return $sm0[$m%4];
  }
  sub SM0 {
    my ($k) = @_;
    return ( 3**$k + sm0($k) * 3**int($k/2) ) /3;
    return 3**($k-1) + sm0($k) * 3**floor(($k-1)/2);
  }
  foreach my $k (0 .. 16) {
    print SM0($k),",";
  }
  print "\n";

  my @sm = (2,1,1,0,-1,-1, -2,-1,-1,0,1,1);
  sub sm {
    my ($m) = @_;
    return $sm[$m%12];
  }
  sub S {
    my ($k,$d) = @_;
    return 3**($k-1) + sm($k-4*$d)*3**floor(($k-1)/2);
  }
  sub SM0_from_S {
    my ($k) = @_;
    return S($k,$k);
  }
  foreach my $k (0 .. 16) {
    print SM0_from_S($k),",";
    die if SM0($k) != SM0_from_S($k);
  }
  print "\n";

  exit 0;
}

{
  # segments moment of inertia -- directions 0,120,240

  # 0    1     2     3
  #      *-----2-----*     2, 1    1,1   norm=(1+3*1)/4=1
  #       \
  #        \
  #         1            1.5, 1    0,0
  #          \
  #           \
  # *----0-----*           1, 0   -1,1
  # 0    1     2
  #
  # y = sqrt(3)/2  y^2 = 3/16     mid L=sqrt(3) L^2=3
  #     m   y^2    1/12  m   L^2
  # 2 * 1 * 3/4 + 1/12 * 1 * 3    = 2*3/4+1/4 = 7/4 = 1.75

  # 0,7,89,696,3441,14067,202464,3871935,47146401,368971848,...
  # 1,3,32,591,7185,56232,278361,1138779,16398720,313626087,...
  # 1,8,41,171,2496,47799,582057,4555224,22548321,92243043,...
  # total 2,18,162,1458
  #      2*3^0 2*3^2 2*3^4 2*3^6


  #    *       mid L=2  1/12*m*L^2 = 1/12 * 3 * 2^2
  #    |\      ends                = 1
  #    | *
  # \  *  \
  #  * |
  #   \|
  #    *
  # longways
  # 1*: 0,1,8,69,624,5625,50616,455517,4099680
  # 0,4,32,276,2496,22500,202464,1822067.99999997,16398720,...
  # 3*: 0,3,24,207,1872,16875,151848,1366551,12299040
  # widthways
  # 3*: 1,6,57,522,4689,42174,379593,3416418,30747681
  # long+width = 3^k

  my $path = Math::PlanePath::TerdragonCurve->new;
  my @values;
  foreach my $level (0 .. 8) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($level);

    my $rot = sub {
      my ($x,$y) = @_;

      # return ($x,$y);
      # return xy_rotate_plus120($x,$y);
      # return xy_rotate_minus120($x,$y);

      foreach (1 .. $level) {
        ($x,$y) = xy_rotate_minus30($x,$y);
        # ($x,$y) = xy_rotate_minus120($x,$y);
      }
      # ($x,$y) = xy_rotate_minus30($x,$y);
      # ($x,$y) = xy_rotate_minus30($x,$y);
      # ($x,$y) = xy_rotate_minus30($x,$y);

      if (abs($x) < 1e-15) { $x = 0; }
      if (abs($y) < 1e-15) { $y = 0; }
      return ($x,$y);
    };

    my $verbose = ($level <= 1);

    my ($ex,$ey) = $path->n_to_xy($n_hi);  # relative to centre
    if ($verbose) { print "  raw end $ex, $ey\n"; }
    ($ex,$ey) = $rot->($ex,$ey);
    if ($verbose) { print "  rot end $ex, $ey\n"; }
    # ($mx,$my) = (0,0);  # relative to origin
    $ey *= sqrt(3);

    my $mx = $ex/2;
    my $my = $ey/2;

    my $sum = 0;
    foreach my $n ($n_lo .. $n_hi-1) {
      if ($verbose) { print "  n=$n\n"; }

      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      ($x,$y) = $rot->($x,$y);
      ($dx,$dy) = $rot->($dx,$dy);

      $y *= sqrt(3);
      $dy *= sqrt(3);
      $x += $dx/2;
      $y += $dy/2;
      $x -= $mx;
      $y -= $my;

      # Ix = Ic + m*y^2
      # Ic = 1/12 * m * L^2

      my $y_squared = $y*$y;
      $sum += $y_squared;
      # 1/12 m*L^2   L=dy
      my $dy_squared = $dy*$dy;
      $sum += $dy_squared / 12;

      if ($verbose) { print "  xy=$x,$y  ysquared=$y_squared dy=$dy dy^2=$dy_squared\n"; }
    }
    push @values, $sum*3;

    my $Lsquared = $ex*$ex + $ey*$ey;
    my $scaled = $sum;
    $scaled /= 3**$level;  # total mass 1
    if ($Lsquared) {
      $scaled /= $Lsquared;    # length scaled to 1
    } else {
      $scaled = 0;
    }
    print "level=$level mid=$mx,$my   $sum  $scaled\n";
  }

  foreach my $value (@values) {
    my $int = int($value+.5);
    my $diff = abs($value - $int);
    if ($diff < 1e-2) {
      $value = $int;  # mutate array
    }
    print "$value    $diff\n";
  }

  require Math::OEIS::Grep;
  Math::OEIS::Grep->search (array => \@values,verbose=>1);
  exit 0;
}

{
  my $x = 2;
  my $y = 0;
  #   ($x,$y) = xy_rotate_minus30($x,$y);
  #   ($x,$y) = xy_rotate_minus30($x,$y);
  #   ($x,$y) = xy_rotate_minus30($x,$y);
  # printf "%-7s  %-7s\n", $x, $y;

  foreach my $i (0 .. 12) {
    my $sx = sprintf "%.4f", $x;
    $sx =~ s/0+$//;
    my $sy = sprintf "%.4f", $y;
    $sy =~ s/0+$//;
    printf "%2d  %-7s  %-7s\n", $i, $sx, $sy;
    ($x,$y) = xy_rotate_minus30($x,$y);
    # ($x,$y) = xy_rotate_plus30($x,$y);
  }
  exit 0;
}

{
  # segments moment of inertia
  # in X,sqrt(3)*Y is triangle side length = 2
  #
  #      *-----2-----*   4,2    1,1   norm=(1+3*1)/4=1
  #       \
  #        \
  #         1            3,1    0,0
  #          \
  #           \
  # *----0-----*         2,0   -1,1
  # 0    2  3  4  5  6
  #
  # relative to centre of gravity
  # 0,2,24,234,2160,19602,176904,1593594,14346720,129133602,...
  # A219205 3^(n-1)*(3^n - 1), n >= 0.
  # half is
  # A016142 Expansion of 1/((1-3x)(1-9x)).
  #
  # b=1+w |b|=sqrt(3)
  # R = b^k/2 - b^(k-1)/2
  #   = (b-1)*b^(k-1)/2
  #   = w*b^(k-1)/2
  # |R| = sqrt(3)^(k-1)/2
  # |R|^2 = 3^(k-1)/4
  #       = 1/4, 3/4, 9/4
  #
  # norm(2,0) = (2^2+3*0^2)/4 = 1
  # norm(3,1) = (3^2+3*1^2)/4 = 3
  # norm(3,3) = (3^2+3*3^2)/4 = 9
  #
  # Ic = m*r^2
  # IR = m/2((R + r cos)^2 + (r sin)^2)
  #    + m/2((R - r cos)^2 + (r sin)^2)
  #    = m*((r cos)^2 + (r sin^2)) + m*R^2
  #    = Ic + m*R^2
  #
  # I[k] = I[k-1] + 2*(I[k-1] + 3^(k-1)*3^(k-1))
  #      = 3*I[k-1] + 2*3^(2k-2)      k >= 1
  #      = 3*I[k-1] + 2*9^(k-1)      k >= 1
  # I[0] = 0
  # I[1] = 2*1 = 2
  # I[2] = 3*2 + 2*3^2 = 24
  # I[3] = 3*24 + 2*3^4 = 234
  # I[4] = 3*234 + 2*3^6 = 2160
  #
  # Ic(k) = 2*9^(k-1) + 3*2*9^(k-2) + ... + 3^(k-1)*2*9^0 + 3^k*I[0]
  #       = 2* (9^k - 3^k) / (9-3)
  # Ic(k) = 3^(k-1) * (3^k - 1)
  #
  # relative to 0,0
  # IE(k) = Ic(k) + 3^k*3^k
  # IE(k) = 3^(k-1) * (3^k - 1) + 3*3^(k-1)*3^k
  # IE(k) = 3^(k-1) * (3^k - 1  + 3*3^(k-1))
  # IE(k) = 3^(k-1) * (4*3^k - 1)
  #
  # radius of gyration
  # 3^k*r^2 = Ic(k)
  # r^2 = 3^(k-1) * (3^k - 1) / 3^k
  # r^2 = (3^k - 1) / 3
  #
  #                             *
  #                                     *
  #
  # *---------------------.5----------------------*
  #   30
  #           *
  #                  *
  # w = (1 + sqrt(3)*I) / 2    6th root
  # b = 1 + w
  # b^-1 = 1/2 - sqrt(3)/6*I
  # d = 1/2 - (b^-1)/2 = 1/4 + sqrt(3)/12*I
  # abs(d) = sqrt(1/16 + 3/144)
  #        = sqrt(3)/6
  #
  # scaled
  # I/9 for 1/3 mass and (1/sqrt(3))^2 length scale
  # I = I/9  + 2*(I/9 + 1/3 * (sqrt(3)/6)^2 )
  # I*(1 - 1/9 - 2/9) = 2/3 * 1/12
  # I = 2/3 * 1/12 * 3/2
  #   = 1/12
  #   = 0.08333
  #
  # 3^(k-1) * (3^k - 1) / (3^k * abs(b^k)^2)
  #
  #
  require Math::PlanePath::TerdragonMidpoint;
  my $path = Math::PlanePath::TerdragonMidpoint->new;
  my @values;
  foreach my $level (0 .. 9) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($level);
    my ($mx,$my) = $path->n_to_xy($n_hi/2);  # relative to centre
    # ($mx,$my) = (0,0);  # relative to origin
    my $sum = 0;
    # ($n_lo,$n_hi) = (($n_hi+1)/3, 2*($n_hi+1)/3-1);  # middle third
    # ($n_lo,$n_hi) = (0, ($n_hi+1)/3-1);  # first third
    foreach my $n ($n_lo .. $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      $x -= $mx;
      $y -= $my;
      $sum += xy_norm($x,$y);

      $sum += to_bigrat(2*2)/12;  # m*L^2/12 with L=2
    }
    push @values, $sum;

    my $m_norm = $mx*$mx + 3*$my*$my;
    my $scaled = $sum;
    $scaled /= 3**$level;  # total mass 1
    $scaled /= $m_norm;    # length scaled to 1
    print "level=$level mid=$mx,$my   $sum  $scaled  m_norm=$m_norm\n";

    ($mx,$my) = mul($mx,$my);
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search (array => \@values,verbose=>1);
  exit 0;
}
{
  # vertices moment of inertia
  # *4 is
  # 1,8,42,288,2322,20088,178362,1597968,14359842,129172968,...
  # cf
  # 0,2,24,234,2160,19602,176904,1593594,14346720,129133602,...
  # diff/2
  # 1/2, 3, 9, 27, 81, 243, 729, 2187, 6561, 19683
  #
  my $path = Math::PlanePath::TerdragonCurve->new;
  my @values;
  foreach my $level (0 .. 10) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($level);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    my ($mx,$my) = ($x_hi/2,$y_hi/2);   # relative to centre
    # ($mx,$my) = (0,0);  # relative to origin
    my $sum = 0;
    foreach my $n ($n_lo .. $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      $x -= $mx;
      $y -= $my;
      $sum += xy_norm($x,$y);
    }
    $sum *= 4;
    push @values, $sum;

    my $m_norm = $mx*$mx + 3*$my*$my;
    my $scaled = $sum;
    $scaled /= 3**$level + 1; # total mass 1
    $scaled /= $m_norm;       # length scaled to 1
    print "level=$level mid=$mx,$my   $sum  $scaled  m_norm=$m_norm\n";

    ($mx,$my) = mul($mx,$my);
  }
  shift @values;
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search (array => \@values,verbose=>1);
  exit 0;
}

{
  # electrical resistance
  # 1,
  # 3,
  # 213/38 = 5+23/38 = 5.60526
  # 1754553/248368 = 7+15977/248368 = 7.06432
  # 128057830452182265521664/14756230027993986049111 = 8.67822
  #
  # equipotential
  # level=3  equipotential 3 at 103/38
  #          equipotential 3 at 55/19
  #
  # level=4 equipotential 3 at 2007929/745104
  #         equipotential 3 at 2512357/745104
  #         equipotential 2 at 1375651/372552
  #         equipotential 3 at 1627865/372552
  #
  # level=8 equipotential 3 at 39763372925209719682426/14756230027993986049111
  #         equipotential 3 at 49641158324704530303939/14756230027993986049111
  #         equipotential 3 at 11246095274343174930917/2682950914180724736202
  #         equipotential 3 at 11360974679326442466869/2682950914180724736202
  #         equipotential 3 at 131144939431773663907769/29512460055987972098222
  #         equipotential 2 at 132408612886589606803241/29512460055987972098222
  #         equipotential 2 at 78416672127477735217725/14756230027993986049111
  #         equipotential 3 at 88294457526972545839238/14756230027993986049111
  #
  # shortcut
  # 1
  # 1
  # 45/29 = 1.55172
  # 2513/1038 = 2.42100
  # 36240988412/10463937919 = 3.46341
  # 11290760109005539467551610935694/2410959685535642656728369422657 = 4.68309
  # equipotential 1/2 at level=1 only

  my $traversed;
  $traversed = \&path_xyxy_is_traversed_shortcut;
  $traversed = \&path_xyxy_is_traversed_either;

  sub to_bigrat {
    my ($n) = @_;
    require Math::BigRat;
    return Math::BigRat->new($n);
    return $n;
  }

  my $path = Math::PlanePath::TerdragonCurve->new;
  foreach my $level (0 .. 9) {
    (my $n_lo, my $n_hi) = $path->level_to_n_range($level);
    print "\nlevel=$level  to N=$n_hi\n";
    my %xy_to_index;
    my %xy_to_value;
    my @index_to_xy;
    my $index = 0;
    my @rows;
    foreach my $n ($n_lo .. $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      my $xy = "$x,$y";
      if (! exists $xy_to_index{$xy}) {
        ### vertex: "$x,$y index=$index"
        $index_to_xy[$index] = [$x,$y];
        $xy_to_index{$xy} = $index;
        $xy_to_value{$xy} = ($n == $n_lo ? to_bigrat(-1)
                             : $n == $n_hi ? to_bigrat(1)
                             : to_bigrat(0));
        $index++;
      }
    }
    foreach my $xy (keys %xy_to_index) {
      my @row = (to_bigrat(0)) x $index;
      $row[$index] = $xy_to_value{$xy};
      my $i = $xy_to_index{$xy};
      if ($i == 0) {
        $row[$i] = 1;
        $row[$index] = 0;
      } else {
        my ($x,$y) = split /,/, $xy;
        ### point: "$x,$y"
        foreach my $dir6 (0 .. $#dir6_to_dx) {
          my $dx = $dir6_to_dx[$dir6];
          my $dy = $dir6_to_dy[$dir6];
          my $x2 = $x+$dx;
          my $y2 = $y+$dy;
          if ($traversed->($path, $x,$y, $x2,$y2, $n_hi)) {
            my $i2 = $xy_to_index{"$x2,$y2"};
            ### edge: "$x,$y to $x2,$y2  $i to $i2"
            $row[$i]++;
            $row[$i2]--;
          }
        }
      }
      push @rows, \@row;
    }
    print "vertices $index\n";
    ### @rows

    require Math::Matrix;
    my $m = Math::Matrix->new(@rows);
    # print $m;
    if (0) {
      my $s = $m->solve;
      # print $s;
      foreach my $i (0 .. $index-1) {
        print " ",$s->[$i][0],",";
      }
      print "\n";
      my $V = $s->[0][0];
      print int($V),"+",$V-int($V),"\n";
    }
    {
      open my $fh, '>', '/tmp/x.gp' or die;
      if ($level <= 1) {
        mm_print_pari($m, \*STDOUT);
        print "\n";
      }
      mm_print_pari($m,$fh);
      print $fh "; s=matsolve(m,v); r=s[$index,1]; print(\"r=\"r,\" r=\",r+0.0); print(s);\n";
      close $fh;
      require IPC::Run;
      my $out;
      IPC::Run::run(['gp','-s','1000000000','--quiet'],'<','/tmp/x.gp','>',\$out);
      print $out;
      $out =~ /\[(0;[^]]*)/ or die;
      my $vec = $1;
      # print "vec: $vec\n";
      my @volts = split /;\s*/, $vec;
      # print "volts: ",@volts, "\n";

      my %equip;
      foreach my $i (0 .. $#volts) {
        my $volt = $volts[$i];
        push @{$equip{$volt}}, $i;
      }
      foreach my $volt (keys %equip) {
        my $aref = $equip{$volt};
        my $count = scalar(@$aref);
        next if $count < 2;
        my @i_list = @$aref;
        print "equi count=$count  ";
        foreach my $i (@i_list) {
          my ($x,$y) = @{$index_to_xy[$i]};
          print "$i xy=$x,$y N=";
          my @n_list = $path->xy_to_n_list($x,$y);
          foreach my $n (@n_list) {
            print "$n,";
          }
          print "; ";
        }
        print "\n";
      }
    }
  }
  exit 0;

  sub mm_print_pari {
    my ($m, $fh) = @_;
    my ($rows, $cols) = $m->size;
    print $fh "m=[\\\n";
    my $semi = '';
    foreach my $r (0 .. $rows-1) {
      print $fh $semi;
      $semi = ";\\\n";
      my $comma = '';
      foreach my $c (0 .. $cols-2) {
        print $fh $comma, $m->[$r][$c];
        $comma = ',';
      }
    }
    print $fh "];\\\nv=[";
    $semi = '';
    foreach my $r (0 .. $rows-1) {
      print $fh $semi, $m->[$r][$cols-1];
      $semi = ';';
    }
    print $fh "]";
  }

  #    1,1   3,1
  #      *---*
  #       \
  #    *---*
  #   0,0  2,0
  sub path_xyxy_is_traversed_shortcut {
    my ($path, $x1,$y1, $x2,$y2, $n_hi) = @_;
    ### path_xyxy_is_traversed_shortcut(): "$x1,$y1 to $x2,$y2"
    if (path_xyxy_is_traversed_either ($path, $x1,$y1, $x2,$y2, $n_hi)) {
      ### yes, plain ...
      return 1;
    }
    my $dx = $x2 - $x1;
    my $dy = $y2 - $y1;

    {
      my ($dxP,$dyP) = (($dx-3*$dy)/2,   # rotate +60
                        ($dx+$dy)/2);
      my $xm = $x1 + $dxP;
      my $ym = $y1 + $dyP;
      if (path_xyxy_is_traversed_either ($path, $x1,$y1, $xm,$ym, $n_hi)
          && path_xyxy_is_traversed_either ($path, $xm,$ym, $x2,$y2, $n_hi)) {
        ### yes, shortcut P ...
        return 1;
      }
    }
    {
      my ($dxM,$dyM) = (($dx+3*$dy)/2,   # rotate -60
                        ($dy-$dx)/2);
      my $xm = $x1 + $dxM;
      my $ym = $y1 + $dyM;
      if (path_xyxy_is_traversed_either ($path, $x1,$y1, $xm,$ym, $n_hi)
          && path_xyxy_is_traversed_either ($path, $xm,$ym, $x2,$y2, $n_hi)) {
        ### yes, shortcut M ...
        return 1;
      }
    }
    ### no ...
    return 0;
  }
}
{
  # ternary permutation digit 1<->2
  # A004488 = decimal Tersum n + n (add digits mod 3, no carry)
  # ternary not in OEIS
  my @values;
  foreach my $n (0 .. 80) {
    my $n3 = Math::BaseCnv::cnv($n,10,3);
    $n3 =~ tr/12/21/;
    # push @values, $n3;   # in ternary
    push @values, Math::BaseCnv::cnv($n3,3,10);  # in decimal
  }
  splice @values, 0,1;
  print join(',',@values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search (array => \@values,
                            use_mmap => 0);
  exit 0;
}


{
  # ternary permutation 1nz <-> 2nz
  # (not in OEIS)
  my @values;
  foreach my $n (0 .. 26) {
    $values[$n] = onetwo_permute($n);
  }
  splice @values, 0,4;
  print join(',',@values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search (array => \@values);
  exit 0;

  sub onetwo_permute {
    my ($n) = @_;
    my @digits = digit_split_lowtohigh($n,3);
    foreach my $i (1 .. $#digits) {
      if ($digits[$i-1] && $digits[$i]) {
        $digits[$i] = ($digits[$i] == 1 ? 2 : 1);
      }
    }
    return digit_join_lowtohigh(\@digits, 3, $n*0);
  }
}

{
  # right boundary segment numbers /3
  # (not in OEIS)
  my $path = Math::PlanePath::TerdragonCurve->new;
  my @values;
  for (my $n = 0; $n < 81; $n += 3) {
    if ($path->_UNDOCUMENTED__n_segment_is_right_boundary($n)) {
      push @values, $n/3;
    } else {
    }
  }
  splice @values, 0,1;
  print join(',',@values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search (array => \@values);
  exit 0;
}

{
  # ternary with or without at least one digit
  #
  # digit       decimal             ternary
  #          with    without     with   without
  #   0     A081605  A032924      --    A007931 1,2s only
  #   1     A081606  A005823      --    A169965 0,2s only
  #   2     A074940  A005836      --    A007088 0,1s only = binary
  #
  # A005823 without any 1 is Cantor dust
  #
  foreach my $digit (0 .. 2) {
    foreach my $without (0, 1) {
      my @values;
      foreach my $n (0 .. 80) {
        my $n3 = Math::BaseCnv::cnv($n,10,3);
        if (($n3 =~ /$digit/) ^ $without) {
          push @values, $n3;
        }
      }
      splice @values, 0,1;
      print join(',',@values),"\n";
      require Math::OEIS::Grep;
      Math::OEIS::Grep->search (array => \@values,
                                name => "$digit without=$without");
    }
  }
  exit 0;
}

{
  # 2xN arrays count
  # 3^k each positions 01 10 11
  # 2^k ways to have a 11, of which one is no 11s
  my @values;
  foreach my $k (0 .. 7) {
    my $count = 0;
    foreach my $i (0 .. 3**$k-1) {
      my $i3 = Math::BaseCnv::cnv($i,10,3);
      if ($k <= 2) {
        printf " %0*s\n", $k, $i3;
      }
      if ($i3 =~ /2/) {
        $count++;
      }
    }
    push @values, $count;
    print "$k  $count\n";
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search (array => \@values);
  exit 0;
}
{
  # multiply

  my $x = 2;
  my $y = 0;
  foreach (1 .. 10) {
    print " $x,$y";
    ($x,$y) = mul($x,$y);
  }
  print "\n";
  exit 0;
}

{
  # left boundary segment number of any level k, in ternary
  my $path = Math::PlanePath::TerdragonCurve->new;
  {
    my $n = 0;
    foreach my $k (0 .. 4) {
      ### $k
      ### $n
      for ( ; $n < 3**$k; $n++) {
        if ($path->_UNDOCUMENTED__n_segment_is_any_left_boundary($n)) {
          my $n3 = Math::BaseCnv::cnv($n,10,3);
          print "$n3,";
        }
      }
      print "\n";
    }
  }
  {
    my $len = 1;
    foreach my $i (0 .. 60) {
      my $n = $path->_UNDOCUMENTED__left_boundary_i_to_n($i,-1);
      my $n3 = Math::BaseCnv::cnv($n,10,3);
      if (length($n3) > $len) {
        print "\n";
        $len++;
      }
      print "$n3,";
    }
    print "\n";
  }
  exit 0;
}

{
  # left boundary segment number of any level k
  my %list;
  require Tie::IxHash;
  tie %list, 'Tie::IxHash';
  my $path = Math::PlanePath::TerdragonCurve->new;
  foreach my $k (0 .. 4) {
    ### $k
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    foreach my $n ($n_lo .. $n_hi-1) {
      ### $n
      if ($path->_UNDOCUMENTED__n_segment_is_left_boundary($n,$k)) {
        $list{$n} = 1;
      }
    }
  }
  my @n_list = keys %list;
  foreach my $n (@n_list) {
    my $n3 = Math::BaseCnv::cnv($n,10,3);
    print "$n,";
  }
  print "\n";

  foreach my $n (0 .. $n_list[-1]) {
    if ($path->_UNDOCUMENTED__n_segment_is_any_left_boundary($n)) {
      my $n3 = Math::BaseCnv::cnv($n,10,3);
      print "$n3,";
    }
  }
  print "\n";
  exit 0;
}

{
  # convex hull
  # 14 points at k=6

  # 2,0                                             1,1 -1,1  *
  # 3,1                                             0,2  -3,1  *
  # 3,3                                            -3,3  -6,0  *
  # -1,7      # 0,6 + -1,1      w                 -11,3 -10,-4  *
  # -10,10    # -9,9 + -1,1     w                 -20,0  -10,-10  *
  # -30,10    # -27,9 + -3,1    w-1               -30,-10 0,-20  *
  # -60,0     # -54,0 + -6,0    -3                -30,-30  30,-30  *
  # -91,-31   # -81,-27 + -10,-4  4w-1            1,-61  92,-30  *
  # -91,-91   # -81,-81 + -10,-10  10w+10         91,-91 182,0  *
  # 0,-182    # 0,-162 + 0,-20
  # 273,-273  # 243,-243 + 30,-30
  # 821,-273  # 729,-243 + 92,-30
  # 1640,0    # 1458,0 + 182,0
  #
  # new point b^k + point[-4]*120
  # starting b, b^2, b^3, b^4+w
  # p[k+4] = b^(k+4) + p[k]*w
  #        = b^(k+4) + w*b^k + w^2*b^(k-4) + w^4*b^(k-8) + w*p[k-12]
  # 2,0 mul 3,1 3,3 0,6 -9,9 -27,9 -54,0 -81,-27 -81,-81 0,-162 243,-243 729,-243 1458,0 2187,729 2187,2187

  # rot60(v) = [ (v[1]-3*v[2])/2, (v[1]+v[2])/2 ]
  # v=[2,0];print(rot60(v),rot60(rot60(v)))

  unshift @INC, "$FindBin::Bin/../../path/xt";
  require MyOEIS;
  require Math::Geometry::Planar;
  require Math::NumSeq::PlanePathCoord;
  require Math::OEIS::Grep;
  foreach my $elem (
                    # curves with overlaps only
                    ['TerdragonCurve', 3, 'triangular'],
                    ['TerdragonCurve,arms=6', 3, 'triangular'],
                   ) {
    my ($name, $radix, $lattice_type) = @$elem;
    $lattice_type ||= 'square';

    print "$name\n";
    my $path = Math::NumSeq::PlanePathCoord::_planepath_name_to_object($name);
    my $n_start = $path->n_start;
    my $arms = $path->arms_count;

    foreach my $convex_type (
                             # 'bbox',
                             # 'minrectangle',
                             'convex',
                             # 'full',
                             # 'left','right',
                            ) {
      my @areas;
      my @boundaries;
      for (my $level = 0; ; $level++) {

        my $n_limit;
        (undef, $n_limit) = $path->level_to_n_range($level);
        last if $n_limit > 1800_000;
        last if @areas > 35;

        my $side = ($convex_type eq 'right' ? 'right'
                    : $convex_type eq 'left' ? 'left'
                    : 0);
        print "n_limit=$n_limit  side=$side\n";
        my $points = MyOEIS::path_boundary_points ($path, $n_limit,
                                                   lattice_type => $lattice_type,
                                                   side => $side);
        ### $n_limit
        ### $points

        my $area;
        my $convex_area;
        my $boundary;
        if (@$points <= 1) {
          $area = 0;
          $boundary = 0;
        } elsif (@$points == 2) {
          $area = 0;
          my $dx = $points->[0]->[0] - $points->[1]->[0];
          my $dy = $points->[0]->[1] - $points->[1]->[1];
          my $h = $dx*$dx + $dy*$dy*($lattice_type eq 'triangular' ? 3 : 0);
          $boundary = 2*sqrt($h);
        } else {
          my $polygon = Math::Geometry::Planar->new;
          $polygon->points($points);

          if (($convex_type eq 'convex'
               || $convex_type eq 'minrectangle')
              && @$points >= 5) {
            $polygon = $polygon->convexhull2;
            $points = $polygon->points;
          }
          if ($convex_type eq 'bbox') {
            $polygon = $polygon->bbox;
            $points = $polygon->points;
          }
          if ($convex_type eq 'minrectangle') {
            if (@$points <= 16) {
              print "   ",points_str($points),"\n";
            }
            $polygon = $polygon->minrectangle;
            $points = $polygon->points;
          }

          $area = $polygon->area;

          if ($lattice_type eq 'triangular') {
            my @new_points;
            foreach my $p (@$points) {
              push @new_points, [ $p->[0], $p->[1] * sqrt(3) ];
            }
            my $psqrt3 = Math::Geometry::Planar->new;
            $psqrt3->points(\@new_points);
            $boundary = $psqrt3->perimeter;
          } else {
            $boundary = $polygon->perimeter;
          }
        }

        if ($convex_type eq 'right' || $convex_type eq 'left') {
          $boundary = scalar(@$points) - 1;
          # my ($end_x,$end_y) = $path->n_to_xy($n_limit);
          # $boundary -= hypot($end_x,$end_y);
          # $boundary = float_error($boundary);
        }
        push @areas, $area;
        push @boundaries, $boundary;

        my $notint = ($boundary == int($boundary) ? '' : ' (not int)');
        my $num_points = scalar(@$points);
        print "$level $n_limit points=$num_points area=$area boundary=$boundary$notint $convex_type\n";
        if (@$points <= 14) {
          my $spoints = [ sort {point_cmp_by_angle($a,$b)} @$points ];
          print "   ",points_str($spoints),"\n";
        }

        if (0) {
          require Image::Base::GD;
          my $width = 800;
          my $height = 700;
          my $image = Image::Base::GD->new (-width => $width, -height => $height);
          $image->rectangle (0,0, $width-1,$height-1, 'black');
          my $x_max = 0;
          my $x_min = 0;
          my $y_max = 0;
          my $y_min = 0;
          foreach my $p (@$points) {
            my ($x,$y) = @$p;
            $x_max = max($x_max, $x);
            $y_max = max($y_max, $y);
            $x_min = min($x_min, $x);
            $y_min = min($y_min, $y);
          }
          my $x_size = $x_max - $x_min;
          my $y_size = $y_max - $y_min;
          $x_size *= 1.1;
          $y_size *= 1.1;
          my $x_scale = $width / $x_size;
          my $y_scale = $height / ($y_size || 1);
          my $scale = min($x_scale,$y_scale);
          my $x_mid = ($x_min + $x_max) / 2;
          my $y_mid = ($y_min + $y_max) / 2;
          my $convert = sub {
            my ($x,$y) = @_;
            $x -= $x_mid;   $y -= $y_mid;
            $x *= $scale;   $y *= $scale;
            $x += $width/2; $y = $height/2 - $y;
            return ($x,$y);
          };
          {
            my ($x,$y) = $convert->(0,0);
            $image->ellipse ($x-3,$y-3, $x+3,$y+3, 'white', 1);
          }
          foreach my $i (0 .. $#$points) {
            my ($x1,$y1) = @{$points->[$i-1]};
            my ($x2,$y2) = @{$points->[$i]};
            ($x1,$y1) = $convert->($x1,$y1);
            ($x2,$y2) = $convert->($x2,$y2);
            $image->line ($x1,$y1, $x2,$y2, 'white');
          }
          $image->save('/tmp/x.png');
          require IPC::Run;
          IPC::Run::run (['xzgv','/tmp/x.png']);
        }
      }

      foreach my $alt_type (# 'even','odd',
                            'all') {
        my @areas = @areas;
        my @boundaries = @boundaries;

        if ($alt_type eq 'odd') {
          aref_keep_odds(\@areas);
          aref_keep_odds(\@boundaries);
        }
        if ($alt_type eq 'even') {
          aref_keep_evens(\@areas);
          aref_keep_evens(\@boundaries);
        }

        print "\n$name area (by $convex_type $alt_type):\n";
        shift_off_zeros(\@areas);
        print join(',',@areas),"\n";
        Math::OEIS::Grep->search(array => \@areas, name => 'area');

        print "\n$name boundary (by $convex_type $alt_type):\n";
        shift_off_zeros(\@boundaries);
        print join(',',@boundaries),"\n";
        Math::OEIS::Grep->search(array => \@boundaries, name => 'boundary');
        print "\n";
      }
    }
  }

  exit 0;

  # shift any leading zeros off @$aref
  sub shift_off_zeros {
    my ($aref) = @_;
    while (@$aref && ! $aref->[0]) {
      shift @$aref;
    }
  }

  sub aref_keep_odds {
    my ($aref) = @_;
    @$aref = map { $_ & 1 ? $aref->[$_] : () } 0 .. $#$aref;
  }
  sub aref_keep_evens {
    my ($aref) = @_;
    @$aref = map { $_ & 1 ? () : $aref->[$_] } 0 .. $#$aref;
  }

  BEGIN {

    sub path_boundary_points_triangular {
      my ($path, $n_limit) = @_;
      my @points;
      my $x = 0;
      my $y = 0;
      my $dir6 = 4;
      my @n_list = ($path->n_start);
      for (;;) {
        ### at: "$x, $y  dir6 = $dir6"
        push @points, [$x,$y];
        $dir6 -= 2;  # rotate -120
        foreach (1 .. 6) {
          $dir6 %= 6;
          my $dx = $dir6_to_dx[$dir6];
          my $dy = $dir6_to_dy[$dir6];
          my @next_n_list = $path->xy_to_n_list($x+$dx,$y+$dy);
          ### @next_n_list
          if (any_consecutive(\@n_list, \@next_n_list, $n_limit)) {
            @n_list = @next_n_list;
            $x += $dx;
            $y += $dy;
            last;
          }
          $dir6++;  # +60
        }
        if ($x == 0 && $y == 0) {
          last;
        }
      }
      return \@points;
    }
  }
}
{
  # shortcut boundary segments in direction
  my @s = (1,0,0,0,0,0);
  my @v;
  foreach my $k (1 .. 24) {
    my @new_s = @s;
    foreach my $i (0 .. 5) {
      $new_s[$i] += $s[$i-1];
    }
    @s = @new_s;
    foreach my $i (0 .. 5) {
      push @{$v[$i]}, $s[$i];
    }
  }

  require Math::OEIS::Grep;
  foreach my $i (0 .. 5) {
    Math::OEIS::Grep->search(array => $v[$i], name => $i);
  }
  exit 0;
}


{
  # boundary num lines, so collapse straight ahead segments
  #   = 2^k + 1
  # terdragon turns 2^k many
  # 0    ...01[zeros] 2^(k-2)+...+1 + 1 = 2^(k-1)
  # 0,1   ...010  2^k/8
  # 1,0  ...1100..0 2^k/16 + 2^k/32 + ... = 2^k/8-1
  # 0,0  0100  hanging triangles 2^k/4
  # total = 2^k/2-1
  # non-zeros 2^k - (2^k/2-1) = 2^k/2-1

  unshift @INC, "$FindBin::Bin/../../path/xt";
  require MyOEIS;
  my $path = Math::PlanePath::TerdragonCurve->new;
  my (@values);
  foreach my $level (0 .. 2) {
    ### $level
    (my $n_lo, my $n_hi) = $path->level_to_n_range($level);

    my $points = MyOEIS::path_boundary_points ($path, $n_hi,
                                               lattice_type => 'triangular',
                                               side => 'right');
    my $count = 0;
    foreach my $i (1 .. $#$points-1) {
      my $p1 = $points->[$i-1];
      my $p2 = $points->[$i];
      my $p3 = $points->[$i+1];
      my ($x1,$y1) = @$p1;
      my ($x2,$y2) = @$p2;
      my ($x3,$y3) = @$p3;
      my $dx1 = $x2-$x1;
      my $dy1 = $y2-$y1;
      my $dx2 = $x3-$x2;
      my $dy2 = $y3-$y2;
      ### cf: "$dx1,$dy1   $dx2,$dy2"
      if ($dx1 == $dx2 && $dy1 == $dy2) {
      } else {
        $count++;
      }
    }
    print "level=$level  $count\n";

    push @values, $count;
  }

  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values);
  exit 0;
}

{
  # left boundary segment numbers
  my $path = Math::PlanePath::TerdragonCurve->new;
  foreach my $n (0 .. 3**4) {
    if ($path->_UNDOCUMENTED__n_segment_is_left_boundary($n)) {
      print "$n, ";
    }
  }
  print "\n";
  foreach my $n (0 .. 3**4) {
    if ($path->_UNDOCUMENTED__n_segment_is_left_boundary($n)) {
      print cnv($n,10,3),", ";
    }
  }
  print "\n";
  exit 0;
}

{
  # hanging triangles
  my $path = Math::PlanePath::TerdragonCurve->new;
  sub n_is_right_hanging_triangle {
    my ($n, $level) = @_;
    ### n_is_hanging_triangle(): "level=$level  n=$n"
    foreach my $i (0 .. 2) {
      if (! $path->_UNDOCUMENTED__n_segment_is_right_boundary($n+$i)) {
        ### not on boundary: "n + $i"
        return 0;
      }
    }
    {
      my ($n_lo, $n_hi) = $path->level_to_n_range($level);
      unless ($n+3 <= $n_hi) {
        ### past end ...
        return 0;
      }
    }

    my ($x,$y) = $path->n_to_xy($n);
    my ($x3,$y3) = $path->n_to_xy($n+3);
    ### coordinates: "n=$n $x,$y cf $x3,$y3"
    return ($x == $x3 && $y == $y3);
  }
  my @values;
  foreach my $level (0 .. 10) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($level);
    my $count = 0;
    for (my $n = $n_lo; $n < $n_hi; $n++) {
      if (n_is_right_hanging_triangle($n, $level)) {
        $count++;
      }
    }
    print "level=$level  $count\n";
    push @values, $count;
  }
  require Math::OEIS::Grep;
  # Math::OEIS::Grep->search(array => \@values);
  exit 0;
}

{
  # rectangular extents relative to endpoints

  require Geometry::AffineTransform;
  foreach my $k (0 .. 10) {
    my $path = Math::PlanePath::TerdragonCurve->new;
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    my $affine = Geometry::AffineTransform->new;
    my $scale = 1 / hypot($y_hi,$x_hi);
    $affine->scale($scale, $scale);
    $affine->rotate(- rad2deg(atan2($y_hi,$x_hi)));

    my $x_min = 0;
    my $y_min = 0;
    my $x_max = 0;
    my $y_max = 0;
    foreach my $n ($n_lo .. $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      ($x,$y) = $affine->transform($x,$y);
      $x_min = min($x_min, $x);
      $x_max = max($x_max, $x);
      $y_min = min($y_min, $y);
      $y_max = max($y_max, $y);
    }
    print "k=$k  $x_min $x_max   $y_min $y_max\n";
  }
  exit 0;
}

#------------------------------------------------------------------------------
# functions

sub path_xyxy_is_traversed_either {
  my ($path, $x1,$y1, $x2,$y2, $n_limit) = @_;
  my $n = $path->xyxy_to_n_either($x1,$y1, $x2,$y2);
  return defined $n && $n < $n_limit;
}
